/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.awt.*;
import java.awt.color.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*; //ChangeListener
import javax.swing.table.*; //JTable
import javax.swing.border.TitledBorder; //border of Panel
import java.util.*; //LinkedList, HashTable

public class AnatoliaMobileDialog extends JDialog implements ActionListener
{
   private AnatoliaMobile anatoliaMobile = null;
   JTabbedPane tabbedPane = new JTabbedPane();
   DescriptionPanel descriptionPanel = new DescriptionPanel();
   ImmResVulnPanel immResVulnPanel = new ImmResVulnPanel();
   ActAffOffPanel actAffOffPanel = new ActAffOffPanel();
   ValuesPanel valuesPanel = new ValuesPanel();
   AppearancePanel appearancePanel = null; //later
   ShopKeeperPanel shopKeeperPanel = null;
   MobileSpecialsPanel mobileSpecialsPanel = new MobileSpecialsPanel();

   private JButton okButton = new JButton("OK");
   private JButton cancelButton = new JButton("Cancel");
   private boolean lastActionOK = false;
   public boolean getLastActionOK() {return lastActionOK;}

   public AnatoliaMobileDialog(Dialog owner, String title, boolean modal)
   {
      super(owner, title, modal);

      appearancePanel = new AppearancePanel(this);
      shopKeeperPanel = new ShopKeeperPanel(this);

      Container contentPane = getContentPane();
      contentPane.setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      constraint.gridx=0;constraint.gridy=0;
      constraint.gridwidth=2;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=95;
      contentPane.add(tabbedPane, constraint);
      tabbedPane.addTab("Description", descriptionPanel);
      tabbedPane.addTab("Appearance",appearancePanel);
      tabbedPane.addTab("Values",valuesPanel);
      tabbedPane.addTab("Imm/Res/Vuln",immResVulnPanel);
      tabbedPane.addTab("Act/Aff/Off",actAffOffPanel);
      tabbedPane.addTab("ShopKeeper", shopKeeperPanel);
      tabbedPane.addTab("Specials", mobileSpecialsPanel);

      constraint.gridx=0;constraint.gridy=1;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=50;constraint.weighty=5;
      contentPane.add(okButton, constraint);

      constraint.gridx=1;constraint.gridy=1;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=50;constraint.weighty=5;
      contentPane.add(cancelButton, constraint);

      this.pack();

      okButton.addActionListener(this);
      cancelButton.addActionListener(this);
   }

   public void actionPerformed(ActionEvent e)
   {
      if (e.getSource() == cancelButton)
      {
         lastActionOK = false;
         setVisible(false);
      }
      else if (e.getSource() == okButton)
      {
         lastActionOK = true;
         setVisible(!applyProperties());
      }
   }

   public void showFor(AnatoliaMobile anatoliaMobile, StringLocation stringLocation)
   {
     showFor(anatoliaMobile);
     this.tabbedPane.setSelectedIndex(0);
   }

   public void showFor(AnatoliaMobile anatoliaMobile)
   {
      if (anatoliaMobile == null) return;
      this.anatoliaMobile = anatoliaMobile;

      if(anatoliaMobile.isExtern()) this.tabbedPane.setSelectedIndex(0);
      for(int i=1 ; i<7 ; ++i)
      {
        this.tabbedPane.setEnabledAt(i, !anatoliaMobile.isExtern());
      }

      //Description
      descriptionPanel.nameList.setText(anatoliaMobile.nameList);
      descriptionPanel.shortDescription.setText(anatoliaMobile.shortDescription);
      descriptionPanel.longDescription.setText(anatoliaMobile.longDescription);
      descriptionPanel.longDescription.setText(anatoliaMobile.lookDescription);

      //Appareance
      appearancePanel.raceBox.setSelectedIndex(anatoliaMobile.race);
      appearancePanel.startPositionBox.setSelectedIndex(anatoliaMobile.startPosition);
      appearancePanel.defaultPositionBox.setSelectedIndex(anatoliaMobile.defaultPosition);
      appearancePanel.sizeBox.setSelectedIndex(anatoliaMobile.size);
      appearancePanel.sexBox.setSelectedIndex(anatoliaMobile.sex);
      appearancePanel.materialTextField.setText(anatoliaMobile.material);
      AreaBuilderConsts.stringToSelection(anatoliaMobile.corpseFlags,
                                          AnatoliaConsts.mobileCorpseFlagTexts,
                                          appearancePanel.corpseCheckBoxes);
      AreaBuilderConsts.stringToSelection(anatoliaMobile.formFlags,
                                          AnatoliaConsts.mobileFormFlagTexts,
                                          appearancePanel.formCheckBoxes);
      AreaBuilderConsts.stringToSelection(anatoliaMobile.partsFlags,
                                          AnatoliaConsts.mobilePartsFlagTexts,
                                          appearancePanel.partsCheckBoxes);
      appearancePanel.setValuesAccordingToRace();

      //Values
      valuesPanel.alignmentSlider.setValue(anatoliaMobile.alignment);
      valuesPanel.mobileGroupTextField.setText(Integer.toString(anatoliaMobile.mobileGroup));
      valuesPanel.levelBox.setSelectedIndex(anatoliaMobile.level-AnatoliaConsts.LEVEL_MIN);
      valuesPanel.hitBonusTextField.setText(Integer.toString(anatoliaMobile.hitBonus));
      valuesPanel.hitDice.setDiceRoll(anatoliaMobile.hitDice);
      valuesPanel.manaDice.setDiceRoll(anatoliaMobile.manaDice);
      valuesPanel.damageDice.setDiceRoll(anatoliaMobile.manaDice);
      valuesPanel.pierceACTextField.setText(Integer.toString(anatoliaMobile.pierceAC));
      valuesPanel.bashACTextField.setText(Integer.toString(anatoliaMobile.bashAC));
      valuesPanel.slashACTextField.setText(Integer.toString(anatoliaMobile.slashAC));
      valuesPanel.magicACTextField.setText(Integer.toString(anatoliaMobile.magicAC));
      valuesPanel.damageTypeBox.setSelectedIndex(anatoliaMobile.damageType);
      valuesPanel.damageBox.setSelectedIndex(anatoliaMobile.damage);
      valuesPanel.treasureTextField.setText(Integer.toString(anatoliaMobile.treasure));
      valuesPanel.setValuesAccordingToLevel();
      mobileSpecialsPanel.specialTextField.setText(anatoliaMobile.special);

      //Imm/Res/Vuln
      AreaBuilderConsts.stringToSelection(anatoliaMobile.immFlags,
                                          AnatoliaConsts.immResVulnFlagTexts,
                                          immResVulnPanel.immCheckBoxes);
      AreaBuilderConsts.stringToSelection(anatoliaMobile.resFlags,
                                          AnatoliaConsts.immResVulnFlagTexts,
                                          immResVulnPanel.resCheckBoxes);
      AreaBuilderConsts.stringToSelection(anatoliaMobile.vulnFlags,
                                          AnatoliaConsts.immResVulnFlagTexts,
                                          immResVulnPanel.vulnCheckBoxes);


      //Act/Aff/Off
      AreaBuilderConsts.stringToSelection(anatoliaMobile.actFlags,
                                          AnatoliaConsts.mobileActFlagTexts,
                                          actAffOffPanel.actCheckBoxes);
      AreaBuilderConsts.stringToSelection(anatoliaMobile.affFlags,
                                          AnatoliaConsts.mobileAffFlagTexts,
                                          actAffOffPanel.affCheckBoxes);
      AreaBuilderConsts.stringToSelection(anatoliaMobile.offFlags,
                                          AnatoliaConsts.mobileOffFlagTexts,
                                          actAffOffPanel.offCheckBoxes);


      //shopkeeper
      shopKeeperPanel.isShopKeeperCheckBox.setSelected(anatoliaMobile.isShopKeeper);
      shopKeeperPanel.nbItemSoldBox.setSelectedIndex(anatoliaMobile.nbItemSold);
      for(int i=0 ; i<shopKeeperPanel.itemSoldBoxes.length ; ++i)
      {
        shopKeeperPanel.itemSoldBoxes[i].setSelectedIndex(anatoliaMobile.itemSold[i]);
      }
      shopKeeperPanel.profitBuySlider.setValue(anatoliaMobile.profitBuy);
      shopKeeperPanel.profitSellSlider.setValue(Math.max(1,anatoliaMobile.profitSell));
      shopKeeperPanel.openHourBox.setSelectedIndex(anatoliaMobile.openHour);
      shopKeeperPanel.closeHourBox.setSelectedIndex(anatoliaMobile.closeHour);

      //special prog
      for(int i=0 ; i<AnatoliaConsts.mobilePracticerTexts.length ; ++i)
      {
        mobileSpecialsPanel.practicerCheckBoxes[i].setSelected(anatoliaMobile.practicer[i]);
      }
      for(int i=0 ; i<AnatoliaConsts.mobileProgTexts.length ; ++i)
      {
        mobileSpecialsPanel.progStrings[i] = new String(anatoliaMobile.specialProgs[i]);
      }
      mobileSpecialsPanel.specialTextField.setText(anatoliaMobile.special);
      mobileSpecialsPanel.updateProgStrings(true);

      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      Dimension mySize = this.getSize();
      this.setLocation((int) (screenSize.getWidth()-mySize.getWidth())/2,
                       (int) (screenSize.getHeight()-mySize.getHeight())/2
                      );
      pack();
      setVisible(true);
   }

   private boolean applyProperties()
   {
      if (anatoliaMobile == null) return true;

      //description
      anatoliaMobile.nameList = descriptionPanel.nameList.getText();
      anatoliaMobile.shortDescription = descriptionPanel.shortDescription.getText();
      anatoliaMobile.longDescription = descriptionPanel.longDescription.getText();
      anatoliaMobile.lookDescription = descriptionPanel.lookDescription.getText();

      //appearance
      anatoliaMobile.race = appearancePanel.raceBox.getSelectedIndex();
      anatoliaMobile.startPosition = (byte) appearancePanel.startPositionBox.getSelectedIndex();
      anatoliaMobile.defaultPosition = (byte) appearancePanel.startPositionBox.getSelectedIndex();
      anatoliaMobile.size = (byte) appearancePanel.sizeBox.getSelectedIndex();
      anatoliaMobile.sex = (byte) appearancePanel.sexBox.getSelectedIndex();
      anatoliaMobile.material = appearancePanel.materialTextField.getText();
      anatoliaMobile.corpseFlags = AreaBuilderConsts.selectionToString(AnatoliaConsts.mobileCorpseFlagTexts,appearancePanel.corpseCheckBoxes);
      anatoliaMobile.formFlags = AreaBuilderConsts.selectionToString(AnatoliaConsts.mobileFormFlagTexts,appearancePanel.formCheckBoxes);
      anatoliaMobile.partsFlags = AreaBuilderConsts.selectionToString(AnatoliaConsts.mobilePartsFlagTexts,appearancePanel.partsCheckBoxes);

      //values
      anatoliaMobile.alignment = valuesPanel.alignmentSlider.getValue();
      anatoliaMobile.mobileGroup = MyInteger.parseInt(valuesPanel.mobileGroupTextField.getText());
      anatoliaMobile.level = valuesPanel.levelBox.getSelectedIndex()+AnatoliaConsts.LEVEL_MIN;
      anatoliaMobile.hitBonus = MyInteger.parseInt(valuesPanel.hitBonusTextField.getText());
      anatoliaMobile.hitDice    = (DiceRoll) valuesPanel.hitDice.clone();
      anatoliaMobile.manaDice = (DiceRoll) valuesPanel.manaDice.clone();
      anatoliaMobile.damageDice = (DiceRoll) valuesPanel.damageDice.clone();
      anatoliaMobile.pierceAC = MyInteger.parseInt(valuesPanel.pierceACTextField.getText());
      anatoliaMobile.bashAC   = MyInteger.parseInt(valuesPanel.bashACTextField.getText());
      anatoliaMobile.slashAC  = MyInteger.parseInt(valuesPanel.slashACTextField.getText());
      anatoliaMobile.magicAC  = MyInteger.parseInt(valuesPanel.magicACTextField.getText());
      anatoliaMobile.damageType = (byte) valuesPanel.damageTypeBox.getSelectedIndex();
      anatoliaMobile.damage = (byte) valuesPanel.damageBox.getSelectedIndex();
      anatoliaMobile.treasure = MyInteger.parseInt(valuesPanel.treasureTextField.getText());
      anatoliaMobile.special = mobileSpecialsPanel.specialTextField.getText();

      //Imm/Res/Vuln
      anatoliaMobile.immFlags  = AreaBuilderConsts.selectionToString(AnatoliaConsts.immResVulnFlagTexts,immResVulnPanel.immCheckBoxes);
      anatoliaMobile.resFlags  = AreaBuilderConsts.selectionToString(AnatoliaConsts.immResVulnFlagTexts,immResVulnPanel.resCheckBoxes);
      anatoliaMobile.vulnFlags = AreaBuilderConsts.selectionToString(AnatoliaConsts.immResVulnFlagTexts,immResVulnPanel.vulnCheckBoxes);

      //Act/Aff/Off
      anatoliaMobile.actFlags = AreaBuilderConsts.selectionToString(AnatoliaConsts.mobileActFlagTexts,actAffOffPanel.actCheckBoxes);
      anatoliaMobile.affFlags = AreaBuilderConsts.selectionToString(AnatoliaConsts.mobileAffFlagTexts,actAffOffPanel.affCheckBoxes);
      anatoliaMobile.offFlags = AreaBuilderConsts.selectionToString(AnatoliaConsts.mobileOffFlagTexts,actAffOffPanel.offCheckBoxes);

      //shopkeeper
      anatoliaMobile.isShopKeeper = shopKeeperPanel.isShopKeeperCheckBox.isSelected();
      anatoliaMobile.nbItemSold = shopKeeperPanel.nbItemSoldBox.getSelectedIndex();
      for(int i=0 ; i<shopKeeperPanel.itemSoldBoxes.length ; ++i)
      {
        anatoliaMobile.itemSold[i] = shopKeeperPanel.itemSoldBoxes[i].getSelectedIndex();
      }
      anatoliaMobile.profitBuy = shopKeeperPanel.profitBuySlider.getValue();
      anatoliaMobile.profitSell = Math.max(1,shopKeeperPanel.profitSellSlider.getValue());
      anatoliaMobile.openHour = (byte) shopKeeperPanel.openHourBox.getSelectedIndex();
      anatoliaMobile.closeHour = (byte) shopKeeperPanel.closeHourBox.getSelectedIndex();

      //special prog
      mobileSpecialsPanel.updateProgStrings(false);
      for(int i=0 ; i<AnatoliaConsts.mobilePracticerTexts.length ; ++i)
      {
        anatoliaMobile.practicer[i] = mobileSpecialsPanel.practicerCheckBoxes[i].isSelected();
      }
      for(int i=0 ; i<AnatoliaConsts.mobileProgTexts.length ; ++i)
      {
        anatoliaMobile.specialProgs[i] = new String(mobileSpecialsPanel.progStrings[i]);
      }
      anatoliaMobile.special = mobileSpecialsPanel.specialTextField.getText();

      return true;
   }
}

class ValuesPanel extends JPanel implements ChangeListener, ActionListener
{
   JSlider alignmentSlider = new JSlider(JSlider.HORIZONTAL, -1000,1000,0);
   private JTextField alignmentTextField = new JTextField("0",5);
   JTextField mobileGroupTextField = new JTextField("",3);
   JComboBox levelBox = new JComboBox(AnatoliaConsts.getMobileLevelsTexts());
   JTextField hitBonusTextField = new JTextField("0", 3);
   DiceRoll hitDice  = new DiceRoll();
   DiceRoll manaDice = new DiceRoll();
   DiceRoll damageDice = new DiceRoll();
   JTextField pierceACTextField = new JTextField(4);
   JTextField bashACTextField   = new JTextField(4);
   JTextField slashACTextField  = new JTextField(4);
   JTextField magicACTextField  = new JTextField(4);
   private JComboBox damagePierceBox = new JComboBox(AnatoliaConsts.damageTypePierceTexts);
   private JComboBox damageBashBox   = new JComboBox(AnatoliaConsts.damageTypeBashTexts);
   private JComboBox damageSlashBox  = new JComboBox(AnatoliaConsts.damageTypeSlashTexts);
   private JComboBox damageMagicBox  = new JComboBox(AnatoliaConsts.damageTypeMagicTexts);
   private JComboBox[] damageBoxes = {damagePierceBox, damageBashBox, damageSlashBox, damageMagicBox};
   JComboBox damageTypeBox = new JComboBox(AnatoliaConsts.damageTypeTexts);
   JComboBox damageBox = new JComboBox(AnatoliaConsts.damageTexts[damageTypeBox.getSelectedIndex()]);
   JTextField treasureTextField = new JTextField(4);

   public ValuesPanel()
   {
      setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      JPanel levelPanel = new JPanel(new GridLayout(2,1));
      levelPanel.add(new JLabel("Level"));
      levelPanel.add(levelBox);
      levelBox.addActionListener(this);
      constraint.gridx=0;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=20;constraint.weighty=20;
      add(levelPanel, constraint);

      JPanel hitBonusPanel = new JPanel(new GridLayout(2,1));
      hitBonusPanel.add(new JLabel("Hit bonus"));
      hitBonusPanel.add(hitBonusTextField);
      hitBonusTextField.setToolTipText("Should be 0 unless for very powerful characters (50 for IVAN, 30 for SENESHAL, 70 for POWERMAN...)");
      constraint.gridx=1;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=20;constraint.weighty=20;
      add(hitBonusPanel, constraint);

      JPanel hitDicePanel = new JPanel(new GridLayout(2,1));
      hitDicePanel.add(new JLabel("Hit dice"));
      hitDicePanel.add(hitDice);
      constraint.gridx=2;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=20;constraint.weighty=20;
      add(hitDicePanel, constraint);

      JPanel manaDicePanel = new JPanel(new GridLayout(2,1));
      manaDicePanel.add(new JLabel("Mana dice"));
      manaDicePanel.add(manaDice);
      constraint.gridx=3;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=20;constraint.weighty=20;
      add(manaDicePanel, constraint);

      JPanel damageDicePanel = new JPanel(new GridLayout(2,1));
      damageDicePanel.add(new JLabel("Dam dice"));
      damageDicePanel.add(damageDice);
      constraint.gridx=4;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=20;constraint.weighty=20;
      add(damageDicePanel, constraint);

      JPanel alignmentPanel = new JPanel(new GridBagLayout());
        constraint.gridx=0;constraint.gridy=0;
        constraint.gridwidth=1;constraint.gridheight=1;
        constraint.weightx=60;constraint.weighty=50;
        alignmentPanel.add(new JLabel("Alignment"), constraint);

        constraint.gridx=1;constraint.gridy=0;
        constraint.gridwidth=1;constraint.gridheight=1;
        constraint.weightx=40;constraint.weighty=50;
        alignmentTextField.setEnabled(false);
        alignmentPanel.add(alignmentTextField, constraint);

        constraint.gridx=0;constraint.gridy=1;
        constraint.gridwidth=2;constraint.gridheight=1;
        constraint.weightx=100;constraint.weighty=50;
        Hashtable labels = new Hashtable();
        labels.put(new Integer(-1000), new JLabel("Evil"));
        labels.put(new Integer(0), new JLabel("Neutral"));
        labels.put(new Integer(1000), new JLabel("Good"));
        alignmentSlider.setLabelTable(labels);
        alignmentSlider.setPaintLabels(true);
        alignmentSlider.setMajorTickSpacing(1000);
        alignmentSlider.setMinorTickSpacing(50);
        alignmentSlider.setPaintTicks(true);
        alignmentSlider.setSnapToTicks(true);
        alignmentSlider.addChangeListener(this);
        alignmentPanel.add(alignmentSlider, constraint);
      constraint.gridx=0;constraint.gridy=1;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=20;constraint.weighty=20;
      add(alignmentPanel, constraint);

      JPanel armorPiercePanel = new JPanel(new GridLayout(2,1));
      armorPiercePanel.add(new JLabel ("Pierce AC"));
      armorPiercePanel.add(pierceACTextField);
      constraint.gridx=1;constraint.gridy=1;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=20;constraint.weighty=20;
      add(armorPiercePanel, constraint);

      JPanel armorBashPanel = new JPanel(new GridLayout(2,1));
      armorBashPanel.add(new JLabel ("Bash AC"));
      armorBashPanel.add(bashACTextField);
      constraint.gridx=2;constraint.gridy=1;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=20;constraint.weighty=20;
      add(armorBashPanel, constraint);

      JPanel armorSlashPanel = new JPanel(new GridLayout(2,1));
      armorSlashPanel.add(new JLabel ("Slash AC"));
      armorSlashPanel.add(slashACTextField);
      constraint.gridx=3;constraint.gridy=1;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=20;constraint.weighty=20;
      add(armorSlashPanel, constraint);

      JPanel armorMagicPanel = new JPanel(new GridLayout(2,1));
      armorMagicPanel.add(new JLabel ("Magic AC"));
      armorMagicPanel.add(magicACTextField);
      constraint.gridx=4;constraint.gridy=1;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=20;constraint.weighty=20;
      add(armorMagicPanel, constraint);

      JPanel damageTypePanel = new JPanel(new GridLayout(2,1));
      damageTypePanel.add(new JLabel("Damage Type"));
      damageTypeBox.addActionListener(this);
      damageTypePanel.add(damageTypeBox);
      constraint.gridx=0;constraint.gridy=2;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=20;constraint.weighty=20;
      add(damageTypePanel, constraint);

      JPanel damagePanel = new JPanel(new GridLayout(2,1));
      damagePanel.add(new JLabel("Damage"));
      damagePanel.add(damageBox);
      constraint.gridx=1;constraint.gridy=2;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=20;constraint.weighty=20;
      add(damagePanel, constraint);

      JPanel treasurePanel = new JPanel(new GridLayout(2,1));
      treasurePanel.add(new JLabel("Treasure"));
      treasureTextField.setToolTipText("Average treasure carried by the mobile; non-intelligent mobile should have no treasure.");
      treasurePanel.add(treasureTextField);
      constraint.gridx=2;constraint.gridy=2;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=20;constraint.weighty=20;
      add(treasurePanel, constraint);

      JPanel mobileGroupPanel = new JPanel(new GridLayout(2,1));
      mobileGroupPanel.add(new JLabel("Mobile group"));
      mobileGroupTextField.setToolTipText("Number<100. Any mobile in the same group as another mobile will assist it in combat.");
      mobileGroupPanel.add(mobileGroupTextField);
      constraint.gridx=3;constraint.gridy=2;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=20;constraint.weighty=20;
      add(mobileGroupPanel, constraint);
   }

   public void stateChanged(ChangeEvent e)
   {
      if (e.getSource() == alignmentSlider)
      {
         alignmentTextField.setText(Integer.toString(alignmentSlider.getValue()));
      }
   }

   public void actionPerformed(ActionEvent e)
   {
      if (e.getSource() == damageTypeBox)
      {
         damageBox.setModel(damageBoxes[damageTypeBox.getSelectedIndex()].getModel());
      }
      else if (e.getSource() == levelBox)
      {
         setValuesAccordingToLevel();
      }
   }

   public void setValuesAccordingToLevel()
   {
         int level = Math.min(levelBox.getSelectedIndex()+AnatoliaConsts.LEVEL_MIN, AnatoliaConsts.mobileLevelRecommendedValues.length-1);

         MobileLevelValues recommendedValues = AnatoliaConsts.mobileLevelRecommendedValues[level];
         hitBonusTextField.setText(Integer.toString(recommendedValues.getHitBonus()));
         hitDice.setAll(recommendedValues.getXHitPoints(),
                        recommendedValues.getYHitPoints(),
                        recommendedValues.getZHitPoints());
         manaDice.setAll(recommendedValues.getXManPoints(),
                           recommendedValues.getYManPoints(),
                           recommendedValues.getZManPoints());
         damageDice.setAll(recommendedValues.getXDamPoints(),
                           recommendedValues.getYDamPoints(),
                           recommendedValues.getZDamPoints());
         pierceACTextField.setText(Integer.toString(recommendedValues.getPierceAC()));
         bashACTextField.setText(Integer.toString(recommendedValues.getBashAC()));
         slashACTextField.setText(Integer.toString(recommendedValues.getSlashAC()));
         magicACTextField.setText(Integer.toString(recommendedValues.getMagicAC()));
         treasureTextField.setText(Integer.toString(level*100));
   }
}

class DescriptionPanel extends JPanel
{
   JTextField nameList = new JTextField(20);
   JTextField shortDescription = new JTextField(20);
   JTextField longDescription = new JTextField(40);
   JTextArea  lookDescription = new JTextArea(5,40);

   public DescriptionPanel()
   {
      setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      JPanel namePanel = new JPanel(new GridLayout(2,1));
      namePanel.add(new JLabel("Name list"));
      namePanel.add(nameList);
      nameList.setToolTipText("A list of names that the mobile can be referenced by (i.e. \"fat worm\" for un ugly fat worm");
      constraint.gridx=0;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=20;
      add(namePanel, constraint);

      JPanel shortDescriptionPanel = new JPanel(new GridLayout(2,1));
      shortDescriptionPanel.add(new JLabel("Short Description"));
      shortDescription.setToolTipText("Seen when the mobile fights or performs actions (i.e. \"The large orc\")");
      shortDescriptionPanel.add(shortDescription);
      constraint.gridx=0;constraint.gridy=1;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=20;
      add(shortDescriptionPanel, constraint);

      JPanel longDescriptionPanel = new JPanel(new GridLayout(2,1));
      longDescriptionPanel.add(new JLabel("Long Description"));
      longDescription.setToolTipText("Seen whenever a character looks at the room the mobile is in");
      longDescriptionPanel.add(longDescription);
      constraint.gridx=0;constraint.gridy=2;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=20;
      add(longDescriptionPanel, constraint);

      JPanel lookDescriptionPanel = new JPanel(new BorderLayout());
      lookDescriptionPanel.add(new JLabel("Look Description"),BorderLayout.NORTH);
      lookDescription.setToolTipText("Seen when you look at the mobile");
      JScrollPane scrollPane = new JScrollPane(lookDescription);
      scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
      scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
      lookDescriptionPanel.add(scrollPane, BorderLayout.CENTER);
      constraint.gridx=0;constraint.gridy=3;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=40;
      add(lookDescriptionPanel, constraint);
   }
}

class ActAffOffPanel extends JPanel
{
   JPanel actPanel = new JPanel(new GridLayout(9,3,0,0));
   JPanel affPanel = new JPanel(new GridLayout(7,3,0,0));
   JPanel offPanel = new JPanel(new GridLayout(8,2,0,0));

   JCheckBox[] actCheckBoxes = new JCheckBox[AnatoliaConsts.mobileActFlagTexts.length];
   JCheckBox[] affCheckBoxes = new JCheckBox[AnatoliaConsts.mobileAffFlagTexts.length];
   JCheckBox[] offCheckBoxes = new JCheckBox[AnatoliaConsts.mobileOffFlagTexts.length];

   public ActAffOffPanel()
   {
      for(int i=0 ; i<actCheckBoxes.length ; ++i)
      {
         actCheckBoxes[i] = new JCheckBox(AnatoliaConsts.mobileActFlagTexts[i].getName());
         actCheckBoxes[i].setToolTipText(AnatoliaConsts.mobileActFlagTexts[i].getToolTipText());
         actPanel.add(actCheckBoxes[i]);
         if (i == AnatoliaConsts.MOBILE_ACT_FLAG_IS_NPC)
         {
           actCheckBoxes[i].setEnabled(false);
         }
      }
      for(int i=0 ; i<affCheckBoxes.length ; ++i)
      {
         affCheckBoxes[i] = new JCheckBox(AnatoliaConsts.mobileAffFlagTexts[i].getName());
         affCheckBoxes[i].setToolTipText(AnatoliaConsts.mobileAffFlagTexts[i].getToolTipText());
         affPanel.add(affCheckBoxes[i]);
      }
      for(int i=0 ; i<offCheckBoxes.length ; ++i)
      {
         offCheckBoxes[i] = new JCheckBox(AnatoliaConsts.mobileOffFlagTexts[i].getName());
         offCheckBoxes[i].setToolTipText(AnatoliaConsts.mobileOffFlagTexts[i].getToolTipText());
         offPanel.add(offCheckBoxes[i]);
      }

      setLayout(new GridLayout(1,3));
      actPanel.setBorder(new TitledBorder("Act flags"));
      affPanel.setBorder(new TitledBorder("Aff flags"));
      offPanel.setBorder(new TitledBorder("Off flags"));
      add(actPanel);
      add(affPanel);
      add(offPanel);
   }
}

class ImmResVulnPanel extends JPanel
{
   JCheckBox[] immCheckBoxes = null;
   JCheckBox[] resCheckBoxes = null;
   JCheckBox[] vulnCheckBoxes = null;
   ButtonGroupMaybeNone[] buttonGroups = null;

   public ImmResVulnPanel()
   {
      setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      int nbLines = AnatoliaConsts.immResVulnFlagTexts.length;
      immCheckBoxes  = new JCheckBox[nbLines];
      for(int i=0 ; i<nbLines ; immCheckBoxes[i++] = new JCheckBox());
      resCheckBoxes  = new JCheckBox[nbLines];
      for(int i=0 ; i<nbLines ; resCheckBoxes[i++] = new JCheckBox());
      vulnCheckBoxes = new JCheckBox[nbLines];
      for(int i=0 ; i<nbLines ; vulnCheckBoxes[i++] = new JCheckBox());
      buttonGroups = new ButtonGroupMaybeNone[nbLines];
      for(int i=0 ; i<nbLines ; buttonGroups[i++] = new ButtonGroupMaybeNone());

      constraint.gridx=0;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=20;constraint.weighty=100/nbLines;
      add(new JLabel("        "), constraint);
      constraint.gridx=1;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=100/nbLines;
      add(new JLabel(" Immune "), constraint);
      constraint.gridx=2;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=100/nbLines;
      add(new JLabel(" Resistant "), constraint);
      constraint.gridx=3;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=100/nbLines;
      add(new JLabel(" Vulnerable"), constraint);

      for(int i=0 ; i<nbLines ; ++i)
      {
         constraint.gridx=0;constraint.gridy=i+1;
         constraint.gridwidth=1;constraint.gridheight=1;
         constraint.weightx=25;constraint.weighty=100/nbLines;
         add(new JLabel(AnatoliaConsts.immResVulnFlagTexts[i].getName()), constraint);
         constraint.gridx=1;constraint.gridy=i+1;
         constraint.gridwidth=1;constraint.gridheight=1;
         constraint.weightx=25;constraint.weighty=100/nbLines;
         add(immCheckBoxes[i], constraint);
         constraint.gridx=2;constraint.gridy=i+1;
         constraint.gridwidth=1;constraint.gridheight=1;
         constraint.weightx=25;constraint.weighty=100/nbLines;
         add(resCheckBoxes[i], constraint);
         constraint.gridx=3;constraint.gridy=i+1;
         constraint.gridwidth=1;constraint.gridheight=1;
         constraint.weightx=25;constraint.weighty=100/nbLines;
         add(vulnCheckBoxes[i], constraint);
         buttonGroups[i].add(immCheckBoxes[i]);
         buttonGroups[i].add(resCheckBoxes[i]);
         buttonGroups[i].add(vulnCheckBoxes[i]);
      }
  }
}

class AppearancePanel extends JPanel implements ActionListener
{
   AnatoliaMobileDialog rootDialog = null;
   JComboBox raceBox = new JComboBox(AnatoliaConsts.getRaceTexts());
   JComboBox startPositionBox = new JComboBox(AnatoliaConsts.positionTexts);
   JComboBox defaultPositionBox = new JComboBox(AnatoliaConsts.positionTexts);
   JComboBox sexBox = new JComboBox(AnatoliaConsts.sexTexts);
   JComboBox sizeBox = new JComboBox(AnatoliaConsts.getMobileSizeTexts());
   JTextField materialTextField = new JTextField(10);

   JCheckBox[] corpseCheckBoxes = new JCheckBox[AnatoliaConsts.mobileCorpseFlagTexts.length];
   JCheckBox[] formCheckBoxes   = new JCheckBox[AnatoliaConsts.mobileFormFlagTexts.length];
   JCheckBox[] partsCheckBoxes  = new JCheckBox[AnatoliaConsts.mobilePartsFlagTexts.length];

   private ButtonGroupMaybeNone corpseButtonGroup = new ButtonGroupMaybeNone();

   public AppearancePanel(AnatoliaMobileDialog rootDialog)
   {
      this.rootDialog = rootDialog;
      setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      JPanel racePanel = new JPanel(new GridLayout(2,1));
      racePanel.add(new JLabel("Race"));
      raceBox.setSelectedIndex(AnatoliaConsts.RACE_TYPE_HUMAN);
      racePanel.add(raceBox);
      raceBox.addActionListener(this);
      constraint.gridx=0;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=33;constraint.weighty=20;
      add(racePanel, constraint);

      JPanel sizePanel = new JPanel(new GridLayout(2,1));
      sizePanel.add(new JLabel("Size"));
      sizePanel.add(sizeBox);
      constraint.gridx=1;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=33;constraint.weighty=10;
      add(sizePanel, constraint);

      JPanel materialPanel = new JPanel(new GridLayout(2,1));
      materialPanel.add(new JLabel("Material"));
      materialTextField.setToolTipText("The material of the mobile");
      materialPanel.add(materialTextField);
      constraint.gridx=2;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=33;constraint.weighty=10;
      add(materialPanel, constraint);

      JPanel startPositionPanel = new JPanel(new GridLayout(2,1));
      startPositionPanel.add(new JLabel("Start pos."));
      startPositionBox.setToolTipText("Position the mobile will be loaded in");
      startPositionPanel.add(startPositionBox);
      constraint.gridx=0;constraint.gridy=1;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=33;constraint.weighty=10;
      add(startPositionPanel, constraint);

      JPanel defaultPositionPanel = new JPanel(new GridLayout(2,1));
      defaultPositionPanel.add(new JLabel("default pos."));
      defaultPositionBox.setToolTipText("Position the mobile will return after fight. Cannot go back to sleep.");
      defaultPositionPanel.add(defaultPositionBox);
      constraint.gridx=1;constraint.gridy=1;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=33;constraint.weighty=10;
      add(defaultPositionPanel, constraint);

      JPanel sexPanel = new JPanel(new GridLayout(2,1));
      sexPanel.add(new JLabel("Sex"));
      sexBox.setToolTipText("\"Either\" will set randomly for each mobile of that type which is loaded");
      sexPanel.add(sexBox);
      constraint.gridx=2;constraint.gridy=1;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=33;constraint.weighty=10;
      add(sexPanel, constraint);

      JPanel corpsePanel = new JPanel(new GridLayout(corpseCheckBoxes.length,1));
      corpsePanel.setBorder(new TitledBorder("Corpse flags"));
      for(int i=0 ; i<corpseCheckBoxes.length ; ++i)
      {
         corpseCheckBoxes[i] = new JCheckBox(AnatoliaConsts.mobileCorpseFlagTexts[i].getName());
         corpseCheckBoxes[i].setToolTipText(AnatoliaConsts.mobileCorpseFlagTexts[i].getToolTipText());
         corpseCheckBoxes[i].addActionListener(this);
         corpsePanel.add(corpseCheckBoxes[i]);
      }
      corpseButtonGroup.add(corpseCheckBoxes[AnatoliaConsts.MOBILE_CORPSE_FLAG_POISON]);
      corpseButtonGroup.add(corpseCheckBoxes[AnatoliaConsts.MOBILE_CORPSE_FLAG_MAGICAL]);
      corpseButtonGroup.add(corpseCheckBoxes[AnatoliaConsts.MOBILE_CORPSE_FLAG_OTHER]);
      corpseButtonGroup.add(corpseCheckBoxes[AnatoliaConsts.MOBILE_CORPSE_FLAG_INSTANT_DECAY]);
      corpseCheckBoxes[AnatoliaConsts.MOBILE_CORPSE_FLAG_EDIBLE].setSelected(true);

      constraint.gridx=0;constraint.gridy=2;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=33;constraint.weighty=10;
      add(corpsePanel, constraint);

      JPanel formPanel = new JPanel(new GridLayout(7, 3));
      formPanel.setBorder(new TitledBorder("Form flags"));
      for(int i=0 ; i<formCheckBoxes.length ; ++i)
      {
         formCheckBoxes[i] = new JCheckBox(AnatoliaConsts.mobileFormFlagTexts[i].getName());
         formCheckBoxes[i].setToolTipText(AnatoliaConsts.mobileFormFlagTexts[i].getToolTipText());
         formPanel.add(formCheckBoxes[i]);
      }
      constraint.gridx=1;constraint.gridy=2;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=33;constraint.weighty=80;
      add(formPanel, constraint);

      JPanel partsPanel = new JPanel(new GridLayout(8, 3));
      partsPanel.setBorder(new TitledBorder("Parts flags"));
      for(int i=0 ; i<partsCheckBoxes.length ; ++i)
      {
         partsCheckBoxes[i] = new JCheckBox(AnatoliaConsts.mobilePartsFlagTexts[i].getName());
         partsCheckBoxes[i].setToolTipText(AnatoliaConsts.mobilePartsFlagTexts[i].getToolTipText());
         partsPanel.add(partsCheckBoxes[i]);
      }
      constraint.gridx=2;constraint.gridy=2;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=33;constraint.weighty=80;
      add(partsPanel, constraint);
   }

   public void actionPerformed(ActionEvent e)
   {
      if (e.getSource() == corpseCheckBoxes[AnatoliaConsts.MOBILE_CORPSE_FLAG_EDIBLE])
      {
         JCheckBox source = (JCheckBox) e.getSource();
         corpseCheckBoxes[AnatoliaConsts.MOBILE_CORPSE_FLAG_POISON].setEnabled(source.isSelected());
         corpseCheckBoxes[AnatoliaConsts.MOBILE_CORPSE_FLAG_MAGICAL].setEnabled(source.isSelected());
         corpseCheckBoxes[AnatoliaConsts.MOBILE_CORPSE_FLAG_OTHER].setEnabled(source.isSelected());
         corpseCheckBoxes[AnatoliaConsts.MOBILE_CORPSE_FLAG_INSTANT_DECAY].setEnabled(source.isSelected());
      }
      else if (e.getSource() == raceBox)
      {
         setValuesAccordingToRace();
      }
   }


   public void setValuesAccordingToRace()
   {
      MobileRaceFlagsValues raceFlagsValues = AnatoliaConsts.mobileRecommendedRaceFlagsValues[raceBox.getSelectedIndex()];

      //corpse/form/parts
      AreaBuilderConsts.stringToSelection(raceFlagsValues.getFormFlags(),
                                          AnatoliaConsts.mobileCorpseFlagTexts,
                                          corpseCheckBoxes);
      AreaBuilderConsts.stringToSelection(raceFlagsValues.getFormFlags(),
                                          AnatoliaConsts.mobileFormFlagTexts,
                                          formCheckBoxes);
      AreaBuilderConsts.stringToSelection(raceFlagsValues.getPartsFlags(),
                                          AnatoliaConsts.mobilePartsFlagTexts,
                                          partsCheckBoxes);

      //Imm/Res/Vuln
      AreaBuilderConsts.stringToSelection(raceFlagsValues.getImmFlags(),
                                           AnatoliaConsts.immResVulnFlagTexts,
                                          rootDialog.immResVulnPanel.immCheckBoxes);
      AreaBuilderConsts.stringToSelection(raceFlagsValues.getResFlags(),
                                          AnatoliaConsts.immResVulnFlagTexts,
                                          rootDialog.immResVulnPanel.resCheckBoxes);
      AreaBuilderConsts.stringToSelection(raceFlagsValues.getVulnFlags(),
                                          AnatoliaConsts.immResVulnFlagTexts,
                                          rootDialog.immResVulnPanel.vulnCheckBoxes);

      //Act/Aff/Off
      AreaBuilderConsts.stringToSelection(raceFlagsValues.getActFlags(),
                                          AnatoliaConsts.mobileActFlagTexts,
                                          rootDialog.actAffOffPanel.actCheckBoxes);
      AreaBuilderConsts.stringToSelection(raceFlagsValues.getAffFlags(),
                                          AnatoliaConsts.mobileAffFlagTexts,
                                          rootDialog.actAffOffPanel.affCheckBoxes);
      AreaBuilderConsts.stringToSelection(raceFlagsValues.getOffFlags(),
                                          AnatoliaConsts.mobileOffFlagTexts,
                                          rootDialog.actAffOffPanel.offCheckBoxes);
      }
}

class ShopKeeperPanel extends JPanel implements ActionListener, ChangeListener
{
   JCheckBox isShopKeeperCheckBox = new JCheckBox("Mobile is a shop keeper");
   static final String[] NB = {"0","1","2","3","4","5"};
   JComboBox nbItemSoldBox = new JComboBox(NB);
   JComboBox[] itemSoldBoxes = new JComboBox[5];
   JSlider profitBuySlider = new JSlider(JSlider.HORIZONTAL,100,300,100);
   JSlider profitSellSlider = new JSlider(JSlider.HORIZONTAL,0,100,100);
   JLabel profitBuyLabel = new JLabel("profit-buy rate : "+ profitBuySlider.getValue()+"%");
   JLabel profitSellLabel = new JLabel("profit-sell rate : "+ profitSellSlider.getValue()+"%");
   JComboBox openHourBox = null;
   JComboBox closeHourBox = null;
   AnatoliaMobileDialog rootDialog = null;

   public ShopKeeperPanel(AnatoliaMobileDialog rootDialog)
   {
     this.rootDialog = rootDialog;
     for(int i=0 ; i<itemSoldBoxes.length ; ++i)
     {
        itemSoldBoxes[i] = new JComboBox(AnatoliaConsts.getObjectTypeTexts());
     }

     Integer[] tab = new Integer[24];
     for(int i=0 ; i<tab.length ; ++i)
     {
       tab[i] = new Integer(i);
     }
     openHourBox = new JComboBox(tab);
     closeHourBox = new JComboBox(tab);

     setLayout(new GridBagLayout());
     GridBagConstraints constraint = new GridBagConstraints();

     constraint.gridx=0;constraint.gridy=0;
     constraint.gridwidth=1;constraint.gridheight=1;
     constraint.weightx=100;constraint.weighty=15;
     isShopKeeperCheckBox.addActionListener(this);
     add(isShopKeeperCheckBox, constraint);

     JPanel hoursPanel = new JPanel(new GridLayout(1,2));
       JPanel openHoursPanel = new JPanel(new GridLayout(2,1));
         openHoursPanel.add(new JLabel("Open hour"));
         openHoursPanel.add(openHourBox);
       hoursPanel.add(openHoursPanel);
       JPanel closeHoursPanel = new JPanel(new GridLayout(2,1));
         closeHoursPanel.add(new JLabel("Close hour"));
         closeHoursPanel.add(closeHourBox);
       hoursPanel.add(closeHoursPanel);
     constraint.gridx=0;constraint.gridy=1;
     constraint.gridwidth=1;constraint.gridheight=1;
     constraint.weightx=100;constraint.weighty=25;
     add(hoursPanel, constraint);

     Hashtable labels1 = new Hashtable();
     for(int i=profitBuySlider.getMinimum() ; i<=profitBuySlider.getMaximum() ; i+=50)
     {
        labels1.put(new Integer(i),new JLabel(Integer.toString(i)));
     }
     JPanel profitPanel = new JPanel(new GridLayout(1,2));
        JPanel profitBuyPanel = new JPanel(new GridLayout(2,1));
        profitBuyPanel.add(profitBuyLabel);
        profitBuySlider.setToolTipText("The mobile will sell you an object at x% of its price");
        profitBuySlider.setLabelTable(labels1);
        profitBuySlider.setPaintLabels(true);
        profitBuySlider.setMajorTickSpacing(10);
        profitBuySlider.setMinorTickSpacing(5);
        profitBuySlider.setPaintTicks(true);
        profitBuySlider.setSnapToTicks(true);
        profitBuySlider.addChangeListener(this);
        profitBuyPanel.add(profitBuySlider);
      profitPanel.add(profitBuyPanel);

     Hashtable labels2 = new Hashtable();
        for(int i=profitSellSlider.getMinimum() ; i<=profitSellSlider.getMaximum() ; i+=25)
        {
           labels2.put(new Integer(i),new JLabel(Integer.toString(i)));
        }

        JPanel profitSellPanel = new JPanel(new GridLayout(2,1));
        profitSellPanel.add(profitSellLabel);
        profitSellSlider.setToolTipText("The mobile will buy you an object at x% of its price");
        profitSellSlider.setLabelTable(labels2);
        profitSellSlider.setPaintLabels(true);
        profitSellSlider.setMajorTickSpacing(10);
        profitSellSlider.setMinorTickSpacing(5);
        profitSellSlider.setPaintTicks(true);
        profitSellSlider.setSnapToTicks(true);
        profitSellSlider.addChangeListener(this);
        profitSellPanel.add(profitSellSlider);
      profitPanel.add(profitSellPanel);

     constraint.gridx=0;constraint.gridy=2;
     constraint.gridwidth=1;constraint.gridheight=1;
     constraint.weightx=100;constraint.weighty=25;
     add(profitPanel, constraint);

     JPanel itemSoldPanel = new JPanel(new GridLayout(1,6));

     JPanel nbPanel = new JPanel(new GridLayout(2,1));
     nbPanel.add(new JLabel("Nb item types to buy "));
     nbItemSoldBox.setToolTipText("The mobile can by you from 0 to 4 different type of things");
     nbItemSoldBox.addActionListener(this);
     nbPanel.add(nbItemSoldBox);
     itemSoldPanel.add(nbPanel);
     JPanel tempPanel = null;
     for(int i=0; i<itemSoldBoxes.length ; ++i)
     {
       tempPanel = new JPanel(new GridLayout(2,1));
       tempPanel.add(new JLabel(" item type "+(i+1)+" "));
       tempPanel.add(itemSoldBoxes[i]);
       itemSoldPanel.add(tempPanel);
     }
     constraint.gridx=0;constraint.gridy=3;
     constraint.gridwidth=1;constraint.gridheight=1;
     constraint.weightx=100;constraint.weighty=35;
     add(itemSoldPanel, constraint);

     showItemSold();
     enableShop(isShopKeeperCheckBox.isSelected());
   }

   public void actionPerformed(ActionEvent e)
   {
     if (e.getSource() == isShopKeeperCheckBox)
     {
       enableShop(isShopKeeperCheckBox.isSelected());
       //according to Anatolia documentation, a shopkeeper should act NO_PURGE
       if (isShopKeeperCheckBox.isSelected())
       {
         rootDialog.actAffOffPanel.actCheckBoxes[AnatoliaConsts.MOBILE_ACT_FLAG_NOPURGE].setSelected(true);
       }
     }
     else if (e.getSource() == nbItemSoldBox)
     {
       showItemSold();
     }
   }

   public void enableShop(boolean enabled)
   {
       nbItemSoldBox.setEnabled(enabled);
       for(int i=0 ; i<itemSoldBoxes.length ; ++i)
       {
         itemSoldBoxes[i].setEnabled(enabled);
       }
       profitBuySlider.setEnabled(enabled);
       profitSellSlider.setEnabled(enabled);
       openHourBox.setEnabled(enabled);
       closeHourBox.setEnabled(enabled);
   }

   public void showItemSold()
   {
      for(int i=0 ; i<itemSoldBoxes.length ; ++i)
      {
        itemSoldBoxes[i].setVisible(i<nbItemSoldBox.getSelectedIndex());
      }
   }

   public void stateChanged(ChangeEvent e)
   {
      if (e.getSource() == profitBuySlider)
      {
         profitBuyLabel.setText("profit-buy rate : "+ profitBuySlider.getValue()+"%");
      }
      else if (e.getSource() == profitSellSlider)
      {
         profitSellLabel.setText("profit-sell rate : "+ profitSellSlider.getValue()+"%");
      }
   }
}

class MobileSpecialsPanel extends JPanel implements ActionListener
{
  JCheckBox[] practicerCheckBoxes = new JCheckBox[AnatoliaConsts.mobilePracticerTexts.length];
  JComboBox   progComboBox = new JComboBox(AnatoliaConsts.mobileProgTexts);
  String[]    progStrings = new String[AnatoliaConsts.mobileProgTexts.length];
  JTextField  progTextField = new JTextField(12);
  JTextField  specialTextField = new JTextField(12);

  int oldComboBoxIndex = 0;

  public MobileSpecialsPanel()
  {
      setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      JPanel practicerPanel = new JPanel(new GridLayout(6,5));
      practicerPanel.setBorder(new TitledBorder("Practicer"));
      for(int i=0 ; i<AnatoliaConsts.mobilePracticerTexts.length ; ++i)
      {
         practicerCheckBoxes[i] = new JCheckBox(AnatoliaConsts.mobilePracticerTexts[i]);
         practicerPanel.add(practicerCheckBoxes[i]);
      }
      constraint.gridx=0;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=60;
      add(practicerPanel, constraint);

      JPanel progPanel = new JPanel(new GridLayout(2,2));
      progPanel.add(new JLabel("Prog"));
      progPanel.add(new JLabel("function to call"));
      progComboBox.setToolTipText("In some situations (below), you can specify a function to call");
      progPanel.add(progComboBox);
      progTextField.setToolTipText("The name of a C function already coded in Anatolia");
      progPanel.add(progTextField);
      progComboBox.addActionListener(this);
      for(int i=0 ; i<progStrings.length ; ++i)
      {
        progStrings[i] = "";
      }
      constraint.gridx=0;constraint.gridy=1;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=20;
      add(progPanel, constraint);

      JPanel specialPanel = new JPanel(new GridLayout(2,1));
      specialPanel.add(new JLabel("Special Function"));
      specialTextField.setToolTipText("A particular C function can be associated to the mobile");
      specialPanel.add(specialTextField);
      constraint.gridx=0;constraint.gridy=2;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=20;
      add(specialPanel, constraint);

      oldComboBoxIndex = progComboBox.getSelectedIndex();
  }

  public void actionPerformed(ActionEvent e)
  {
    if (e.getSource() == progComboBox)
    {
      updateProgStrings(false);
    }
  }

  public void updateProgStrings(boolean showingNewMobile)
  {
    if (!showingNewMobile) progStrings[oldComboBoxIndex] = progTextField.getText();
    progTextField.setText(progStrings[progComboBox.getSelectedIndex()]);
    oldComboBoxIndex = progComboBox.getSelectedIndex();
  }
}