/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.awt.*;
import java.awt.color.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*; //JTable
import javax.swing.border.TitledBorder; //border of Panel
import java.util.*; //LinkedList

public class AnatoliaObjectDialog extends JDialog implements ActionListener
{
   AnatoliaObject anatoliaObject = null;
   private AreaMap areaMap = null; //or to this
   private JButton okButton = new JButton("OK");
   private JButton cancelButton = new JButton("Cancel");
   private JTabbedPane tabbedPane = new JTabbedPane();
   private ObjectDescriptionPanel objectDescriptionPanel = new ObjectDescriptionPanel();
   private ObjectTypePanel objectTypePanel;
   private ObjectFlagsPanel objectFlagsPanel = new ObjectFlagsPanel();
   private ObjectImmResVulnPanel immResVulnPanel = new ObjectImmResVulnPanel();
   private AffectsPanel affectsPanel = new AffectsPanel();
   private ObjectSpecialsPanel objectSpecialsPanel = new ObjectSpecialsPanel();
   private boolean lastActionOK = false;

   public AnatoliaObjectDialog(Dialog owner, String title, boolean modal, AreaMap areaMap)
   {
      super(owner, title, modal);
      this.areaMap = areaMap;
      objectTypePanel = new ObjectTypePanel(areaMap);
      Container contentPane = getContentPane();
      contentPane.setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      tabbedPane.addTab("Description", objectDescriptionPanel);
      tabbedPane.addTab("Type", objectTypePanel);
      tabbedPane.addTab("Flags", objectFlagsPanel);
      tabbedPane.addTab("Imm/Res/Vuln", immResVulnPanel);
      tabbedPane.addTab("Affects", affectsPanel);
      tabbedPane.addTab("Specials", objectSpecialsPanel);
      constraint.gridx=0;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=90;
      contentPane.add(tabbedPane, constraint);

      JPanel confirmPanel = new JPanel(new GridLayout(1,2));
      confirmPanel.add(okButton);
      confirmPanel.add(cancelButton);
      constraint.gridx=0;constraint.gridy=1;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=10;
      contentPane.add(confirmPanel, constraint);

      okButton.addActionListener(this);
      cancelButton.addActionListener(this);

      pack();
   }

   public boolean getLastActionOK() {return lastActionOK;}

   public void actionPerformed(ActionEvent e)
   {
      if (e.getSource() == cancelButton)
      {
         lastActionOK = false;
         setVisible(false);
      }
      else if (e.getSource() == okButton)
      {
         lastActionOK = true;
         setVisible(!applyProperties(anatoliaObject));
      }
   }

   public void showFor(AnatoliaObject anatoliaObject, StringLocation stringLocation)
   {
     showFor(anatoliaObject);
     this.tabbedPane.setSelectedIndex(0);
   }

   public void showFor(AnatoliaObject anatoliaObject)
   {
      if (anatoliaObject == null) return;
      this.anatoliaObject = anatoliaObject;

      if(anatoliaObject.isExtern()) this.tabbedPane.setSelectedIndex(0);
      for(int i=1 ; i<6 ; ++i)
      {
        this.tabbedPane.setEnabledAt(i, !anatoliaObject.isExtern());
      }


      //description
      objectDescriptionPanel.nameTextField.setText(anatoliaObject.name);
      objectDescriptionPanel.shortTextField.setText(anatoliaObject.shortDescription);
      objectDescriptionPanel.longTextField.setText(anatoliaObject.longDescription);
      Iterator it = anatoliaObject.extraDescriptions.iterator();
      Pair pair = null;
      int row=0;
      for(int i=0; i<objectDescriptionPanel.table.getRowCount() ; ++i)
      {
         objectDescriptionPanel.table.setValueAt("",i,0);
         objectDescriptionPanel.table.setValueAt("",i,1);
      }
      while(it.hasNext() && row<objectDescriptionPanel.table.getRowCount())
      {
         pair = (Pair) it.next();
         objectDescriptionPanel.table.setValueAt((String) pair.getFirst(),row,0);
         objectDescriptionPanel.table.setValueAt((String) pair.getSecond(),row,1);
         ++row;
      }

      //status
      objectTypePanel.statusBox.setSelectedIndex(
        Math.max(0,AreaBuilderConsts.findInTab(AnatoliaConsts.getObjectConditionCodes(),anatoliaObject.cond))
      );

      //type
      objectTypePanel.levelBox.setSelectedIndex(anatoliaObject.level-AnatoliaConsts.LEVEL_MIN);
      objectTypePanel.materialBox.setSelectedIndex(AreaBuilderConsts.findInTab(AnatoliaConsts.objectMaterialTexts,anatoliaObject.material));
      objectTypePanel.weightTextField.setText(Integer.toString(anatoliaObject.weight));
      objectTypePanel.costTextField.setText(Integer.toString(anatoliaObject.cost));
      objectTypePanel.typeBox.setSelectedIndex(anatoliaObject.type);

      ObjectSpecialType objectSpecialType = objectTypePanel.getObjectSpecialType();

      //build the list of actual rooms
      it = areaMap.theRooms.roomList.iterator();
      objectTypePanel.portalTypePanel.exitRoomBox.removeAllItems();
      while(it.hasNext())
      {
         objectTypePanel.portalTypePanel.exitRoomBox.addItem((Room) it.next());
      }
      objectSpecialType.setV0(anatoliaObject.v0);
      objectSpecialType.setV1(anatoliaObject.v1);
      objectSpecialType.setV2(anatoliaObject.v2);
      objectSpecialType.setV3(anatoliaObject.v3);
      objectSpecialType.setV4(anatoliaObject.v4);

      //extra & wear flags
      AreaBuilderConsts.stringToSelection(anatoliaObject.extraFlags, AnatoliaConsts.extraFlagTexts, objectFlagsPanel.extraFlagsCheckBoxes);
      AreaBuilderConsts.stringToSelection(anatoliaObject.wearFlags, AnatoliaConsts.wearFlagTexts, objectFlagsPanel.wearFlagsRadioButtons);

      //imm/res/vuln
      AreaBuilderConsts.stringToSelection(anatoliaObject.immFlags , AnatoliaConsts.immResVulnFlagTexts, this.immResVulnPanel.immCheckBoxes);
      AreaBuilderConsts.stringToSelection(anatoliaObject.resFlags , AnatoliaConsts.immResVulnFlagTexts, this.immResVulnPanel.resCheckBoxes);
      AreaBuilderConsts.stringToSelection(anatoliaObject.vulnFlags, AnatoliaConsts.immResVulnFlagTexts, this.immResVulnPanel.vulnCheckBoxes);

      //general & local affects
      AreaBuilderConsts.stringToSelection(anatoliaObject.detectFlags, AnatoliaConsts.detTexts, affectsPanel.detCheckBoxes);
      AreaBuilderConsts.stringToSelection(anatoliaObject.generalAffects, AnatoliaConsts.affTexts, affectsPanel.generalAffCheckBoxes);
      for(int i=0 ; i<AnatoliaConsts.applyTexts.length ; ++i)
      {
         affectsPanel.applyPanels[i].modifierTextField.setText(Integer.toString(anatoliaObject.localAffects[i]));
      }

      //special prog
      for(int i=0 ; i<AnatoliaConsts.objectProgTexts.length ; ++i)
      {
        objectSpecialsPanel.progStrings[i] = new String(anatoliaObject.specialProgs[i]);
      }
      objectSpecialsPanel.updateProgStrings(true);

      objectSpecialsPanel.limitedBox.setSelectedIndex(anatoliaObject.limit);


      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      Dimension mySize = this.getSize();
      this.setLocation((int) (screenSize.getWidth()-mySize.getWidth())/2,
                       (int) (screenSize.getHeight()-mySize.getHeight())/2
                      );

      pack();
      setVisible(true);
   }

   private boolean applyProperties(AnatoliaObject anatoliaObject)
   {
      if (anatoliaObject == null) return true;

      //description
      anatoliaObject.name = objectDescriptionPanel.nameTextField.getText();
      anatoliaObject.shortDescription = objectDescriptionPanel.shortTextField.getText();
      anatoliaObject.longDescription = objectDescriptionPanel.longTextField.getText();

      anatoliaObject.extraDescriptions.clear();
      String s1;
      String s2;
      for(int i=0; i<objectDescriptionPanel.table.getRowCount() ; ++i)
      {
         s1 = (String) objectDescriptionPanel.table.getValueAt(i,0);
         s2 = (String) objectDescriptionPanel.table.getValueAt(i,1);
         if ((s1 != null) && (!s1.trim().equals("")) && (s2 != null) &&  (!s2.trim().equals("")))
            anatoliaObject.extraDescriptions.add(new Pair(s1,s2));
      }

      //status
      anatoliaObject.cond = AnatoliaConsts.objectConditionTexts[objectTypePanel.statusBox.getSelectedIndex()].getCode();

      //type
      anatoliaObject.level = (byte) (objectTypePanel.levelBox.getSelectedIndex()+AnatoliaConsts.LEVEL_MIN);
      anatoliaObject.material = objectTypePanel.materialBox.getSelectedItem().toString();
      anatoliaObject.weight = MyInteger.parseInt(objectTypePanel.weightTextField.getText());
      anatoliaObject.cost = MyInteger.parseInt(objectTypePanel.costTextField.getText());
      //anatoliaObject.type = MyInteger.parseInt(AnatoliaConsts.objectTypeTexts[objectTypePanel.typeBox.getSelectedIndex()].getCode());
      anatoliaObject.type = objectTypePanel.typeBox.getSelectedIndex();
      ObjectSpecialType objectSpecialType = objectTypePanel.getObjectSpecialType();
      anatoliaObject.v0 = objectSpecialType.getV0();
      anatoliaObject.v1 = objectSpecialType.getV1();
      anatoliaObject.v2 = objectSpecialType.getV2();
      anatoliaObject.v3 = objectSpecialType.getV3();
      anatoliaObject.v4 = objectSpecialType.getV4();

      //extra & wear flags
      anatoliaObject.extraFlags = AreaBuilderConsts.selectionToString(AnatoliaConsts.extraFlagTexts, objectFlagsPanel.extraFlagsCheckBoxes);
      anatoliaObject.wearFlags  = AreaBuilderConsts.selectionToString(AnatoliaConsts.wearFlagTexts, objectFlagsPanel.wearFlagsRadioButtons);

      //imm/res/vuln
      anatoliaObject.immFlags  = AreaBuilderConsts.selectionToString(AnatoliaConsts.immResVulnFlagTexts, this.immResVulnPanel.immCheckBoxes);
      anatoliaObject.resFlags  = AreaBuilderConsts.selectionToString(AnatoliaConsts.immResVulnFlagTexts, this.immResVulnPanel.resCheckBoxes);
      anatoliaObject.vulnFlags = AreaBuilderConsts.selectionToString(AnatoliaConsts.immResVulnFlagTexts, this.immResVulnPanel.vulnCheckBoxes);

      //general & local affects
      anatoliaObject.generalAffects = AreaBuilderConsts.selectionToString(AnatoliaConsts.affTexts, affectsPanel.generalAffCheckBoxes);
      anatoliaObject.detectFlags    = AreaBuilderConsts.selectionToString(AnatoliaConsts.detTexts, affectsPanel.detCheckBoxes);
      for(int i=0 ; i<AnatoliaConsts.applyTexts.length ; ++i)
      {
         anatoliaObject.localAffects[i] = MyInteger.parseInt(affectsPanel.applyPanels[i].modifierTextField.getText());
      }

      //special prog
      objectSpecialsPanel.updateProgStrings(false);
      for(int i=0 ; i<AnatoliaConsts.objectProgTexts.length ; ++i)
      {
        anatoliaObject.specialProgs[i] = new String(objectSpecialsPanel.progStrings[i]);
      }
      anatoliaObject.limit = objectSpecialsPanel.limitedBox.getSelectedIndex();

      setVisible(false);
      return true;
   }
}

class ObjectDescriptionPanel extends JPanel
{
   JTextField nameTextField = new JTextField(15);
   JTextField shortTextField = new JTextField(20);
   JTextField longTextField = new JTextField(25);
   JTable table = new JTable(8,2);

   public ObjectDescriptionPanel()
   {
      setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      JPanel namePanel = new JPanel(new GridLayout(2,1));
      namePanel.add(new JLabel("Name of the object"));
      nameTextField.setToolTipText("A list of names by which the object may be referenced");
      namePanel.add(nameTextField);
      constraint.gridx=0;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=15;
      add(namePanel, constraint);

      JPanel shortPanel = new JPanel(new GridLayout(2,1));
      shortPanel.add(new JLabel("short description"));
      shortTextField.setToolTipText("Seen when the object is used, picked up or worn");
      shortPanel.add(shortTextField);
      constraint.gridx=0;constraint.gridy=1;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=15;
      add(shortPanel, constraint);

      JPanel longPanel = new JPanel(new GridLayout(2,1));
      longPanel.add(new JLabel("Long description"));
      longTextField.setToolTipText("Seen when the object is on the ground");
      longPanel.add(longTextField);
      constraint.gridx=0;constraint.gridy=2;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=15;
      add(longPanel, constraint);

      JPanel tablePanel = new JPanel();
      tablePanel.setBorder(new TitledBorder("Extra descriptions"));
      for(int i=0 ; i<table.getRowCount() ; ++i)
      {
         table.setValueAt("", i, 0);
         table.setValueAt("", i, 1);
      }
      table.setToolTipText("You can associate a special description to some special parts of the object");
      TableColumn column = null;
      TableColumnModel columnModel = table.getColumnModel();
      column = columnModel.getColumn(0);
      column.setHeaderValue("keywords");
      column.setPreferredWidth(200);
      column = columnModel.getColumn(1);
      column.setHeaderValue("Description");
      column.setPreferredWidth(800);
      constraint.gridx=0;constraint.gridy=3;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=55;
      tablePanel.add(new JScrollPane(table));
      add(tablePanel, constraint);
   }
}

class ObjectTypePanel extends JPanel implements ActionListener
{
   AreaMap areaMap = null;
   JComboBox materialBox = new JComboBox(AnatoliaConsts.objectMaterialTexts);
   JComboBox statusBox = new JComboBox(AnatoliaConsts.getObjectConditionTexts());
   JComboBox typeBox = new JComboBox(AnatoliaConsts.getObjectTypeTexts());
   CardLayout typeLayout = new CardLayout();
   JPanel currentTypePanel = new JPanel(typeLayout);
   JComboBox levelBox = new JComboBox(AnatoliaConsts.getMobileLevelsTexts());
   JTextField weightTextField = new JTextField("0",5);
   JTextField costTextField = new JTextField("0",5);

   WeaponTypePanel   weaponTypePanel   = new WeaponTypePanel();
   ArmorTypePanel    armorTypePanel    = new ArmorTypePanel();
   LightTypePanel    lightTypePanel    = new LightTypePanel();
   MoneyTypePanel    moneyTypePanel    = new MoneyTypePanel();
   DrinkTypePanel    drinkTypePanel    = new DrinkTypePanel();
   FountainTypePanel fountainTypePanel = new FountainTypePanel();
   WandsAndStavesTypePanel wandsAndStavesTypePanel = new WandsAndStavesTypePanel();
   PotionsScrollsPillsTypePanel potionsScrollsPillsTypePanel = new PotionsScrollsPillsTypePanel();
   ContainerTypePanel containerTypePanel = new ContainerTypePanel();
   FoodTypePanel    foodTypePanel    = new FoodTypePanel();
   PortalTypePanel  portalTypePanel;
   FurnitureTypePanel  furnitureTypePanel  = new FurnitureTypePanel();
   BombTypePanel  bombTypePanel  = new BombTypePanel();
   ObjectSpecialType noneTypePanel = new ObjectSpecialType();

   public ObjectTypePanel(AreaMap areaMap)
   {
      this.areaMap = areaMap;
      portalTypePanel  = new PortalTypePanel(areaMap);
      setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      JPanel materialPanel = new JPanel(new GridLayout(2,1));
      materialPanel.add(new JLabel("Material"));
      materialPanel.add(materialBox);
      constraint.gridx=0;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=15;
      add(materialPanel, constraint);

      JPanel levelPanel = new JPanel(new GridLayout(2,1));
      levelPanel.add(new JLabel("Level"));
      levelBox.addActionListener(this);
      levelPanel.add(levelBox);
      constraint.gridx=1;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=15;
      add(levelPanel, constraint);

      JPanel weightPanel = new JPanel(new GridLayout(2,1));
      weightPanel.add(new JLabel("Weight"));
      weightTextField.addActionListener(this);
      weightPanel.add(weightTextField);
      constraint.gridx=2;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=15;
      add(weightPanel, constraint);

      JPanel costPanel = new JPanel(new GridLayout(2,1));
      costPanel.add(new JLabel("Cost"));
      costPanel.add(costTextField);
      constraint.gridx=3;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=15;
      add(costPanel, constraint);

      JPanel statusPanel = new JPanel(new GridLayout(2,1));
      statusPanel.add(new JLabel("Status"));
      statusPanel.add(statusBox);
      constraint.gridx=0;constraint.gridy=1;
      constraint.gridwidth=2;constraint.gridheight=1;
      constraint.weightx=50;constraint.weighty=15;
      add(statusPanel, constraint);

      JPanel typePanel = new JPanel(new GridLayout(2,1));
      typePanel.add(new JLabel("Type"));
      typeBox.addActionListener(this);
      typePanel.add(typeBox);
      constraint.gridx=1;constraint.gridy=1;
      constraint.gridwidth=2;constraint.gridheight=1;
      constraint.weightx=50;constraint.weighty=15;
      add(typePanel, constraint);

      currentTypePanel.add(weaponTypePanel, "WEAPON");
      currentTypePanel.add(armorTypePanel , "ARMOR");
      currentTypePanel.add(lightTypePanel , "LIGHT");
      currentTypePanel.add(moneyTypePanel , "MONEY");
      currentTypePanel.add(drinkTypePanel , "DRINK");
      currentTypePanel.add(fountainTypePanel , "FOUNTAIN");
      currentTypePanel.add(wandsAndStavesTypePanel , "WANDS_AND_STAVES");
      currentTypePanel.add(potionsScrollsPillsTypePanel , "POTIONS_SCROLLS_PILLS");
      currentTypePanel.add(containerTypePanel , "CONTAINER");
      currentTypePanel.add(foodTypePanel      , "FOOD");
      currentTypePanel.add(portalTypePanel    , "PORTAL");
      currentTypePanel.add(furnitureTypePanel , "FURNITURE");
      currentTypePanel.add(bombTypePanel , "BOMB");
      currentTypePanel.add(noneTypePanel      , "NONE");
      currentTypePanel.setBorder(new TitledBorder("Current type properties"));
      constraint.gridx=0;constraint.gridy=2;
      constraint.gridwidth=4;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=70;
      add(currentTypePanel, constraint);

      setCost();
   }

   public void actionPerformed(ActionEvent e)
   {
      if (e.getSource() == typeBox)
      {
         ObjectSpecialType current = getObjectSpecialType();
         current.setV0("0");
         current.setV1("0");
         current.setV2("0");
         current.setV3("0");
         current.setV4("0");
         typeLayout.show(currentTypePanel, getObjectSpecialTypeCode());
      }
      else if (e.getSource() == levelBox)
      {
         setCost();
      }
   }

   public String getObjectSpecialTypeCode()
   {
      int selected = typeBox.getSelectedIndex();
      if (selected == AnatoliaConsts.OBJECT_TYPE_WEAPON)
         return "WEAPON";
      else if (selected == AnatoliaConsts.OBJECT_TYPE_ARMOR)
         return "ARMOR";
      else if (selected == AnatoliaConsts.OBJECT_TYPE_LIGHT)
         return "LIGHT";
      else if (selected == AnatoliaConsts.OBJECT_TYPE_MONEY)
         return "MONEY";
      else if (selected == AnatoliaConsts.OBJECT_TYPE_DRINK_CON)
         return "DRINK";
      else if (selected == AnatoliaConsts.OBJECT_TYPE_FOUNTAIN)
         return "FOUNTAIN";
      else if (selected == AnatoliaConsts.OBJECT_TYPE_WAND ||
               selected == AnatoliaConsts.OBJECT_TYPE_STAFF)
         return "WANDS_AND_STAVES";
      else if (selected == AnatoliaConsts.OBJECT_TYPE_POTION ||
               selected == AnatoliaConsts.OBJECT_TYPE_SCROLL ||
               selected == AnatoliaConsts.OBJECT_TYPE_PILL)
         return "POTIONS_SCROLLS_PILLS";
      else if (selected == AnatoliaConsts.OBJECT_TYPE_CONTAINER)
         return "CONTAINER";
      else if (selected == AnatoliaConsts.OBJECT_TYPE_FOOD)
         return "FOOD";
      else if (selected == AnatoliaConsts.OBJECT_TYPE_PORTAL)
         return "PORTAL";
      else if (selected == AnatoliaConsts.OBJECT_TYPE_FURNITURE)
         return "FURNITURE";
      else if (selected == AnatoliaConsts.OBJECT_TYPE_BOMB)
         return "BOMB";
      else
         return "NONE";
   }

   public ObjectSpecialType getObjectSpecialType()
   {
      if (typeBox.getSelectedIndex() == AnatoliaConsts.OBJECT_TYPE_WEAPON)
         return weaponTypePanel;
      else if (typeBox.getSelectedIndex() == AnatoliaConsts.OBJECT_TYPE_ARMOR)
         return armorTypePanel;
      else if (typeBox.getSelectedIndex() == AnatoliaConsts.OBJECT_TYPE_LIGHT)
         return lightTypePanel;
      else if (typeBox.getSelectedIndex() == AnatoliaConsts.OBJECT_TYPE_MONEY)
         return moneyTypePanel;
      else if (typeBox.getSelectedIndex() == AnatoliaConsts.OBJECT_TYPE_DRINK_CON)
         return drinkTypePanel;
      else if (typeBox.getSelectedIndex() == AnatoliaConsts.OBJECT_TYPE_FOUNTAIN)
         return fountainTypePanel;
      else if (typeBox.getSelectedIndex() == AnatoliaConsts.OBJECT_TYPE_WAND ||
            typeBox.getSelectedIndex() == AnatoliaConsts.OBJECT_TYPE_STAFF)
         return wandsAndStavesTypePanel;
      else if (typeBox.getSelectedIndex() == AnatoliaConsts.OBJECT_TYPE_POTION ||
            typeBox.getSelectedIndex() == AnatoliaConsts.OBJECT_TYPE_SCROLL ||
            typeBox.getSelectedIndex() == AnatoliaConsts.OBJECT_TYPE_PILL)
         return potionsScrollsPillsTypePanel;
      else if (typeBox.getSelectedIndex() == AnatoliaConsts.OBJECT_TYPE_CONTAINER)
         return containerTypePanel;
      else if (typeBox.getSelectedIndex() == AnatoliaConsts.OBJECT_TYPE_FOOD)
         return foodTypePanel;
      else if (typeBox.getSelectedIndex() == AnatoliaConsts.OBJECT_TYPE_PORTAL)
         return portalTypePanel;
      else if (typeBox.getSelectedIndex() == AnatoliaConsts.OBJECT_TYPE_FURNITURE)
         return furnitureTypePanel;
      else if (typeBox.getSelectedIndex() == AnatoliaConsts.OBJECT_TYPE_BOMB)
         return bombTypePanel;
      else
         return noneTypePanel;
   }

   public void setCost()
   {
      costTextField.setText(Integer.toString((levelBox.getSelectedIndex()+AnatoliaConsts.LEVEL_MIN)*10));
   }
}

class ObjectSpecialType extends JPanel
{
   public String getV0() {return "0";}
   public String getV1() {return "0";}
   public String getV2() {return "0";}
   public String getV3() {return "0";}
   public String getV4() {return "0";}
   public void setV0(String v0) {}
   public void setV1(String v1) {}
   public void setV2(String v2) {}
   public void setV3(String v3) {}
   public void setV4(String v4) {}
}

class WeaponTypePanel extends ObjectSpecialType
{
   JComboBox weaponTypeBox = new JComboBox(AnatoliaConsts.weaponTypeTexts);
   JTextField diceTextField  =  new JTextField("0",2);
   JTextField facesTextField =  new JTextField("0",2);
   JComboBox damageMessageBox = new JComboBox(AnatoliaConsts.weaponDamageTypeTexts);
   JCheckBox[] checkBoxes = new JCheckBox[AnatoliaConsts.weaponFlagsTexts.length];

   public String getV0() {return weaponTypeBox.getSelectedItem().toString();}
   public String getV1() {return diceTextField.getText();}
   public String getV2() {return facesTextField.getText();}
   public String getV3() {return damageMessageBox.getSelectedItem().toString();}
   public String getV4() {return AreaBuilderConsts.selectionToString(AnatoliaConsts.weaponFlagsTexts,checkBoxes);}
   public void setV0(String v0) {weaponTypeBox.setSelectedIndex(Math.max(0,AreaBuilderConsts.findInTab(AnatoliaConsts.weaponTypeTexts, v0)));}
   public void setV1(String v1) {diceTextField.setText(Integer.toString(MyInteger.parseInt(v1)));}
   public void setV2(String v2) {facesTextField.setText(Integer.toString(MyInteger.parseInt(v2)));}
   public void setV3(String v3) {damageMessageBox.setSelectedIndex(Math.max(0,AreaBuilderConsts.findInTab(AnatoliaConsts.weaponDamageTypeTexts, v3)));}
   public void setV4(String v4) {AreaBuilderConsts.stringToSelection(v4, AnatoliaConsts.weaponFlagsTexts, checkBoxes);}

   public WeaponTypePanel()
   {
      setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      JPanel typePanel = new JPanel(new GridLayout(2,1));
      typePanel.add(new JLabel("Type"));
      typePanel.add(weaponTypeBox);
      constraint.gridx=0;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=25;
      add(typePanel, constraint);

      JPanel dicePanel = new JPanel(new GridLayout(2,1));
      dicePanel.add(new JLabel("Nb Dices"));
      dicePanel.add(diceTextField);
      constraint.gridx=1;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=25;
      add(dicePanel, constraint);

      JPanel facesPanel = new JPanel(new GridLayout(2,1));
      facesPanel.add(new JLabel("Nb Faces"));
      facesPanel.add(facesTextField);
      constraint.gridx=2;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=25;
      add(facesPanel, constraint);

      JPanel damagePanel = new JPanel(new GridLayout(2,1));
      damagePanel.add(new JLabel("Damage"));
      damagePanel.add(damageMessageBox);
      constraint.gridx=3;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=25;
      add(damagePanel, constraint);

      JPanel flagsPanel = new JPanel(new GridLayout(3,3));
      flagsPanel.setBorder(new TitledBorder("Flags"));
      for(int i=0 ; i< AnatoliaConsts.weaponFlagsTexts.length ; ++i)
      {
         checkBoxes[i] = new JCheckBox(AnatoliaConsts.weaponFlagsTexts[i].getName());
         checkBoxes[i].setToolTipText(AnatoliaConsts.weaponFlagsTexts[i].getToolTipText());
         flagsPanel.add(checkBoxes[i]);
      }
      constraint.gridx=0;constraint.gridy=1;
      constraint.gridwidth=4;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=75;
      add(flagsPanel, constraint);
   }
}

class ArmorTypePanel extends ObjectSpecialType
{
   JTextField armorVsPierceTextField = new JTextField("0");
   JTextField armorVsBashTextField   = new JTextField("0");
   JTextField armorVsSlashTextField  = new JTextField("0");
   JTextField armorVsMagicTextField  = new JTextField("0");
   JComboBox bulkBox = new JComboBox(AnatoliaConsts.getObjectBulkTexts());

   public String getV0() {return armorVsPierceTextField.getText();}
   public String getV1() {return   armorVsBashTextField.getText();}
   public String getV2() {return  armorVsSlashTextField.getText();}
   public String getV3() {return  armorVsMagicTextField.getText();}
   public String getV4() {return AnatoliaConsts.objectBulkTexts[bulkBox.getSelectedIndex()].getCode();}
   public void setV0(String v0) {armorVsPierceTextField.setText(v0);}
   public void setV1(String v1) {armorVsBashTextField.setText(v1);}
   public void setV2(String v2) {armorVsSlashTextField.setText(v2);}
   public void setV3(String v3) {armorVsMagicTextField.setText(v3);}
   public void setV4(String v4) {bulkBox.setSelectedIndex(MyInteger.parseInt(v4)*2);}

   public ArmorTypePanel()
   {
      JPanel armorVsPanel = new JPanel(new GridLayout(2,4));
      armorVsPanel.setBorder(new TitledBorder("Armor versus..."));
      armorVsPanel.add(new JLabel("pierce"));
      armorVsPanel.add(new JLabel("bash"));
      armorVsPanel.add(new JLabel("slash"));
      armorVsPanel.add(new JLabel("magic"));
      armorVsPanel.add(armorVsPierceTextField);
      armorVsPanel.add(armorVsBashTextField);
      armorVsPanel.add(armorVsSlashTextField);
      armorVsPanel.add(armorVsMagicTextField);
      add(armorVsPanel);

      JPanel bulkPanel = new JPanel(new GridLayout(2,1));
      bulkPanel.add(new JLabel("Bulk"));
      bulkPanel.add(bulkBox);
      add(bulkPanel);
  }
}

class LightTypePanel extends ObjectSpecialType implements ActionListener
{
   JCheckBox  permanentCheckBox = new JCheckBox("permanent", true);
   JTextField durationTextField = new JTextField("-1");

   public String getV0() {return "0";}
   public String getV1() {return "0";}
   public String getV2() {return durationTextField.getText();}
   public String getV3() {return "0";}
   public String getV4() {return "0";}
   public void setV0(String v0) {}
   public void setV1(String v1) {}
   public void setV2(String v2)
   {
      int duration = MyInteger.parseInt(v2);
      v2 = Integer.toString(Math.max(duration, -1));
      durationTextField.setText(v2);
      permanentCheckBox.setSelected(duration<0);
   }
   public void setV3(String v3) {}
   public void setV4(String v4) {}


   public LightTypePanel()
   {
      permanentCheckBox.addActionListener(this);
      add(permanentCheckBox);

      JPanel durationPanel = new JPanel(new GridLayout(1,2));
      durationPanel.add(new JLabel("Duration:"));
      durationTextField.setToolTipText("Duration in game hours (-1 for permanent)");
      durationTextField.setEnabled(!permanentCheckBox.isSelected());
      durationPanel.add(durationTextField);
      add(durationPanel);
   }

   public void actionPerformed(ActionEvent e)
   {
      if (e.getSource() == permanentCheckBox)
      {
         if (permanentCheckBox.isSelected()) durationTextField.setText("-1");
         durationTextField.setEnabled(!permanentCheckBox.isSelected());
      }
   }
}

class MoneyTypePanel extends ObjectSpecialType
{
   JTextField silverTextField = new JTextField("0");
   JTextField   goldTextField = new JTextField("0");

   public String getV0() {return silverTextField.getText();}
   public String getV1() {return goldTextField.getText();}
   public String getV2() {return "0";}
   public String getV3() {return "0";}
   public String getV4() {return "0";}
   public void setV0(String v0) {silverTextField.setText(v0);}
   public void setV1(String v1) {goldTextField.setText(v1);}
   public void setV2(String v2) {}
   public void setV3(String v3) {}
   public void setV4(String v4) {}

   public MoneyTypePanel()
   {
      JPanel silverPanel = new JPanel(new GridLayout(2,1));
      silverPanel.add(new JLabel("Silver value"));
      silverPanel.add(silverTextField);
      add(silverPanel);

      JPanel goldPanel = new JPanel(new GridLayout(2,1));
      goldPanel.add(new JLabel("Gold value"));
      goldPanel.add(goldTextField);
      add(goldPanel);
   }
}

class DrinkTypePanel extends ObjectSpecialType
{
   JTextField capacityTextField = new JTextField("64");
   JTextField  contentTextField = new JTextField("64");
   JComboBox liquidBox = new JComboBox(AnatoliaConsts.getLiquidTexts());
   JCheckBox poisonedCheckBox = new JCheckBox("Poisoned");

   public String getV0() {return capacityTextField.getText();}
   public String getV1() {return contentTextField.getText();}
   public String getV2() {return "'"+liquidBox.getSelectedItem()+"'";}
   public String getV3() {if (poisonedCheckBox.isSelected()) return "A"; else return "0";}
   public String getV4() {return "0";}
   public void setV0(String v0) {capacityTextField.setText(v0);}
   public void setV1(String v1) {contentTextField.setText(v1);}
   public void setV2(String v2)
   {
      String liquid = "";
      if (v2.length() >=3)
        liquid = v2.substring(1,v2.length()-1); //removes quotes
      liquidBox.setSelectedIndex(AreaBuilderConsts.findInTab(AnatoliaConsts.getLiquidTexts(), liquid));
   }
   public void setV3(String v3) {poisonedCheckBox.setSelected(!v3.equals("0"));}
   public void setV4(String v4) {}


   public DrinkTypePanel()
   {
      JPanel capacityPanel = new JPanel(new GridLayout(2,1));
      capacityPanel.add(new JLabel("Capacity"));
      capacityTextField.setToolTipText("64 is a good value for a buffalo water skin");
      capacityPanel.add(capacityTextField);
      add(capacityPanel);

      JPanel contentPanel = new JPanel(new GridLayout(2,1));
      contentPanel.add(new JLabel("Content"));
      contentTextField.setToolTipText("less than the capacity, please");
      contentPanel.add(contentTextField);
      add(contentPanel);

      JPanel liquidPanel = new JPanel(new GridLayout(2,1));
      liquidPanel.add(new JLabel("Type"));
      liquidPanel.add(liquidBox);
      add(liquidPanel);

      add(poisonedCheckBox);
   }
}

class FountainTypePanel extends ObjectSpecialType
{
   JComboBox liquidBox = new JComboBox(AnatoliaConsts.getLiquidTexts());

   public String getV0() {return "9999";}
   public String getV1() {return "9999";}
   public String getV2() {return "'"+liquidBox.getSelectedItem()+"'";}
   public String getV3() {return "0";}
   public String getV4() {return "0";}
   public void setV0(String v0) {}
   public void setV1(String v1) {}
   public void setV2(String v2)
   {
      String liquid = "";
      if (v2.length() >= 3)
         liquid = v2.substring(1,v2.length()-1); //removes quotes
      liquidBox.setSelectedIndex(AreaBuilderConsts.findInTab(AnatoliaConsts.getLiquidTexts(), liquid));
   }
   public void setV3(String v3) {}
   public void setV4(String v4) {}

   public FountainTypePanel()
   {
      JPanel liquidPanel = new JPanel(new GridLayout(2,1));
      liquidPanel.add(new JLabel("Type"));
      liquidPanel.add(liquidBox);
      add(liquidPanel);
   }
}

class WandsAndStavesTypePanel extends ObjectSpecialType
{
   JComboBox levelBox = new JComboBox(AnatoliaConsts.getMobileLevelsTexts());
   JTextField chargeCapacityTextField = new JTextField("10");
   JTextField  chargeContentTextField = new JTextField("10");
   JComboBox spellBox = new JComboBox(AnatoliaConsts.getSpellTexts());

   public String getV0() {return Integer.toString(levelBox.getSelectedIndex()+AnatoliaConsts.LEVEL_MIN);}
   public String getV1() {return chargeCapacityTextField.getText();}
   public String getV2() {return chargeContentTextField.getText();}
   public String getV3() {return "'"+spellBox.getSelectedItem()+"'";}
   public String getV4() {return "0";}
   public void setV0(String v0) {levelBox.setSelectedIndex(Math.max(0,MyInteger.parseInt(v0)-AnatoliaConsts.LEVEL_MIN));}
   public void setV1(String v1) {chargeCapacityTextField.setText(v1);}
   public void setV2(String v2) {chargeContentTextField.setText(v2);}
   public void setV3(String v3)
   {
      int sn=0;
      try
      {
        sn = Integer.parseInt(v3); //is it a number?
        sn = AreaBuilderConsts.findInTab(AnatoliaConsts.getSpellCodes(), v3);
        spellBox.setSelectedIndex(sn);
      }
      catch(Exception e)
      {
        String spell = "";
        if (v3.length() >= 3)
          spell = v3.substring(1,v3.length()-1); //removes quotes
        spellBox.setSelectedIndex(AreaBuilderConsts.findInTab(AnatoliaConsts.getSpellTexts(), spell));
      }
   }
   public void setV4(String v4) {}

   public WandsAndStavesTypePanel()
   {
      JPanel levelPanel = new JPanel(new GridLayout(2,1));
      levelPanel.add(new JLabel("Level"));
      levelPanel.add(levelBox);
      add(levelPanel);

      JPanel chargeCapacityPanel = new JPanel(new GridLayout(2,1));
      chargeCapacityPanel.add(new JLabel("Charges capacity"));
      chargeCapacityTextField.setToolTipText("number of charges that can be contained");
      chargeCapacityPanel.add(chargeCapacityTextField);
      add(chargeCapacityPanel);

      JPanel chargeContentPanel = new JPanel(new GridLayout(2,1));
      chargeContentPanel.add(new JLabel("Charges content"));
      chargeContentTextField.setToolTipText("current number of charges");
      chargeContentPanel.add(chargeContentTextField);
      add(chargeContentPanel);

      JPanel spellPanel = new JPanel(new GridLayout(2,1));
      spellPanel.add(new JLabel("Spell"));
      spellPanel.add(spellBox);
      add(spellPanel);
   }
}

class PotionsScrollsPillsTypePanel extends ObjectSpecialType implements ActionListener
{
   JComboBox levelBox = new JComboBox(AnatoliaConsts.getMobileLevelsTexts());
   private final static String[] numbers = {"0","1","2","3","4"};
   JComboBox numberOfSpellsBox = new JComboBox(numbers);
   JComboBox[] spellBoxes = new JComboBox[4];

   public String getV0() {return Integer.toString(levelBox.getSelectedIndex()+AnatoliaConsts.LEVEL_MIN);}
   public String getV1() {return getSpell(1);}
   public String getV2() {return getSpell(2);}
   public String getV3() {return getSpell(3);}
   public String getV4() {return getSpell(4);}
   public void setV0(String v0) {levelBox.setSelectedIndex(Math.max(0,MyInteger.parseInt(v0)-AnatoliaConsts.LEVEL_MIN));}
   public void setV1(String v1) {setSpell(1,v1);}
   public void setV2(String v2) {setSpell(2,v2);}
   public void setV3(String v3) {setSpell(3,v3);}
   public void setV4(String v4) {setSpell(4,v4);}

   public PotionsScrollsPillsTypePanel()
   {
      for(int i=0 ; i<spellBoxes.length ; ++i)
      {
         spellBoxes[i] = new JComboBox(AnatoliaConsts.getSpellTexts());
      }
      hideSpells();

      JPanel levelPanel = new JPanel(new GridLayout(2,1));
      levelPanel.add(new JLabel("Level"));
      levelPanel.add(levelBox);
      add(levelPanel);

      JPanel numberOfSpellsPanel = new JPanel(new GridLayout(2,1));
      numberOfSpellsPanel.add(new JLabel("Nb of spells"));
      numberOfSpellsBox.addActionListener(this);
      numberOfSpellsPanel.add(numberOfSpellsBox);
      add(numberOfSpellsPanel);

      for(int i=0 ; i<spellBoxes.length ; ++i)
      {
         JPanel spellPanel = new JPanel(new GridLayout(2,1));
         spellPanel.add(new JLabel("Spell"+Integer.toString(i+1)));
         spellPanel.add(spellBoxes[i]);
         add(spellPanel);
      }
   }

   public void actionPerformed(ActionEvent e)
   {
      if (e.getSource() == numberOfSpellsBox)
      {
         hideSpells();
      }
   }

   public void hideSpells()
   {
      for(int i=0 ; i<spellBoxes.length ; ++i)
      {
         spellBoxes[i].setVisible(i+1<=numberOfSpellsBox.getSelectedIndex());
      }
   }

   public String getSpell(int index)
   {
      if (index <= numberOfSpellsBox.getSelectedIndex()+1)
        return "'"+spellBoxes[index-1].toString()+"'";
      else
        return "''";
   }
   public void setSpell(int index, String v)
   {
      int sn=0;
      try
      {
        sn = Integer.parseInt(v); //is it a number?
        sn = AreaBuilderConsts.findInTab(AnatoliaConsts.getSpellCodes(), v);

        if (sn >= 0)
        {
          //if no Exception is raised, this IS the spell number
          spellBoxes[index-1].setSelectedIndex(sn);
          numberOfSpellsBox.setSelectedIndex(Math.max(index, numberOfSpellsBox.getSelectedIndex()));
        }
        else
        {
          numberOfSpellsBox.setSelectedIndex(Math.min(index-1, numberOfSpellsBox.getSelectedIndex()));
        }
      }
      catch(Exception e)
      {
        String spell = "";
        if (v.length() >= 3)
           spell = v.substring(1,v.length()-1); //removes quotes
        int found = AreaBuilderConsts.findInTab(AnatoliaConsts.getSpellTexts(), spell);
        if (found>=0)
        {
          spellBoxes[index-1].setSelectedIndex(found);
          numberOfSpellsBox.setSelectedIndex(Math.max(index, numberOfSpellsBox.getSelectedIndex()));
        }
        else
        {
          numberOfSpellsBox.setSelectedIndex(Math.min(index-1, numberOfSpellsBox.getSelectedIndex()));
        }
     }
   }
}

class ContainerTypePanel extends ObjectSpecialType implements ChangeListener
{
   JTextField maximalWeightTextField   = new JTextField("0");
   JTextField heaviestContentTextField = new JTextField("0");
   JSlider  weightAdapterSlider = new JSlider(JSlider.HORIZONTAL,0,200,100);
   JLabel   weightAdapterLabel = new JLabel("Weight adapter :"+Integer.toString(weightAdapterSlider.getValue())+"%");
   JCheckBox[] flagsCheckBoxes = new JCheckBox[AnatoliaConsts.containerFlagTexts.length];

   public String getV0() {return maximalWeightTextField.getText();}
   public String getV1() {return getFlags();}
   public String getV2() {return getV1();}
   public String getV3() {return heaviestContentTextField.getText();}
   public String getV4() {return Integer.toString(Math.max(1,weightAdapterSlider.getValue()));}
   public void setV0(String v0) {maximalWeightTextField.setText(v0);}
   public void setV1(String v1) {setFlags(v1);}
   public void setV2(String v2) {}
   public void setV3(String v3) {heaviestContentTextField.setText(v3);}
   public void setV4(String v4)
   {
      weightAdapterSlider.setToolTipText("A container can reduce the weight of its content by x%");
      weightAdapterSlider.setValue(Math.max(1,MyInteger.parseInt(v4)));
      weightAdapterLabel.setText(Integer.toString(weightAdapterSlider.getValue()));
   }

   public ContainerTypePanel()
   {
      setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      JPanel maxWeightPanel = new JPanel(new GridLayout(2,1));
      maxWeightPanel.add(new JLabel("Maximum weight"));
      maximalWeightTextField.setToolTipText("Maximum weight that can be contained");
      maxWeightPanel.add(maximalWeightTextField);
      constraint.gridx=0;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=40;
      add(maxWeightPanel, constraint);

      JPanel heaviestWeightPanel = new JPanel(new GridLayout(2,1));
      heaviestWeightPanel.add(new JLabel("Heaviest weight"));
      maximalWeightTextField.setToolTipText("Heaviest object that can be contained");
      heaviestWeightPanel.add(heaviestContentTextField);
      constraint.gridx=1;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=40;
      add(heaviestWeightPanel,constraint);

      JPanel weightPanel = new JPanel(new GridLayout(2,1));
      weightPanel.add(weightAdapterLabel);
      Hashtable labels = new Hashtable();
      for(int i=0 ; i<=200 ; i+=25)
      {
         labels.put(new Integer(i),new JLabel(Integer.toString(i)));
      }
      weightAdapterSlider.setLabelTable(labels);
      weightAdapterSlider.setPaintLabels(true);
      weightAdapterSlider.setMajorTickSpacing(10);
      weightAdapterSlider.setMinorTickSpacing(5);
      weightAdapterSlider.setPaintTicks(true);
      weightAdapterSlider.setSnapToTicks(true);
      weightAdapterSlider.setToolTipText("Weight modificator of container (%)");
      weightAdapterSlider.addChangeListener(this);
      weightPanel.add(weightAdapterSlider);
      constraint.gridx=2;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=50;constraint.weighty=40;
      add(weightPanel,constraint);

      JPanel flagsPanel = new JPanel(new GridLayout(2,3));
      for(int i=0 ; i<flagsCheckBoxes.length ; ++i)
      {
        flagsCheckBoxes[i] = new JCheckBox(AnatoliaConsts.containerFlagTexts[i].getName());
        flagsCheckBoxes[i].setToolTipText(AnatoliaConsts.containerFlagTexts[i].getToolTipText());
        flagsPanel.add(flagsCheckBoxes[i]);
      }
      constraint.gridx=1;constraint.gridy=1;
      constraint.gridwidth=3;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=60;
      add(flagsPanel,constraint);
   }

   public void stateChanged(ChangeEvent e)
   {
      if (e.getSource() == weightAdapterSlider)
      {
         weightAdapterLabel.setText("Weight adapter :"+ Integer.toString(weightAdapterSlider.getValue())+"%");
      }
   }

   public String getFlags()
   {
     String flags = "";
     for(int i=0 ; i<flagsCheckBoxes.length ; ++i)
     {
       if (flagsCheckBoxes[i].isSelected())
       {
         flags += AnatoliaConsts.containerFlagTexts[i].getCode();
       }
     }
     if (flags.equals("")) flags = "0";
     return flags;
   }

   public void setFlags(String flags)
   {
     AreaBuilderConsts.stringToSelection(flags,AnatoliaConsts.containerFlagTexts, flagsCheckBoxes);
   }

}

class FoodTypePanel extends ObjectSpecialType
{
   JTextField fullTextField = new JTextField("10");
   JTextField hungryTextField = new JTextField("5");
   JCheckBox poisonedCheckBox = new JCheckBox("Poisoned");

   public String getV0() {return fullTextField.getText();}
   public String getV1() {return hungryTextField.getText();}
   public String getV2() {return "0";}
   public String getV3() {if (poisonedCheckBox.isSelected()) return "A"; else return "0";}
   public String getV4() {return "0";}
   public void setV0(String v0) {fullTextField.setText(v0);}
   public void setV1(String v1) {hungryTextField.setText(v1);}
   public void setV2(String v2) {}
   public void setV3(String v3) {poisonedCheckBox.setSelected(!v3.equals("0"));}
   public void setV4(String v4) {}

   public FoodTypePanel()
   {
      JPanel fullPanel = new JPanel(new GridLayout(2,1));
      fullPanel.add(new JLabel("\"Fullness\""));
      fullTextField.setToolTipText("number of hours this food keeps full");
      fullPanel.add(fullTextField);
      add(fullPanel);

      JPanel hungryPanel = new JPanel(new GridLayout(2,1));
      hungryPanel.add(new JLabel("\"Hungryness\""));
      hungryTextField.setToolTipText("number of hours this food fights hunger");
      hungryPanel.add(hungryTextField);
      add(hungryPanel);

      add(poisonedCheckBox);
   }
}

class PortalTypePanel extends ObjectSpecialType implements ActionListener
{
   AreaMap areaMap = null;
   JCheckBox  permanentCheckBox = new JCheckBox("permanent", true);
   JTextField durationTextField = new JTextField("0");
   JComboBox  exitRoomBox = new JComboBox();
   JCheckBox  areaGate = new JCheckBox("towards a room of this area");
   JTextField roomVnum = new JTextField("00000",6);
   JCheckBox[] exitFlagsCheckBoxes   = new JCheckBox[0];
   JCheckBox[] gateFlagsCheckBoxes = new JCheckBox[AnatoliaConsts.objectPortalGateFlagTexts.length];

   public String getV0() {return durationTextField.getText();}
   public String getV1() {return "0";}//exit flags
   public String getV2()
   {
      String flags = AreaBuilderConsts.selectionToString(AnatoliaConsts.objectPortalGateFlagTexts,gateFlagsCheckBoxes);
      if (flags.trim().equals("")) flags = AnatoliaConsts.objectPortalGateFlagTexts[AnatoliaConsts.OBJECT_PORTAL_GATEFLAG_NORMAL_EXIT].getCode();
      return flags;
   }

   public String getV3() {return Integer.toString(((Room) exitRoomBox.getSelectedItem()).anatoliaRoom.getVnum());}
   public String getV4() {return "0";}
   public void setV0(String v0) {durationTextField.setText(v0);}
   public void setV1(String v1) {}
   public void setV2(String v2) {AreaBuilderConsts.stringToSelection(v2, AnatoliaConsts.objectPortalGateFlagTexts, gateFlagsCheckBoxes);}
   public void setV3(String v3)
   {
      roomVnum.setText(v3);
      AnatoliaRoom anatoliaRoom = (AnatoliaRoom) areaMap.theRooms.anatoliaCollection.getAnatoliaThingOfVnum(MyInteger.parseInt(v3));
      Room room = (anatoliaRoom == null) ? null : anatoliaRoom.myRoom;
      if (room == null)
      {
        areaGate.setSelected(false);
        exitRoomBox.setEnabled(false);
        roomVnum.setEnabled(true);
      }
      else
      {
        areaGate.setSelected(true);
        exitRoomBox.setEnabled(true);
        roomVnum.setEnabled(false);
        exitRoomBox.setSelectedItem(room);
      }
   }

   public void setV4(String v4) {}


   public PortalTypePanel(AreaMap areaMap)
   {
      this.areaMap = areaMap;
      setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      permanentCheckBox.addActionListener(this);
      constraint.gridx=0;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=33;constraint.weighty=20;
      add(permanentCheckBox, constraint);

      JPanel durationPanel = new JPanel(new GridLayout(2,1));
      durationPanel.add(new JLabel("Duration"));
      durationTextField.setToolTipText("Duration in game hours (0 for permanent)");
      durationTextField.setEnabled(!permanentCheckBox.isSelected());
      durationPanel.add(durationTextField);
      constraint.gridx=1;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=33;constraint.weighty=20;
      add(durationPanel, constraint);

      JPanel exitRoomPanel = new JPanel(new GridLayout(5,1));
      exitRoomPanel.add(new JLabel("Exit Room"));
      exitRoomPanel.add(areaGate);
      areaGate.addActionListener(this);
      exitRoomPanel.add(exitRoomBox);
      exitRoomPanel.add(new JLabel("Exit room vnum"));
      exitRoomPanel.add(roomVnum);
      constraint.gridx=2;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=33;constraint.weighty=20;
      add(exitRoomPanel, constraint);

      JPanel gateFlagsPanel = new JPanel(new GridLayout(2,3));
      for(int i=0 ; i<gateFlagsCheckBoxes.length ; ++i)
      {
         gateFlagsCheckBoxes[i] = new JCheckBox(AnatoliaConsts.objectPortalGateFlagTexts[i].getName());
         if (i != AnatoliaConsts.OBJECT_PORTAL_GATEFLAG_NORMAL_EXIT)
            gateFlagsPanel.add(gateFlagsCheckBoxes[i]);
         gateFlagsCheckBoxes[i].setToolTipText(AnatoliaConsts.objectPortalGateFlagTexts[i].getToolTipText());
      }
      ButtonGroupMaybeNone group1 = new ButtonGroupMaybeNone();
      group1.add(gateFlagsCheckBoxes[AnatoliaConsts.OBJECT_PORTAL_GATEFLAG_GATE_BUGGY]);
      group1.add(gateFlagsCheckBoxes[AnatoliaConsts.OBJECT_PORTAL_GATEFLAG_RANDOM]);
      constraint.gridx=0;constraint.gridy=1;
      constraint.gridwidth=2;constraint.gridheight=1;
      constraint.weightx=50;constraint.weighty=80;
      add(gateFlagsPanel, constraint);
   }

   public void actionPerformed(ActionEvent e)
   {
      if (e.getSource() == permanentCheckBox)
      {
         if (permanentCheckBox.isSelected()) durationTextField.setText("0");
         durationTextField.setEnabled(!permanentCheckBox.isSelected());
      }
      if (e.getSource() == areaGate)
      {
        exitRoomBox.setEnabled(areaGate.isSelected());
        roomVnum.setEnabled(!areaGate.isSelected());
      }
   }
}

class FurnitureTypePanel extends ObjectSpecialType implements ChangeListener
{
   JTextField peopleTextField = new JTextField("0");
   JTextField weightTextField = new JTextField("0");
   JSlider  hpSlider = new JSlider(JSlider.HORIZONTAL,0,200,100);
   JSlider  mpSlider = new JSlider(JSlider.HORIZONTAL,0,200,100);
   JCheckBox[] flagsCheckBoxes = new JCheckBox[AnatoliaConsts.objectFurnitureFlagTexts.length];
   JLabel hpLabel = new JLabel("HP recovery rate : "+ hpSlider.getValue()+"%");
   JLabel mpLabel = new JLabel("MP recovery rate : "+ mpSlider.getValue()+"%");

   public String getV0() {return peopleTextField.getText();}
   public String getV1() {return weightTextField.getText();}
   public String getV2() {return AreaBuilderConsts.selectionToString(AnatoliaConsts.objectFurnitureFlagTexts,flagsCheckBoxes);}
   public String getV3() {return Integer.toString(Math.max(1, hpSlider.getValue()));}
   public String getV4() {return Integer.toString(Math.max(1, mpSlider.getValue()));}
   public void setV0(String v0) {peopleTextField.setText(v0);}
   public void setV1(String v1) {weightTextField.setText(v1);}
   public void setV2(String v2) {AreaBuilderConsts.stringToSelection(v2, AnatoliaConsts.objectFurnitureFlagTexts, flagsCheckBoxes);}
   public void setV3(String v3)
   {
      hpSlider.setValue(Math.max(1,MyInteger.parseInt(v3)));
      hpLabel.setText("HP recovery rate : "+ hpSlider.getValue()+"%");
   }
   public void setV4(String v4)
   {
      mpSlider.setValue(Math.max(1,MyInteger.parseInt(v4)));
      mpLabel.setText("MP recovery rate : "+ mpSlider.getValue()+"%");
   }

   public FurnitureTypePanel()
   {
      setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      JPanel peoplePanel = new JPanel(new GridLayout(2,1));
      peoplePanel.add(new JLabel("Max people"));
      peopleTextField.setToolTipText("Maximum number of people that can be contained in");
      peoplePanel.add(peopleTextField);
      constraint.gridx=0;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=25;
      add(peoplePanel, constraint);

      JPanel weightPanel = new JPanel(new GridLayout(2,1));
      weightPanel.add(new JLabel("Max weight"));
      weightTextField.setToolTipText("Maximum weight that can be supported");
      weightPanel.add(weightTextField);
      constraint.gridx=1;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=25;
      add(weightPanel, constraint);


      Hashtable labels = new Hashtable();
      for(int i=0 ; i<=200 ; i+=25)
      {
         labels.put(new Integer(i),new JLabel(Integer.toString(i)));
      }

      JPanel hpPanel = new JPanel(new GridLayout(2,1));
      hpPanel.add(hpLabel);
      hpSlider.setLabelTable(labels);
      hpSlider.setPaintLabels(true);
      hpSlider.setMajorTickSpacing(10);
      hpSlider.setMinorTickSpacing(5);
      hpSlider.setPaintTicks(true);
      hpSlider.setSnapToTicks(true);
      hpSlider.addChangeListener(this);
      hpPanel.add(hpSlider);
      constraint.gridx=2;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=25;
      add(hpPanel, constraint);

      JPanel mpPanel = new JPanel(new GridLayout(2,1));
      mpPanel.add(mpLabel);
      mpSlider.setLabelTable(labels);
      mpSlider.setPaintLabels(true);
      mpSlider.setMajorTickSpacing(10);
      mpSlider.setMinorTickSpacing(5);
      mpSlider.setPaintTicks(true);
      mpSlider.setSnapToTicks(true);
      mpSlider.addChangeListener(this);
      mpPanel.add(mpSlider);
      constraint.gridx=3;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=25;
      add(mpPanel, constraint);

      JPanel flagsPanel = new JPanel(new GridLayout(6,3));
      for(int i=0 ; i<flagsCheckBoxes.length ; ++i)
      {
         flagsCheckBoxes[i] = new JCheckBox(AnatoliaConsts.objectFurnitureFlagTexts[i].getName());
         flagsPanel.add(flagsCheckBoxes[i]);
      }
      for(int i=0 ; i<4 ; ++i)
      {
         ButtonGroupMaybeNone group = new ButtonGroupMaybeNone();
         for(int j=0; j<3 ; ++j)
         {
            group.add(flagsCheckBoxes[i*3+j]);
         }
      }
      ButtonGroupMaybeNone group2 = new ButtonGroupMaybeNone();
      for(int i=AnatoliaConsts.OBJECT_FURNITURE_FLAG_PUT_AT ; i<=AnatoliaConsts.OBJECT_FURNITURE_FLAG_PUT_INSIDE ; ++i)
      {
         group2.add(flagsCheckBoxes[i]);
      }
      constraint.gridx=0;constraint.gridy=1;
      constraint.gridwidth=4;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=75;
      add(flagsPanel, constraint);
   }

   public void stateChanged(ChangeEvent e)
   {
      if (e.getSource() == hpSlider)
      {
         hpLabel.setText("HP recovery rate : "+hpSlider.getValue()+"%");
      }
      else if (e.getSource() == mpSlider)
      {
         mpLabel.setText("MP recovery rate : "+mpSlider.getValue()+"%");
      }
   }
}

class BombTypePanel extends ObjectSpecialType
{
   JComboBox bombLevelBox = new JComboBox(AnatoliaConsts.getMobileLevelsTexts());
   JComboBox bombSpellBox = new JComboBox(AnatoliaConsts.getSpellTexts());
   JComboBox detonatorLevelBox = new JComboBox(AnatoliaConsts.getMobileLevelsTexts());
   JComboBox detonatorSpellBox = new JComboBox(AnatoliaConsts.getSpellTexts());

   public String getV0() {return Integer.toString(bombLevelBox.getSelectedIndex()+AnatoliaConsts.LEVEL_MIN);}
   public String getV1() {return "'"+bombSpellBox.getSelectedItem()+"'";}
   public String getV2() {return Integer.toString(detonatorLevelBox.getSelectedIndex()+AnatoliaConsts.LEVEL_MIN);}
   public String getV3() {return "'"+detonatorSpellBox.getSelectedItem()+"'";}
   public String getV4() {return "0";}
   public void setV0(String v0) {bombLevelBox.setSelectedIndex(Math.max(0,MyInteger.parseInt(v0)-AnatoliaConsts.LEVEL_MIN));}
   public void setV1(String v1)
   {
      String spell = "";
      if (v1.length() >= 3) spell = v1.substring(1,v1.length()-1); //removes quotes
      bombSpellBox.setSelectedIndex(AreaBuilderConsts.findInTab(AnatoliaConsts.getSpellTexts(), spell));
   }

   public void setV2(String v2) {detonatorLevelBox.setSelectedIndex(Math.max(0,MyInteger.parseInt(v2)-AnatoliaConsts.LEVEL_MIN));}
   public void setV3(String v3)
   {
      String spell = "";
      if (v3.length() >= 3)
        spell = v3.substring(1,v3.length()-1); //removes quotes
      detonatorSpellBox.setSelectedIndex(AreaBuilderConsts.findInTab(AnatoliaConsts.getSpellTexts(), spell));
   }
   public void setV4(String v4) {}

   public BombTypePanel()
   {
      JPanel bombLevelPanel = new JPanel(new GridLayout(2,1));
      bombLevelPanel.add(new JLabel("Bomb Level"));
      bombLevelPanel.add(bombLevelBox);
      add(bombLevelPanel);

      JPanel bombSpellPanel = new JPanel(new GridLayout(2,1));
      bombSpellPanel.add(new JLabel("Bomb Spell"));
      bombSpellPanel.add(bombSpellBox);
      add(bombSpellPanel);

      JPanel detonatorLevelPanel = new JPanel(new GridLayout(2,1));
      detonatorLevelPanel.add(new JLabel("Detonator Level"));
      detonatorLevelPanel.add(detonatorLevelBox);
      add(detonatorLevelPanel);

      JPanel detonatorSpellPanel = new JPanel(new GridLayout(2,1));
      detonatorSpellPanel.add(new JLabel("Detonator Spell"));
      detonatorSpellPanel.add(detonatorSpellBox);
      add(detonatorSpellPanel);
   }
}


class ObjectFlagsPanel extends JPanel implements ActionListener
{
   JCheckBox canBeTaken = new JCheckBox(AnatoliaConsts.wearFlagTexts[AnatoliaConsts.WEAR_FLAG_TAKE].getName());
   JCheckBox[] extraFlagsCheckBoxes = new JCheckBox[AnatoliaConsts.extraFlagTexts.length];
   JRadioButton[] wearFlagsRadioButtons = new JRadioButton[AnatoliaConsts.wearFlagTexts.length];
   ButtonGroupMaybeNone wearGroup = new ButtonGroupMaybeNone();

   public ObjectFlagsPanel()
   {
      setLayout(new GridLayout(1,2));

      JPanel extraFlagsPanel = new JPanel(new GridLayout(9,3));
      extraFlagsPanel.setBorder(new TitledBorder("Extra flags"));
      for(int i=0 ; i<extraFlagsCheckBoxes.length ; ++i)
      {
         extraFlagsCheckBoxes[i] = new JCheckBox(AnatoliaConsts.extraFlagTexts[i].getName());
         extraFlagsCheckBoxes[i].setToolTipText(AnatoliaConsts.extraFlagTexts[i].getToolTipText());
         extraFlagsPanel.add(extraFlagsCheckBoxes[i]);
      }
      add(extraFlagsPanel);

      JPanel wearFlagsPanel = new JPanel(new GridLayout(11,2));
      wearFlagsPanel.setBorder(new TitledBorder("Wear flags"));

      canBeTaken.setToolTipText(AnatoliaConsts.wearFlagTexts[AnatoliaConsts.WEAR_FLAG_TAKE].getToolTipText());
      canBeTaken.addActionListener(this);
      canBeTaken.setSelected(true);
      wearFlagsPanel.add(canBeTaken);
      for(int i=0 ; i<wearFlagsRadioButtons.length ; ++i)
      {
         wearFlagsRadioButtons[i] = new JRadioButton(AnatoliaConsts.wearFlagTexts[i].getName());
         wearFlagsRadioButtons[i].setToolTipText(AnatoliaConsts.wearFlagTexts[i].getToolTipText());
         if (i>0) wearFlagsPanel.add(wearFlagsRadioButtons[i]);
      }
      checkCanBeTaken();
      add(wearFlagsPanel);
   }

   public void actionPerformed(ActionEvent e)
   {
      if (e.getSource() == canBeTaken)
      {
         checkCanBeTaken();
      }
   }

   public void checkCanBeTaken()
   {
      for(int i=1 ; i<wearFlagsRadioButtons.length ; ++i)
      {
         wearFlagsRadioButtons[i].setEnabled(canBeTaken.isSelected());
      }
   }
}

class ObjectImmResVulnPanel extends JPanel
{
   JCheckBox[] immCheckBoxes = null;
   JCheckBox[] resCheckBoxes = null;
   JCheckBox[] vulnCheckBoxes = null;
   ButtonGroupMaybeNone[] buttonGroups = null;

   public ObjectImmResVulnPanel()
   {
      setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      int nbLines = AnatoliaConsts.immResVulnFlagTexts.length;
      immCheckBoxes  = new JCheckBox[nbLines];
      for(int i=0 ; i<nbLines ; immCheckBoxes[i++] = new JCheckBox());
      resCheckBoxes  = new JCheckBox[nbLines];
      for(int i=0 ; i<nbLines ; resCheckBoxes[i++] = new JCheckBox());
      vulnCheckBoxes = new JCheckBox[nbLines];
      for(int i=0 ; i<nbLines ; vulnCheckBoxes[i++] = new JCheckBox());
      buttonGroups = new ButtonGroupMaybeNone[nbLines];
      for(int i=0 ; i<nbLines ; buttonGroups[i++] = new ButtonGroupMaybeNone());

      constraint.gridx=0;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=20;constraint.weighty=100/nbLines;
      add(new JLabel("makes..."), constraint);
      constraint.gridx=1;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=100/nbLines;
      add(new JLabel(" Immune to "), constraint);
      constraint.gridx=2;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=100/nbLines;
      add(new JLabel(" Resistant to "), constraint);
      constraint.gridx=3;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=100/nbLines;
      add(new JLabel(" Vulnerable to"), constraint);

      for(int i=0 ; i<nbLines ; ++i)
      {
         constraint.gridx=0;constraint.gridy=i+1;
         constraint.gridwidth=1;constraint.gridheight=1;
         constraint.weightx=25;constraint.weighty=100/nbLines;
         add(new JLabel(AnatoliaConsts.immResVulnFlagTexts[i].getName()), constraint);
         constraint.gridx=1;constraint.gridy=i+1;
         constraint.gridwidth=1;constraint.gridheight=1;
         constraint.weightx=25;constraint.weighty=100/nbLines;
         add(immCheckBoxes[i], constraint);
         constraint.gridx=2;constraint.gridy=i+1;
         constraint.gridwidth=1;constraint.gridheight=1;
         constraint.weightx=25;constraint.weighty=100/nbLines;
         add(resCheckBoxes[i], constraint);
         constraint.gridx=3;constraint.gridy=i+1;
         constraint.gridwidth=1;constraint.gridheight=1;
         constraint.weightx=25;constraint.weighty=100/nbLines;
         add(vulnCheckBoxes[i], constraint);
         buttonGroups[i].add(immCheckBoxes[i]);
         buttonGroups[i].add(resCheckBoxes[i]);
         buttonGroups[i].add(vulnCheckBoxes[i]);
      }
  }
}

class AffectsPanel extends JPanel
{
   JCheckBox[] generalAffCheckBoxes = new JCheckBox[AnatoliaConsts.affTexts.length];
   JCheckBox[] detCheckBoxes = new JCheckBox[AnatoliaConsts.detTexts.length];
   ApplyPanel[] applyPanels = new ApplyPanel[AnatoliaConsts.applyTexts.length];

   public AffectsPanel()
   {
      setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      JPanel generalAffPanel = new JPanel();
      generalAffPanel.setLayout(new GridLayout(10,5));
      generalAffPanel.setBorder(new TitledBorder("General affects"));
      for(int i=0 ; i<generalAffCheckBoxes.length ; ++i)
      {
         generalAffCheckBoxes[i] = new JCheckBox(AnatoliaConsts.affTexts[i].getName());
         generalAffCheckBoxes[i].setToolTipText(AnatoliaConsts.affTexts[i].getToolTipText());
         generalAffPanel.add(generalAffCheckBoxes[i]);
      }
      constraint.gridx=0;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=50;constraint.weighty=50;
      add(generalAffPanel, constraint);

      JPanel detectPanel = new JPanel();
      detectPanel.setLayout(new GridLayout(10,2));
      detectPanel.setBorder(new TitledBorder("Detection affects"));
      for(int i=0 ; i<detCheckBoxes.length ; ++i)
      {
         detCheckBoxes[i] = new JCheckBox(AnatoliaConsts.detTexts[i].getName());
         detCheckBoxes[i].setToolTipText(AnatoliaConsts.detTexts[i].getToolTipText());
         detectPanel.add(detCheckBoxes[i]);
      }
      constraint.gridx=1;constraint.gridy=0;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=50;constraint.weighty=50;
      add(detectPanel, constraint);

      JPanel localAffPanel = new JPanel(new GridLayout(5,3));
      localAffPanel.setBorder(new TitledBorder("Local affects"));
      for(int i=0 ; i<AnatoliaConsts.applyTexts.length ; ++i)
      {
         applyPanels[i] = new ApplyPanel(AnatoliaConsts.applyTexts[i].getName());
         localAffPanel.add(applyPanels[i]);
      }
      constraint.gridx=0;constraint.gridy=1;
      constraint.gridwidth=2;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=50;
      add(localAffPanel, constraint);
   }
}

class ApplyPanel extends JPanel
{
   JTextField modifierTextField = new JTextField("0", 3);
   public ApplyPanel(String name)
   {
      JPanel modifierPanel = new JPanel(new GridLayout(2,1));
      modifierPanel.add(new JLabel(name + " modifier : "));
      modifierPanel.add(modifierTextField);
      add(modifierPanel);
   }
}

class ObjectSpecialsPanel extends JPanel implements ActionListener
{
   private static String[] limitedStrings =
      {"Not limited","1","2","3","4","5","6","7","8","9","10"};
   JComboBox   limitedBox = new JComboBox(limitedStrings);
   String[]    objectProgTexts = AnatoliaConsts.objectProgTexts;
   JComboBox   progComboBox = new JComboBox(objectProgTexts);
   String[]    progStrings = new String[AnatoliaConsts.objectProgTexts.length];
   JTextField  progTextField = new JTextField(12);

   int oldComboBoxIndex = 0;

   public ObjectSpecialsPanel()
   {
      setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      JPanel progPanel = new JPanel(new GridLayout(2,2));
      progPanel.add(new JLabel("Prog"));
      progPanel.add(new JLabel("function to call"));
      progComboBox.setToolTipText("In some situations (below), you can specify a function to call");
      progPanel.add(progComboBox);
      progTextField.setToolTipText("The name of a C function already coded in Anatolia");
      progPanel.add(progTextField);
      progComboBox.addActionListener(this);
      for(int i=0 ; i<progStrings.length ; ++i)
      {
        progStrings[i] = "";
      }
      constraint.gridx=0;constraint.gridy=1;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=20;
      add(progPanel, constraint);

      JPanel limitPanel = new JPanel(new GridLayout(2,1));
      limitPanel.add(new JLabel("Number of limitations"));
      limitPanel.add(limitedBox);
      constraint.gridx=0;constraint.gridy=2;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=20;
      add(limitPanel, constraint);

      constraint.gridx=0;constraint.gridy=3;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=60;
      add(new JPanel(), constraint);

      oldComboBoxIndex = progComboBox.getSelectedIndex();
  }

  public void actionPerformed(ActionEvent e)
  {
    if (e.getSource() == progComboBox)
    {
      updateProgStrings(false);
    }
  }

  public void updateProgStrings(boolean showingNewObject)
  {
    if (!showingNewObject) progStrings[oldComboBoxIndex] = progTextField.getText();
    /*for(int i=0 ; i<progComboBox.getItemCount() ; ++i)
    {
      String s = progComboBox.getItemAt(i).toString();
      if (!progStrings[i].trim().equals(""))
      {
        if (!s.startsWith("*")) s = "*"+s;
        objectProgTexts[i] = s;
        progComboBox = new JComboBox(objectProgTexts);
        progComboBox.addActionListener(this);
      }
      else if (s.trim().startsWith("*"))
      {
        s = s.substring(1,s.length());
        objectProgTexts[i] = s;
        progComboBox = new JComboBox(objectProgTexts);
        progComboBox.addActionListener(this);
      }
    }*/
    progTextField.setText(progStrings[progComboBox.getSelectedIndex()]);
    oldComboBoxIndex = progComboBox.getSelectedIndex();
  }

}