/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class AreaDialog extends JDialog implements ActionListener
{
  private AreaMap area = null;

  private JButton buttonOK = new JButton("OK");
  private JButton buttonCancel = new JButton("Cancel");
  JTextField nameTextField = new JTextField(15);
  JComboBox minLevelComboBox = new JComboBox(AnatoliaConsts.getMobileLevelsTexts());
  JComboBox maxLevelComboBox = new JComboBox(AnatoliaConsts.getMobileLevelsTexts());
  JTextField authorTextField = new JTextField(10);
  JComboBox numberComboBox = new JComboBox(AnatoliaConsts.getMobileLevelsTexts());
  JTextField resetMessageTextField = new JTextField(40);
  JTextArea  helpMessageTextArea   = new JTextArea(200,66);

  public AreaDialog()
  {
     Container panel = getContentPane();
     panel.setLayout(new GridBagLayout());
     GridBagConstraints constraint = new GridBagConstraints();

     JPanel namePanel = new JPanel(new GridLayout(2,1));
     namePanel.add(new JLabel("Name"));
     namePanel.add(nameTextField);
     namePanel.setToolTipText("Define the name of the area");
     nameTextField.setToolTipText("Define the name of the area");
     constraint.gridx=0;constraint.gridy=0;
     constraint.gridwidth=1;constraint.gridheight=1;
     constraint.weightx=50;constraint.weighty=10;
     panel.add(namePanel, constraint);

     JPanel authorPanel = new JPanel(new GridLayout(2,1));
     authorPanel.add(new JLabel("Author"));
     authorPanel.add(authorTextField);
     authorPanel.setToolTipText("Define the name of the author of the area");
     authorTextField.setToolTipText("Define the name of the author of the area");
     constraint.gridx=1;constraint.gridy=0;
     constraint.gridwidth=2;constraint.gridheight=1;
     constraint.weightx=50;constraint.weighty=10;
     panel.add(authorPanel, constraint);

     JPanel numberPanel = new JPanel(new GridLayout(2,1));
     numberPanel.add(new JLabel("Number"));
     numberPanel.add(numberComboBox);
     numberPanel.setToolTipText("If the number is xx, the VNUMS generated for this area will be within [xx000;xx999]");
     numberComboBox.setToolTipText("If the number is xx, the VNUMS generated for this area will be within [xx000;xx999]");
     constraint.gridx=0;constraint.gridy=1;
     constraint.gridwidth=1;constraint.gridheight=1;
     constraint.weightx=33;constraint.weighty=10;
     panel.add(numberPanel, constraint);

     JPanel minLevelPanel = new JPanel(new GridLayout(2,1));
     minLevelPanel.add(new JLabel("Level min."));
     minLevelPanel.add(minLevelComboBox);
     minLevelPanel.setToolTipText("Recommended minimum char level to go to this area");
     minLevelComboBox.setToolTipText("Recommended minimum char level to go to this area");
     constraint.gridx=1;constraint.gridy=1;
     constraint.gridwidth=1;constraint.gridheight=1;
     constraint.weightx=33;constraint.weighty=10;
     panel.add(minLevelPanel, constraint);

     JPanel maxLevelPanel = new JPanel(new GridLayout(2,1));
     maxLevelPanel.add(new JLabel("Level max."));
     maxLevelPanel.add(maxLevelComboBox);
     maxLevelPanel.setToolTipText("Recommended maximum char level to go to this area");
     maxLevelComboBox.setToolTipText("Recommended maximum char level to go to this area");
     constraint.gridx=2;constraint.gridy=1;
     constraint.gridwidth=1;constraint.gridheight=1;
     constraint.weightx=33;constraint.weighty=10;
     panel.add(maxLevelPanel, constraint);

     JPanel resetMessagePanel = new JPanel(new GridLayout(2,1));
     resetMessagePanel.add(new JLabel("Reset Message"));
     resetMessagePanel.add(resetMessageTextField);
     resetMessagePanel.setToolTipText("Message sent to players when the area is reset");
     resetMessageTextField.setToolTipText("Message sent to players when the area is reset");
     constraint.gridx=0;constraint.gridy=2;
     constraint.gridwidth=3;constraint.gridheight=1;
     constraint.weightx=100;constraint.weighty=10;
     panel.add(resetMessagePanel, constraint);

     JPanel helpMessagePanel = new JPanel(new BorderLayout());
     helpMessagePanel.add(new JLabel("Help Message"), BorderLayout.NORTH);
     JScrollPane helpMessageScrollPane = new JScrollPane(helpMessageTextArea,
                                                         JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                                                         JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
     helpMessageScrollPane.setPreferredSize(new Dimension(400,250));
     helpMessagePanel.add(helpMessageScrollPane,BorderLayout.CENTER);
     helpMessagePanel.setToolTipText("HELPS section of the file. Beware of the \"official\" syntax!");
     helpMessageTextArea.setToolTipText("HELPS section of the file. Beware of the \"official\" syntax!");
     constraint.gridx=0;constraint.gridy=3;
     constraint.gridwidth=3;constraint.gridheight=1;
     constraint.weightx=100;constraint.weighty=60;
     panel.add(helpMessagePanel, constraint);

     JPanel okCancelPanel = new JPanel(new GridLayout(1,2));
     okCancelPanel.add(buttonOK);
     okCancelPanel.add(buttonCancel);
     constraint.gridx=0;constraint.gridy=4;
     constraint.gridwidth=1;constraint.gridheight=1;
     constraint.weightx=100;constraint.weighty=10;
     panel.add(okCancelPanel, constraint);

     pack();

     buttonOK.addActionListener(this);
     buttonCancel.addActionListener(this);
  }

  public void actionPerformed(ActionEvent e)
  {
     if (e.getSource() == buttonOK)
     {
        apply();
        setVisible(false);
     }
     else if (e.getSource() == buttonCancel)
     {
        setVisible(false);
     }
  }

  public void showFor(AreaMap area)
  {
     setModal(true);
     this.area = area;

     nameTextField.setText(area.areaName);
     authorTextField.setText(area.author);
     minLevelComboBox.setSelectedIndex(area.minLevel-AnatoliaConsts.LEVEL_MIN);
     maxLevelComboBox.setSelectedIndex(area.maxLevel-AnatoliaConsts.LEVEL_MIN);
     numberComboBox.setSelectedIndex(area.areaNumber);
     resetMessageTextField.setText(area.resetMessage);
     helpMessageTextArea.setText(area.helpMessage);

     Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
     Dimension mySize = this.getSize();
     this.setLocation((int) (screenSize.getWidth()-mySize.getWidth())/2,
                       (int) (screenSize.getHeight()-mySize.getHeight())/2
                     );
     this.pack();
     setVisible(true);
  }

  public void apply()
  {
     if (area == null) return;

     area.areaName = nameTextField.getText();
     area.author = authorTextField.getText();
     area.minLevel = minLevelComboBox.getSelectedIndex()+AnatoliaConsts.LEVEL_MIN;
     area.maxLevel = maxLevelComboBox.getSelectedIndex()+AnatoliaConsts.LEVEL_MIN;
     area.setAreaNumber(numberComboBox.getSelectedIndex());
     area.resetMessage = resetMessageTextField.getText();
     area.helpMessage  = helpMessageTextArea.getText();
     setVisible(false);
  }
}