/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class ExternDialog extends JDialog implements ActionListener
{
  public static final byte WELLKNOWNMOBS = 0;
  public static final byte WELLKNOWNOBJS = 1;

  JComboBox wellKnownComboBox = null;

  JPanel wellKnownPanel = new JPanel();
  JPanel  explicitPanel = new JPanel();

  JTextField vnumTextField = new JTextField(5);

  JRadioButton wellKnownRadioButton = new JRadioButton("Well known");
  JRadioButton explicitRadioButton = new JRadioButton("Explicit vnum");
  ButtonGroup buttonGroup = new ButtonGroup();

  JButton okButton = new JButton("OK");
  JButton cancelButton = new JButton("Cancel");
  JPanel okCancelPanel = new JPanel();

  int option=0;
  String externName = "";

  public ExternDialog(Dialog owner, String title, boolean modal, byte whichWellKnown)
  {
    super(owner, title, modal);
    if (whichWellKnown == WELLKNOWNMOBS)
      wellKnownComboBox = new JComboBox(AnatoliaConsts.getWellKnownMobs());
    else
      wellKnownComboBox = new JComboBox(AnatoliaConsts.getWellKnownObjs());

    buttonGroup.add(wellKnownRadioButton);
    buttonGroup.add(explicitRadioButton);
    wellKnownRadioButton.setSelected(true);
     explicitRadioButton.setSelected(false);
    wellKnownRadioButton.addActionListener(this);
     explicitRadioButton.addActionListener(this);

    wellKnownPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    wellKnownPanel.add(wellKnownRadioButton);
    wellKnownPanel.add(wellKnownComboBox);
    explicitPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    explicitPanel.add(explicitRadioButton);
    explicitPanel.add(vnumTextField);

    Container contentPane = getContentPane();
    contentPane.setLayout(new GridLayout(3,1));
    contentPane.add(wellKnownPanel);
    contentPane.add(explicitPanel);

    okCancelPanel.setLayout(new GridLayout(1,2));
    okButton.addActionListener(this);
    okCancelPanel.add(okButton);
    cancelButton.addActionListener(this);
    okCancelPanel.add(cancelButton);
    contentPane.add(okCancelPanel);

    updateButtons();
    pack();
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension mySize = this.getSize();
    this.setLocation((int) (screenSize.getWidth()-mySize.getWidth())/2,
                     (int) (screenSize.getHeight()-mySize.getHeight())/2
                    );
  }

  public void actionPerformed(ActionEvent e)
  {
    if (   (e.getSource() == wellKnownRadioButton)
         ||(e.getSource() ==  explicitRadioButton)
        )
    {
      updateButtons();
    }
    else if (e.getSource() == okButton)
    {
      option = JOptionPane.OK_OPTION;
      setVisible(false);
    }
    else if (e.getSource() == cancelButton)
    {
      option = JOptionPane.CANCEL_OPTION;
      setVisible(false);
    }
  }

  public int open()
  {
     updateButtons();
     setVisible(true);
     return option;
  }

  private void updateButtons()
  {
     wellKnownComboBox.setEnabled(wellKnownRadioButton.isSelected());
      vnumTextField.setEnabled(explicitRadioButton.isSelected());
  }

  public int getVnum()
  {
    int vnum = -1;
    if (wellKnownRadioButton.isSelected())
    {
      Object obj = wellKnownComboBox.getSelectedItem();
      try
      {
        vnum = ((WellKnownMob) obj).vnum;
        externName = ((WellKnownMob) obj).label;
      }
      catch (Exception e) {}
      try
      {
        vnum = ((WellKnownObj) obj).vnum;
        externName = ((WellKnownObj) obj).label;
      }
      catch (Exception e) {}
    }
    else
    {
      try
      {
        vnum = Integer.parseInt(vnumTextField.getText());
      }
      catch (Exception e) {}
    }
    return vnum;
  }
}
