/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import javax.swing.*;
import java.net.URL;
import java.awt.Toolkit;

public class Icones
{
  static ImageIcon newIcon        = null;
  static ImageIcon openIcon       = null;
  static ImageIcon saveIcon       = null;
  static ImageIcon cutIcon        = null;
  static ImageIcon copyIcon       = null;
  static ImageIcon pasteIcon      = null;
  static ImageIcon propertiesIcon = null;
  static ImageIcon cursorIcon     = null;
  static ImageIcon roomIcon       = null;
  static ImageIcon wireIcon       = null;
  static ImageIcon zoomIcon       = null;
  static ImageIcon buildIcon      = null;
  static ImageIcon chachaIcon     = null;

  static String[] objectsTreeIconNames =
     {"weapon","armor","container","light","food","drink","money","wand","staff","potion",
      "scroll","pill","furniture","portal","room_key","clothing","fountain","key","boat","map",
      "warp_stone","treasure","jewelry","gem","trash","tattoo","bomb","tattoo", "jukebox",
      "externObject"};
  static String[] mobilesTreeIconNames =
     {"mobile","externMobile"};
  static String[] roomsTreeIconNames =
     {"inside","city","field","forest","hill","mountain","water","deepwater",null,
      "air","desert","externRoom"};  //null value is a reserved field! Do not remove it!

  static ImageIcon[] objectsTreeIcons = new ImageIcon[objectsTreeIconNames.length];
  static ImageIcon[] mobilesTreeIcons = new ImageIcon[mobilesTreeIconNames.length];
  static ImageIcon[] roomsTreeIcons   = new ImageIcon[roomsTreeIconNames.length];

  public Icones()
  {
     URL url = null;

     url = Icones.class.getResource("Resources/ToolBars/new.gif");
     newIcon        = new ImageIcon(Toolkit.getDefaultToolkit().getImage(url));
     url = Icones.class.getResource("Resources/ToolBars/open.gif");
     openIcon       = new ImageIcon(Toolkit.getDefaultToolkit().getImage(url));
     url = Icones.class.getResource("Resources/ToolBars/save.gif");
     saveIcon       = new ImageIcon(Toolkit.getDefaultToolkit().getImage(url));
     url = Icones.class.getResource("Resources/ToolBars/cut.gif");
     cutIcon        = new ImageIcon(Toolkit.getDefaultToolkit().getImage(url));
     url = Icones.class.getResource("Resources/ToolBars/copy.gif");
     copyIcon       = new ImageIcon(Toolkit.getDefaultToolkit().getImage(url));
     url = Icones.class.getResource("Resources/ToolBars/paste.gif");
     pasteIcon      = new ImageIcon(Toolkit.getDefaultToolkit().getImage(url));
     url = Icones.class.getResource("Resources/ToolBars/properties.gif");
     propertiesIcon = new ImageIcon(Toolkit.getDefaultToolkit().getImage(url));
     url = Icones.class.getResource("Resources/ToolBars/cursor.gif");
     cursorIcon     = new ImageIcon(Toolkit.getDefaultToolkit().getImage(url));
     url = Icones.class.getResource("Resources/ToolBars/room.gif");
     roomIcon       = new ImageIcon(Toolkit.getDefaultToolkit().getImage(url));
     url = Icones.class.getResource("Resources/ToolBars/wire.gif");
     wireIcon       = new ImageIcon(Toolkit.getDefaultToolkit().getImage(url));
     url = Icones.class.getResource("Resources/ToolBars/zoom.gif");
     zoomIcon       = new ImageIcon(Toolkit.getDefaultToolkit().getImage(url));
     url = Icones.class.getResource("Resources/ToolBars/build.gif");
     buildIcon      = new ImageIcon(Toolkit.getDefaultToolkit().getImage(url));
     url = Icones.class.getResource("Resources/About/logo.gif");
     chachaIcon     = new ImageIcon(Toolkit.getDefaultToolkit().getImage(url));

     for(int i=0 ; i<objectsTreeIcons.length ; ++i)
     {
        url = Icones.class.getResource("Resources/Tree/Objects/"+objectsTreeIconNames[i]+".gif");
        //objectsTreeIcons[i] = new ImageIcon("areabuilder/Resources/Tree/Objects/"+objectsTreeIconNames[i]+".gif");
        objectsTreeIcons[i] = new ImageIcon(Toolkit.getDefaultToolkit().getImage(url));
     }
     for(int i=0 ; i<mobilesTreeIcons.length ; ++i)
     {
        url = Icones.class.getResource("Resources/Tree/Mobiles/"+mobilesTreeIconNames[i]+".gif");
        mobilesTreeIcons[i] = new ImageIcon(Toolkit.getDefaultToolkit().getImage(url));
        //mobilesTreeIcons[i] = new ImageIcon("areabuilder/Resources/Tree/Mobiles/"+mobilesTreeIconNames[i]+".gif");
     }
     for(int i=0 ; i<roomsTreeIcons.length ; ++i)
     {
        url = Icones.class.getResource("Resources/Tree/Rooms/"+roomsTreeIconNames[i]+".gif");
        if (url != null)
        {
          roomsTreeIcons[i] = new ImageIcon(Toolkit.getDefaultToolkit().getImage(url));
          //roomsTreeIcons[i] = new ImageIcon("areabuilder/Resources/Tree/Rooms/"+roomsTreeIconNames[i]+".gif");
        }
     }
  }

  public static ImageIcon getIcon(AnatoliaThing anatoliaThing)
  {
    try
    {
      AnatoliaRoom room = (AnatoliaRoom) anatoliaThing;
      return roomsTreeIcons[room.sectorType];
    }
    catch(Exception e) {}
    try
    {
      AnatoliaMobile mobile = (AnatoliaMobile) anatoliaThing;
      return mobilesTreeIcons[0];
    }
    catch(Exception e) {}
    try
    {
      AnatoliaObject object = (AnatoliaObject) anatoliaThing;
      int index;
      if (object.isExtern())
      {
        index = AreaBuilderConsts.findInTab(objectsTreeIconNames, "externObject");
      }
      else
      {
        String type = AnatoliaConsts.objectTypeTexts[object.type].getName();
        index = AreaBuilderConsts.findInTab(objectsTreeIconNames, type);
      }
      return objectsTreeIcons[index];
    }
    catch(Exception e) {}
    return null;
  }
}