/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;
import java.util.*; //for <HashMap> and <Set>

abstract public class LinkingAndSelectableNodeObject extends SelectableNodeObject
{
  static private long firstFreeObjectID = 0; //used by areaBuilder, not for Anatolia.
  static private Map mapOfID = new HashMap(); //<#ID, @object>

  private long objectID;

  public LinkingAndSelectableNodeObject()
  {
     objectID = ++firstFreeObjectID;
  }

  public long getObjectID() {return objectID;}
  public void setObjectID(long newObjectID)
  {
     this.objectID = newObjectID;
     mapOfID.put(new Long(newObjectID), this);
  }

  static public void staticClear()
  {
     firstFreeObjectID = 0;
     mapOfID.clear();
  }

  static Object lookForObjectOfID(long objectID)
  {
     return mapOfID.get(new Long(objectID));
  }
  static Object lookForObjectOfID(Long objectID)
  {
     return mapOfID.get(objectID);
  }

  //renumbers the IDs to remove "holes"
  static public void defragIDs()
  {
     Map tempMapOfID = new HashMap(); //we'll use a temporary table
     Set entriesOfMapOfID = mapOfID.entrySet();
     Iterator it = entriesOfMapOfID.iterator();
     long number = 0;
     LinkingAndSelectableNodeObject object = null;
     Map.Entry entry = null;
     while (it.hasNext())
     {
        //!can be optimized
        entry = (Map.Entry) it.next();        //<  #ID  , @object>
        object = (LinkingAndSelectableNodeObject) entry.getValue();
        object.objectID = ++number;
        tempMapOfID.put(new Long(number), object); //<new #ID, @object>
     }
     //overwrites old table
     firstFreeObjectID = number+1;
     mapOfID = tempMapOfID;
  }

  //redefine links, should use <this.lookForObjectOfID(...)>
  abstract public void establishLinks();
}