/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import javax.swing.event.*;
import java.io.*;

public class MainScreen extends JFrame implements ActionListener, WindowListener
{
    public static final double version = 0.9;

    Icones icones = new Icones();
    UndoableActionStack actionStack = new UndoableActionStack();
    AreaDialog areaDialog = null;

    PointerState pointerState = null;
    Container contentPane = getContentPane();
      JPanel panneauSuperieur = new JPanel(new GridLayout(3,1));
      JMenuBar menus = new JMenuBar();
        JMenu menuFile = new JMenu("File");
          JMenuItem menuFileNew    = new JMenuItem("New");
          JMenuItem menuFileOpen   = new JMenuItem("Open");
          JMenuItem menuFileSave   = new JMenuItem("Save");
          JMenuItem menuFileSaveAs = new JMenuItem("Save as...");
          JMenuItem menuFileAreaProperties = new JMenuItem("Area properties");
          JMenuItem menuFileBuildArea = new JMenuItem("Build Area");
          JMenuItem menuFileQuit   = new JMenuItem("Quit");
        JMenu menuEdition = new JMenu("Edition");
          JMenuItem menuEditionUndo = new JMenuItem("Undo");
          JMenuItem menuEditionRedo = new JMenuItem("Redo");
          JMenuItem menuEditionCopy  = new JMenuItem("Copy");
          JMenuItem menuEditionCut  = new JMenuItem("Cut");
          JMenuItem menuEditionPaste = new JMenuItem("Paste");
          JMenuItem menuEditionProperties = new JMenuItem("Properties");
        JMenu menuSearch = new JMenu("Search");
          JMenuItem menuSearchFind = new JMenuItem("Find");
        JMenu menuDisplay = new JMenu("Display");
           JCheckBoxMenuItem menuDisplayMagneticGrid = new JCheckBoxMenuItem("Magnetic Grid", true);
           JCheckBoxMenuItem menuDisplayTree = new JCheckBoxMenuItem("Object tree");
           JCheckBoxMenuItem menuDisplayLayers = new JCheckBoxMenuItem("Showing layers...");
        JMenu menuLookAndFeel = new JMenu("Look&Feel");
        JMenu menuAbout = new JMenu("?");
          JMenuItem menuAboutAbout = new JMenuItem("A propos de...");
      JToolBar classicToolBar = new JToolBar();
        JButton newButton = new JButton(Icones.newIcon);
        JButton openButton = new JButton(Icones.openIcon);
        JButton saveButton = new JButton(Icones.saveIcon);
        JButton copyButton = new JButton(Icones.copyIcon);
        JButton cutButton = new JButton(Icones.cutIcon);
        JButton pasteButton = new JButton(Icones.pasteIcon);
        JButton propertiesButton = new JButton(Icones.propertiesIcon);
        JButton buildButton = new JButton(Icones.buildIcon);
      JToolBar toolBar = new JToolBar();
        JToggleButton pointerButton   = new JToggleButton(Icones.cursorIcon, true);
        JToggleButton zoomButton      = new JToggleButton(Icones.zoomIcon  , false);
        JToggleButton roomButton      = new JToggleButton(Icones.roomIcon  , false);
        JToggleButton wireButton      = new JToggleButton(Icones.wireIcon  , false);
      ButtonGroup groupToolBar = new ButtonGroup();

      UIManager.LookAndFeelInfo[] lookAndFeels;
      JCheckBoxMenuItem[] menuLookAndFeelChoices;
      ButtonGroup lookAndFeelButtonsGroup = new ButtonGroup();

      JScrollPane mapZone = null;
      AreaMap areaMap = null;
      boolean mustSave = false;
      String areaFileName = "";

      File lastSaveDir = null;
      File lastOpenDir = null;

  public MainScreen(String title)
  {
    pointerState = new PointerState(this.getCursor());

    areaMap = new AreaMap(this);

    areaDialog = new AreaDialog();

    areaMap.setPointerStateCursor(pointerState);

    setTitle(title);
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);

    //construction de l'interface
    contentPane.setLayout(new BorderLayout());
    contentPane.add(panneauSuperieur,BorderLayout.NORTH);
      panneauSuperieur.add(menus,BorderLayout.NORTH);
          menus.add(menuFile);
            menuFile.add(menuFileNew);
              menuFileNew.setAccelerator(KeyStroke.getKeyStroke(
                                             KeyEvent.VK_N,
                                             InputEvent.CTRL_MASK));
            menuFile.addSeparator();
            menuFile.add(menuFileOpen);
              menuFileOpen.setAccelerator(KeyStroke.getKeyStroke(
                                             KeyEvent.VK_O,
                                             InputEvent.CTRL_MASK));
            menuFile.add(menuFileSave);
              menuFileSave.setAccelerator(KeyStroke.getKeyStroke(
                                             KeyEvent.VK_S,
                                             InputEvent.CTRL_MASK));
            menuFile.add(menuFileSaveAs);
              menuFileSaveAs.setAccelerator(KeyStroke.getKeyStroke(
                                             KeyEvent.VK_S,
                                             InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK));
            menuFile.addSeparator();
            menuFile.add(menuFileAreaProperties);
            menuFile.addSeparator();
            menuFile.add(menuFileBuildArea);
            menuFile.addSeparator();
            menuFile.add(menuFileQuit);
              menuFileQuit.setAccelerator(KeyStroke.getKeyStroke(
                                             KeyEvent.VK_Q,
                                             InputEvent.CTRL_MASK));
          menus.add(menuEdition);
            menuEdition.add(menuEditionUndo);
            menuEditionUndo.setAccelerator(KeyStroke.getKeyStroke(
                                           KeyEvent.VK_Z,
                                           InputEvent.CTRL_MASK));
            menuEditionUndo.setEnabled(false);
            menuEdition.add(menuEditionRedo);
            menuEditionRedo.setAccelerator(
                KeyStroke.getKeyStroke(KeyEvent.VK_Z,
                                       InputEvent.CTRL_MASK |
                                       InputEvent.SHIFT_MASK));
            menuEditionRedo.setEnabled(false);
            menuEdition.addSeparator();
            menuEdition.add(menuEditionCopy);
            menuEditionCopy.setAccelerator(
                KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK));
            menuEdition.add(menuEditionCut);
            menuEditionCut.setAccelerator(
                KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK));
            menuEdition.add(menuEditionPaste);
            menuEditionPaste.setAccelerator(
                KeyStroke.getKeyStroke(KeyEvent.VK_V, InputEvent.CTRL_MASK));
            menuEdition.addSeparator();
            menuEdition.add(menuEditionProperties);
            menuEditionProperties.setEnabled(false);
            menuEditionProperties.setAccelerator(
                KeyStroke.getKeyStroke(KeyEvent.VK_P, InputEvent.CTRL_MASK));

          menus.add(menuSearch);
            menuSearch.add(menuSearchFind);
            menuSearchFind.setAccelerator(
                KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.CTRL_MASK));
          menus.add(menuDisplay);
            menuDisplay.add(menuDisplayTree);
            menuDisplayTree.setAccelerator(
                KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_MASK));
            menuDisplay.add(menuDisplayLayers);
            menuDisplayLayers.setAccelerator(
                KeyStroke.getKeyStroke(KeyEvent.VK_L, InputEvent.CTRL_MASK));
            menuDisplay.add(menuDisplayMagneticGrid);
            menuDisplayMagneticGrid.setAccelerator(
                KeyStroke.getKeyStroke(KeyEvent.VK_M, InputEvent.CTRL_MASK));
          menus.add(menuLookAndFeel);
          lookAndFeels = UIManager.getInstalledLookAndFeels();
          menuLookAndFeelChoices = new JCheckBoxMenuItem[lookAndFeels.length];
          if (lookAndFeels.length == 0)
          {
            menuLookAndFeel.setEnabled(false);
          }
          else
          {
            for(int i=0 ; i<lookAndFeels.length ; ++i)
            {
              menuLookAndFeelChoices[i] = new JCheckBoxMenuItem(lookAndFeels[i].getName());
              if (UIManager.getLookAndFeel().getName() == lookAndFeels[i].getName())
              {
                menuLookAndFeelChoices[i].setSelected(true);
              }
              menuLookAndFeel.add(menuLookAndFeelChoices[i]);
              menuLookAndFeelChoices[i].addActionListener(this);
              lookAndFeelButtonsGroup.add(menuLookAndFeelChoices[i]);
            }
          }
          menus.add(menuAbout);
            menuAbout.add(menuAboutAbout);
      panneauSuperieur.add(classicToolBar);
        classicToolBar.add(newButton);
          newButton.setToolTipText("New area");
        classicToolBar.add(openButton);
          openButton.setToolTipText("Open existing area");
        classicToolBar.add(saveButton);
          saveButton.setToolTipText("Save current area");
        classicToolBar.add(new JToolBar.Separator());
        classicToolBar.add(copyButton);
          copyButton.setToolTipText("Copy current selection");
        classicToolBar.add(cutButton);
          cutButton.setToolTipText("Cut current selection");
        classicToolBar.add(pasteButton);
          pasteButton.setToolTipText("Paste current clipboard");
        classicToolBar.add(new JToolBar.Separator());
        classicToolBar.add(propertiesButton);
          propertiesButton.setToolTipText("Display current selection properties");
        classicToolBar.add(new JToolBar.Separator());
        classicToolBar.add(buildButton);
          buildButton.setToolTipText("Build .are file");
        classicToolBar.setFloatable(false);

     JPanel mapZonePanel = new JPanel(new BorderLayout());
       mapZone = new JScrollPane(areaMap);
       areaMap.setSize(mapZone.getSize());
       mapZonePanel.add(mapZone, BorderLayout.CENTER);

       groupToolBar.add(pointerButton);
       groupToolBar.add(zoomButton);
       groupToolBar.add(roomButton);
       groupToolBar.add(wireButton);
       Box box = Box.createVerticalBox();
       toolBar.add(pointerButton);
         pointerButton.setToolTipText("Normal cursor for moving and selecting");
       toolBar.add(zoomButton);
         zoomButton.setToolTipText("Zooming in, out, and in a window");
       toolBar.add(roomButton);
         roomButton.setToolTipText("Building rooms");
       toolBar.add(wireButton);
         wireButton.setToolTipText("Connecting rooms");
       toolBar.add(Box.createGlue());
       toolBar.setFloatable(true);
       mapZonePanel.add(toolBar, BorderLayout.WEST);
       toolBar.setOrientation(toolBar.VERTICAL);
    contentPane.add(mapZonePanel,BorderLayout.CENTER);

    pointerButton.addActionListener(this);
    roomButton.addActionListener(this);
    zoomButton.addActionListener(this);
    wireButton.addActionListener(this);
    menuFileNew.addActionListener(this);
    menuFileOpen.addActionListener(this);
    menuFileSave.addActionListener(this);
    menuFileSaveAs.addActionListener(this);
    menuFileAreaProperties.addActionListener(this);
    menuFileBuildArea.addActionListener(this);
    menuFileQuit.addActionListener(this);
    menuEditionUndo.addActionListener(this);
    menuEditionRedo.addActionListener(this);
    menuEditionCopy.addActionListener(this);
    menuEditionCut.addActionListener(this);
    menuEditionPaste.addActionListener(this);
    menuEditionProperties.addActionListener(this);
    menuSearchFind.addActionListener(this);
    menuDisplayMagneticGrid.addActionListener(this);
    menuDisplayTree.addActionListener(this);
    menuDisplayLayers.addActionListener(this);
    menuAboutAbout.addActionListener(this);
    newButton.addActionListener(this);
    openButton.addActionListener(this);
    saveButton.addActionListener(this);
    copyButton.addActionListener(this);
    cutButton.addActionListener(this);
    pasteButton.addActionListener(this);
    propertiesButton.addActionListener(this);
    buildButton.addActionListener(this);

    addWindowListener(this);
 }

 public void windowClosing(WindowEvent e) {quitProgram();}
 public void windowClosed(WindowEvent e)      {}
 public void windowOpened(WindowEvent e)      {}
 public void windowDeactivated(WindowEvent e) {}
 public void windowActivated(WindowEvent e)   {}
 public void windowDeiconified(WindowEvent e) {}
 public void windowIconified(WindowEvent e)   {}

  public void actionPerformed(ActionEvent e)
  {
      if (e.getSource() == pointerButton)
        pointerState.setPointerState(PointerState.POINTERSTATE);
      else if (e.getSource() == roomButton)
        pointerState.setPointerState(PointerState.ROOMSTATE);
      else if (e.getSource() == zoomButton)
        pointerState.setPointerState(PointerState.ZOOMSTATE);
      else if (e.getSource() == wireButton)
        pointerState.setPointerState(PointerState.WIRESTATE);
      else if ((e.getSource() == menuFileNew) || (e.getSource() == newButton))
      {
        if (askForSaving()) newAreaMap();
      }
      else if ((e.getSource() == menuFileOpen) ||  (e.getSource() == openButton))
      {
        if (askForSaving()) openAreaMap();
      }
      else if ((e.getSource() == menuFileSave) || (e.getSource() == saveButton))
        saveAreaMap();
      else if (e.getSource() == menuFileSaveAs)
        saveAsAreaMap();
      else if ((e.getSource() == menuFileAreaProperties))
        selectAreaNumber();
      else if ((e.getSource() == menuFileBuildArea) || (e.getSource() == buildButton))
        buildArea();
      else if (e.getSource() == menuFileQuit)
        quitProgram();
      else if (e.getSource() == menuEditionUndo)
      {
        setMustSave(true);
        actionStack.undoAction();
        setUndoState(!actionStack.undoStack.isEmpty());
        setRedoState(!actionStack.redoStack.isEmpty());
        areaMap.repaint();
      }
      else if (e.getSource() == menuEditionRedo)
      {
        setMustSave(true);
        actionStack.redoAction();
        setUndoState(!actionStack.undoStack.isEmpty());
        setRedoState(!actionStack.redoStack.isEmpty());
        areaMap.repaint();
      }
      else if (e.getSource() == menuEditionCopy)
      {
        setMustSave(true);
        areaMap.act_copy();
      }
      else if (e.getSource() == menuEditionCut)
      {
        setMustSave(true);
        areaMap.act_cut();
      }
      else if (e.getSource() == menuEditionPaste)
      {
        setMustSave(true);
        areaMap.act_paste();
      }
      else if ((e.getSource() == menuEditionProperties) || (e.getSource() == propertiesButton))
        areaMap.anatoliaRoomDialog.showFor(areaMap.selectedRooms);
      else if (e.getSource() == menuSearchFind)
      {
        areaMap.findReplaceStringDialog.setVisible(true);
      }
      else if (e.getSource() == menuDisplayMagneticGrid)
      {
        areaMap.setMagneticGrid(menuDisplayMagneticGrid.isSelected());
        if (areaMap.getMagneticGrid()) areaMap.magneticRooms();
      }
      else if (e.getSource() == menuDisplayTree)
      {
        areaMap.objectTreeDialog.setVisible(((JCheckBoxMenuItem) e.getSource()).isSelected());
        areaMap.objectTreeDialog.enableMenus();
      }
      else if (e.getSource() == menuDisplayLayers)
      {
        areaMap.layersDialog.update();
        areaMap.layersDialog.setVisible(((JCheckBoxMenuItem) e.getSource()).isSelected());
      }
      else if (e.getSource() == menuAboutAbout)
      {
         JOptionPane.showMessageDialog(this, "AreaBuilder v"+version+" BETA"+"\n"+
                                        "Brought to you by Pierre Chatelier"+"\n"+
                                        "ktd@club-internet.fr",
                                       "About AreaBuilder",JOptionPane.PLAIN_MESSAGE, Icones.chachaIcon);
      }
      else
      {
         for(int i=0 ; i<menuLookAndFeelChoices.length ; ++i)
         {
           if (e.getSource() == menuLookAndFeelChoices[i])
           {
             try
             {
               UIManager.setLookAndFeel(lookAndFeels[i].getClassName());
               SwingUtilities.updateComponentTreeUI(this);
               SwingUtilities.updateComponentTreeUI(this.areaMap.objectTreeDialog);
               SwingUtilities.updateComponentTreeUI(this.areaMap.findReplaceStringDialog);
               SwingUtilities.updateComponentTreeUI(this.areaMap.anatoliaMobileDialog);
               SwingUtilities.updateComponentTreeUI(this.areaMap.anatoliaObjectDialog);
               SwingUtilities.updateComponentTreeUI(this.areaMap.anatoliaRoomDialog);
             }
             catch(Exception ex)
             {
             }
           }
         }
      }
  }

  public void setUndoState(boolean undoState) {menuEditionUndo.setEnabled(undoState);}
  public void setRedoState(boolean redoState) {menuEditionRedo.setEnabled(redoState);}
  public void setMustSave(boolean mustSave)
  {
     //if (this.mustSave == mustSave) return;
     if (mustSave)
     {
        setTitle("AreaBuilder ["+areaFileName+"*]");
        menuFileSave.setText("Save (needed)");
     }
     else
     {
        setTitle("AreaBuilder ["+areaFileName+"]");
        menuFileSave.setText("Save");
     }

     this.mustSave = mustSave;
  }

  public void newAreaMap()
  {
     actionStack.clear(); //free the stack
     areaMap.clear();
     actionStack.clear(); //we redo it because freeing map could have pushed some elements
     this.areaFileName = "";
     setMustSave(false);
     areaMap.repaint();
     System.gc(); //it's a good occasion to call garbage collector
  }

  public void openAreaMap()
  {
     JFileChooser chooser = null;
     if (lastOpenDir != null)
     {
       chooser = new JFileChooser(lastOpenDir);
     }
     else
     {
       chooser = new JFileChooser(".");
     }
     chooser.setFileFilter(new AreaBuilderAreFileFilter());
     chooser.addChoosableFileFilter(new AreaBuilderProjectFileFilter());
     int returnVal = chooser.showOpenDialog(this);
     if(returnVal == JFileChooser.APPROVE_OPTION)
     {
       lastOpenDir = chooser.getCurrentDirectory();
       String tempAreaName = chooser.getSelectedFile().getPath();
       AreaMap tempAreaMap = null;
       if (tempAreaName.toLowerCase().endsWith(".abp"))
       {
         try
         {
            ObjectInputStream input = new ObjectInputStream(new FileInputStream(tempAreaName));
            tempAreaMap = new AreaMap(this);
            tempAreaMap.version = input.readUTF(); //version number
            tempAreaMap.readObject(input);
            input.close();
         }
         catch(IOException exception)
         {
            JOptionPane.showMessageDialog(this, "Loading failed",
                                                "Input error",
                                                JOptionPane.ERROR_MESSAGE);
            System.gc();
            return;
         }
         catch(ClassNotFoundException exception)
         {
            JOptionPane.showMessageDialog(this, "Internal error : class not found",
                                               "Class not found",
                                               JOptionPane.ERROR_MESSAGE);
            System.gc();
            return;
          }
        }
        else if (tempAreaName.toLowerCase().endsWith(".are"))
        {
            tempAreaMap = new AreaMap(this);
            AnatoliaAreFileReader reader = new AnatoliaAreFileReader(tempAreaMap);
            try
            {
              reader.load(tempAreaName);
            }
            catch(IOException exception)
            {
               JOptionPane.showMessageDialog(this, "Loading failed",
                                                   "Input error",
                                                   JOptionPane.ERROR_MESSAGE);
               System.gc();
               return;
            }
            catch(Exception e)
            {
               JOptionPane.showMessageDialog(this, "Unexpected error",
                                                   "Unexpected error",
                                                   JOptionPane.ERROR_MESSAGE);
               System.gc();
              return;
            }
        }
        else
        {
           JOptionPane.showConfirmDialog(this, "Unknown extension", "Error", JOptionPane.ERROR_MESSAGE);
           return;
        }
        boolean b = areaMap.objectTreeDialog.isVisible();
        areaMap.objectTreeDialog.setVisible(false);
        areaMap = tempAreaMap;
        tempAreaMap.objectTreeDialog.setVisible(b);
        areaMap.validateSize();
        mapZone.setViewportView(areaMap);
        areaMap.setPointerStateCursor(pointerState);

        areaMap.layersDialog.update();
        areaMap.layersDialog.enableAllLayers();

        areaFileName = tempAreaName;
        if (!areaFileName.trim().toLowerCase().endsWith(".abp"))
        {
          areaFileName += ".abp";
        }
        setTitle("AreaBuilder ["+areaFileName+"]");
        setMustSave(false);
        actionStack.clear();
        areaMap.theRooms.resetMinMax();
        areaMap.fitScreen();
        areaMap.repaint();
        repaint();
        System.gc();
     }
  }

  //return true for continue, false if must cancel
  public boolean askForSaving()
  {
    if (!mustSave) return true;
    int choice = JOptionPane.showConfirmDialog(this, "Do you want to save"+"\n"+
                                               "current work before?",
                                  "Ask for saving", JOptionPane.YES_NO_CANCEL_OPTION,
                                  JOptionPane.WARNING_MESSAGE);
    if (choice == JOptionPane.CANCEL_OPTION) return false;
    if (choice == JOptionPane.YES_OPTION)
      saveAreaMap();

    return true;
  }

  public void saveAreaMap()
  {
     if (areaFileName.trim().equals(""))
        saveAsAreaMap();
     else
        saveWithName(areaFileName);
  }

  public void saveWithName(String name)
  {
     try
     {
        ObjectOutputStream output = new ObjectOutputStream(new FileOutputStream(name));
        output.writeUTF("v"+version);
        areaMap.writeObject(output);
        output.close();
        areaFileName = name;
        setTitle("AreaBuilder ["+areaFileName+"]");
        setMustSave(false);
     }
     catch(IOException exception)
     {
        JOptionPane.showMessageDialog(this, "Saving failed",
                                            "Output error",
                                            JOptionPane.ERROR_MESSAGE);
     }
     //this is the occasion for calling garbage collector
     System.gc();
  }

  public void saveAsAreaMap()
  {
     JFileChooser chooser = null;
     if (lastSaveDir != null)
     {
       chooser = new JFileChooser(lastSaveDir);
     }
     else
     {
       chooser = new JFileChooser(".");
     }
     chooser.setApproveButtonText("Save");
     chooser.setDialogTitle("Save Area Builder Project File");
     chooser.setToolTipText("Save Area Builder Project File");
     chooser.setFileFilter(new AreaBuilderProjectFileFilter());
     int returnChooserVal;
     int  returnOverwriteVal;
     File returnChooserFile;
     boolean ok=false;
     do
     {
        returnChooserVal  = chooser.showSaveDialog(this);
        returnChooserFile = chooser.getSelectedFile();
        if (returnChooserVal == JFileChooser.APPROVE_OPTION)
        {
           this.lastSaveDir = chooser.getCurrentDirectory();
          if(returnChooserFile.exists())
          {
             this.lastSaveDir = chooser.getCurrentDirectory();
             returnOverwriteVal = JOptionPane.showConfirmDialog(this,
                    "This file already exists, do you want to overwrite it ?",
                    "File already exists",
                    JOptionPane.YES_NO_CANCEL_OPTION);
             switch(returnOverwriteVal)
             {
                case JOptionPane.YES_OPTION    : ok = true; break;
                case JOptionPane.CANCEL_OPTION : return;
                default : ok = false; break;
             }
          }//end if already exists
        }//end if OK
        else ok = true;
     } while(!ok);
     if (returnChooserVal == JFileChooser.APPROVE_OPTION)
     {
        String filename = returnChooserFile.getPath();
        if (!filename.endsWith(".abp")) filename = filename + ".abp";
        if (!filename.trim().equals(""))
        {
          saveWithName(filename);
        }
     }
  }

  public void buildArea()
  {
     String proposedName = "";
     if (!areaFileName.trim().equals(""))
     {
        if (!areaFileName.endsWith(".abp"))
        {
          proposedName = areaFileName + ".are";
        }
        else
        {
          proposedName = areaFileName.substring(0,areaFileName.lastIndexOf(".abp"))+".are";
        }
     }

     /*if (areaFileName.trim().equals(""))
     {
       saveAsAreaMap();
     }
     if (areaFileName.trim().equals("")) return;*/
     try
     {
/*        String name;
        if (!areaFileName.endsWith(".abp"))
        {
          name = areaFileName + ".are";
        }
        else
        {
          name = areaFileName.substring(0,areaFileName.lastIndexOf(".abp"))+".are";
        }*/

        /*int choice;
        int count = 1;

        choice = JOptionPane.YES_OPTION;
        File f = new File(name);
        if (f.exists())
        {
           choice = JOptionPane.showConfirmDialog(this, "The file "+f.getName()+" already exists.\n"+
                                               "Do you want to overwrite it?","Filename conflict",
                                                JOptionPane.YES_NO_OPTION,
                                                JOptionPane.QUESTION_MESSAGE);
        }
        if (choice == JOptionPane.NO_OPTION) return;*/
        JFileChooser chooser = new JFileChooser(lastOpenDir);
        chooser.setSelectedFile(new File(proposedName));
        boolean ok;
        int choice;
        do
        {
          ok = true;
          choice = chooser.showSaveDialog(this);
          if (choice != JFileChooser.APPROVE_OPTION) return;
          File f = chooser.getSelectedFile();
          if (f.exists())
          {
            choice = JOptionPane.showConfirmDialog(this, "The file "+f.getName()+" already exists.\n"+
                                               "Do you want to overwrite it?","Filename conflict",
                                                JOptionPane.YES_NO_OPTION,
                                                JOptionPane.QUESTION_MESSAGE);
            ok =  (choice == JOptionPane.YES_OPTION);
          }
        } while(!ok);
        if (choice != JFileChooser.APPROVE_OPTION) return;
        lastOpenDir = chooser.getCurrentDirectory();
        proposedName = chooser.getName(chooser.getSelectedFile());

        FileWriter output = new FileWriter(proposedName);
        areaMap.buildArea(output);
        output.close();
        JOptionPane.showMessageDialog(this, "The file "+proposedName+"\n"+
           "has been successfully built","Build complete",
           JOptionPane.INFORMATION_MESSAGE);
     }
     catch(IOException exception)
     {
        JOptionPane.showMessageDialog(this, "Building area failed",
                                            "Output error",
                                            JOptionPane.ERROR_MESSAGE);
     }
     //this is an occasion for calling garbage collector
     System.gc();
  }

  public void selectAreaNumber()
  {
     areaDialog.showFor(areaMap);
  }

  public void quitProgram()
  {
      int ok=JOptionPane.showConfirmDialog(this, "Do you really want to quit?","Quit",JOptionPane.YES_NO_OPTION);
      if (ok == JOptionPane.YES_OPTION)
      {
         if (askForSaving())
         {
           this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
           System.exit(0);
         }
         else
          this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
      }
      else
       this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
  }

   public void performUndoableAction(UndoableAction action)
   {
      actionStack.doAction(action);
      setMustSave(true);
      setUndoState(true);
      setRedoState(false);
   }
}