/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.awt.*;
import java.util.*;

public class SelectionOfRooms extends AbstractSelection
{
  private LinkedList listOfInternWires; //wires connecting pairs of rooms contained in this selection

  public SelectionOfRooms()
  {
     super();
     listOfInternWires = new LinkedList();
  }

  public Object clone()
  {
     SelectionOfRooms newSelection = new SelectionOfRooms();
     newSelection.copy(this);
     return newSelection;
  }

  public Iterator getListOfInternWiresIterator() {return listOfInternWires.listIterator();}

  protected void copy(SelectionOfRooms source)
  {
     super.copy((AbstractSelection) source);
     this.listOfInternWires = new LinkedList(source.listOfInternWires);
  }

  public void clear()
  {
     super.clear();
     listOfInternWires.clear();
  }

  public void addInternWire(Wire wire) //adds a wire as intern wire
  {
     if (wire != null) listOfInternWires.add(wire);
  }

  private void addInternWire(Room room) //checks if adding this room adds intern wires, and do it
  {
     Room fromRoom = null;
     Wire wire = null;
     Iterator it = listOfSelectedObjects.iterator();
     while(it.hasNext())
     {
        fromRoom = (Room) it.next();
        wire = fromRoom.lookForConnectionToRoom(room);
        if ((wire != null) && (!listOfInternWires.contains(wire)))
        {
          listOfInternWires.add(wire);
        }
        wire = room.lookForConnectionToRoom(fromRoom);
        if ((wire != null) && (!listOfInternWires.contains(wire)))
        {
          listOfInternWires.add(wire);
        }
     }
  }

  private void removeInternWire(Room room) //checks if removing this room removes an intern wire, and do it
  {
     Room fromRoom = null;
     Room toRoom = null;
     Wire wire = null;
     Iterator it = listOfInternWires.iterator();
     while(it.hasNext())
     {
        wire = (Wire) it.next();
        if ((wire.getFromRoom() == room) || (wire.getToRoom() == room)) it.remove();
     }
  }

  public Iterator getIterator()
    {return listOfSelectedObjects.iterator();}

  public void translating(int deltaCandidateXOnScreen, int deltaCandidateYOnScreen)
  {
     Iterator it = listOfSelectedObjects.listIterator();

     Room room = null;
     while(it.hasNext())
     {
        room = (Room) it.next();
        room.translating(deltaCandidateXOnScreen, deltaCandidateYOnScreen);
     }

     it = listOfInternWires.iterator();
     Wire wire = null;
     while(it.hasNext())
     {
        wire = (Wire) it.next();
        wire.translate(deltaCandidateXOnScreen, deltaCandidateYOnScreen);
     }

  }

  public void resetSelection()
  {
    super.resetSelection();
    listOfInternWires.clear();
  }
  public void resetSelection(SelectableNodeObject o)
  {
    super.resetSelection(o);
    listOfInternWires.clear();
    addInternWire((Room) o);
  };

  public void addToSelection(SelectableNodeObject o)
  {
      super.addToSelection(o);
      addInternWire((Room) o);
  }
  public void removeFromSelection(SelectableNodeObject o)
  {
     super.removeFromSelection(o);
     removeInternWire((Room) o);
  }
  public void toggleInSelection(SelectableNodeObject o)
  {
     if (o.isSelected())
        removeFromSelection(o);
     else
        addToSelection(o);
  }
}