/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.util.*;
import java.awt.*;
import java.io.*; //for Archivation

public class WireCollection implements Serializable
{
  LinkedList wireList = null;

  public WireCollection()
  {
     wireList = new LinkedList();
  }

  public void clear()
  {
     wireList.clear();
  }

  public void add(Wire wire)
  {
     wireList.add(wire);
  }

  public void remove(Wire wire)
  {
     wireList.remove(wire);
  }

 public void unselectAll()
 {
    Iterator it = wireList.listIterator();
    Wire wire = null;
    while (it.hasNext())
    {
      wire = (Wire) it.next();
      wire.unselect();
    }
 }

  public void reScale(float newScale)
  {
    Iterator it = wireList.listIterator();
    Wire wire = null;
    while (it.hasNext())
    {
      wire = (Wire) it.next();
      wire.reScale(newScale);
    }
  }

  public Wire getWireGoingTrough(Point point)
  {
     Iterator i  = wireList.listIterator();
     Wire wire = null;
     while (i.hasNext())
     {
        wire = (Wire) i.next();
        if (wire.isGoingThrough(point))
        {
           return wire;
        }
     }
     return null;
  }

  public void translate(int deltaXOnScreen, int deltaYOnScreen)
  {
    Iterator it = wireList.listIterator();
    Wire wire = null;
    while (it.hasNext())
    {
      wire = (Wire) it.next();
      wire.translate(deltaXOnScreen, deltaYOnScreen);
    }
  }

  public void paint(Graphics g)
  {
    Iterator it = wireList.listIterator();
    Wire wire = null;
    while (it.hasNext())
    {
      wire = (Wire) it.next();
      wire.paint(g);
    }
  }

  public void writeObject(ObjectOutputStream output) throws IOException
  {
     output.writeInt(wireList.size());
     Iterator it = wireList.iterator();
     while(it.hasNext())
     {
        ((Wire) it.next()).writeObject(output);
     }
 }

 public void readObject(ObjectInputStream input) throws IOException, ClassNotFoundException
 {
    this.clear();

    int sizeOfWireList = input.readInt();
    Wire wire = null;
    for(int i=0; i<sizeOfWireList ; ++i)
    {
       wire = new Wire();
       wire.readObject(input);
       wireList.add(wire);
    }
 }

 public void establishLinks()
 {
    Iterator it = wireList.listIterator();
    Wire wire = null;
    while (it.hasNext())
    {
      wire = (Wire) it.next();
      wire.establishLinks();
    }
 }
}