/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.util.*;

public class AnatoliaConsts
{
   public final static byte NORTH = 0;
   public final static byte EAST  = 1;
   public final static byte SOUTH = 2;
   public final static byte WEST  = 3;
   public final static byte UP    = 4;
   public final static byte DOWN  = 5;
   public final static String[] directionTexts={"North", "East", "South", "West", "Up", "Down"};

   public final static byte LEVEL_MIN = 0;
   public final static byte LEVEL_MAX = 100;

   public final static byte OBJECT_CONDITION_EXCELLENT = 0;
   public final static byte OBJECT_CONDITION_GOOD      = 1;
   public final static byte OBJECT_CONDITION_FINE      = 2;
   public final static byte OBJECT_CONDITION_AVERAGE   = 3;
   public final static byte OBJECT_CONDITION_POOR      = 4;
   public final static byte OBJECT_CONDITION_FRAGILE   = 5;
   public final static byte OBJECT_CONDITION_BROKEN    = 6;
   public static FlagStructure[] objectConditionTexts = new FlagStructure[7];

   public final static byte CONT_CLOSEABLE = 0;
   public final static byte CONT_PICKPROOF = 1;
   public final static byte CONT_CLOSED    = 2;
   public final static byte CONT_LOCKED    = 3;
   public final static byte CONT_PUT_ON    = 4;
   public final static byte CONT_FOR_ARROW = 5;
   public static FlagStructure[] containerFlagTexts = new FlagStructure[6];

   public final static byte OBJECT_PROG_FIGHT_PROG=0;
   public final static byte OBJECT_PROG_DEATH_PROG=1;
   public final static byte OBJECT_PROG_GIVE_PROG=2;
   public final static byte OBJECT_PROG_WEAR_PROG=3;
   public final static byte OBJECT_PROG_REMOVE_PROG=4;
   public final static byte OBJECT_PROG_GET_PROG=5;
   public final static byte OBJECT_PROG_SAC_PROG=6;
   public final static String[] objectProgTexts =
      {"fight_prog","death_prog","give_prog","wear_prog","remove_prog",
       "get_prog","sac_prog"};

   public final static byte MOBILE_PROG_FIGHT_PROG=0;
   public final static byte MOBILE_PROG_DEATH_PROG=1;
   public final static byte MOBILE_PROG_GIVE_PROG=2;
   public final static byte MOBILE_PROG_GREET_PROG=3;
   public final static byte MOBILE_PROG_SPEECH_PROG=4;
   public final static byte MOBILE_PROG_BRIBE_PROG=5;
   public final static byte MOBILE_PROG_AREA_PROG=6;
   public final static String[] mobileProgTexts =
      {"fight_prog","death_prog","give_prog","greet_prog","speech_prog",
       "bribe_prog","area_prog"};

   public final static byte MOBILE_PRACTICER_GROUP_NONE          =0;
   public final static byte MOBILE_PRACTICER_GROUP_WEAPONMASTER  =1;
   public final static byte MOBILE_PRACTICER_GROUP_ATTACK        =2;
   public final static byte MOBILE_PRACTICER_GROUP_BEGUILING     =3;
   public final static byte MOBILE_PRACTICER_GROUP_BENEDICTIONS  =4;
   public final static byte MOBILE_PRACTICER_GROUP_COMBAT        =5;
   public final static byte MOBILE_PRACTICER_GROUP_CREATION      =6;
   public final static byte MOBILE_PRACTICER_GROUP_CURATIVE      =7;
   public final static byte MOBILE_PRACTICER_GROUP_DETECTION     =8;
   public final static byte MOBILE_PRACTICER_GROUP_DRACONIAN     =9;
   public final static byte MOBILE_PRACTICER_GROUP_ENCHANTMENT   =10;
   public final static byte MOBILE_PRACTICER_GROUP_ENHANCMENT    =11;
   public final static byte MOBILE_PRACTICER_GROUP_HARMFUL       =12;
   public final static byte MOBILE_PRACTICER_GROUP_HEALING       =13;
   public final static byte MOBILE_PRACTICER_GROUP_ILLUSION      =14;
   public final static byte MOBILE_PRACTICER_GROUP_MALADICTIONS  =15;
   public final static byte MOBILE_PRACTICER_GROUP_PROTECTIVE    =16;
   public final static byte MOBILE_PRACTICER_GROUP_TRANSPORTATION=17;
   public final static byte MOBILE_PRACTICER_GROUP_WEATHER       =18;
   public final static byte MOBILE_PRACTICER_GROUP_FIGHTMASTER   =19;
   public final static byte MOBILE_PRACTICER_GROUP_SUDDENDEATH   =20;
   public final static byte MOBILE_PRACTICER_GROUP_MEDITATION    =21;
   public final static byte MOBILE_PRACTICER_GROUP_CABAL         =22;
   public final static byte MOBILE_PRACTICER_GROUP_DEFENSE       =23;
   public final static byte MOBILE_PRACTICER_GROUP_DEFENSIVE     =24;
   public final static byte MOBILE_PRACTICER_GROUP_WIZARD        =25;
   public final static String[] mobilePracticerTexts =
      {"group_none","group_weaponmaster","group_attack","group_beguiling",
       "group_benedictions","group_combat","group_creation","group_curative",
       "group_detection","group_draconian","group_enchantment","group_enhancment",
       "group_harmful","group_healing","group_illusion","group_maladictions",
       "group_protective","group_transportation","group_weather","group_fightmaster",
       "group_suddendeath","group_meditation","group_cabal","group_defense",
       "group_defensive","group_wizard"
      };

   public final static int SPELL_AMNESIA             = 0;
   public final static int SPELL_ATTRACT_OTHER       = 1;
   public final static int SPELL_DRAGON_BREATH       = 2;
   public final static int SPELL_DRAGON_STRENGTH     = 3;
   public final static int SPELL_GENERAL_PURPOSE     = 4;
   public final static int SPELL_HIGH_EXPLOSIVE      = 5;
   public final static int SPELL_LOVE_POTION         = 6;
   public final static int SPELL_REMOVE_TATTOO       = 7;
   public final static int SPELL_TATTOO              = 8;
   public final static int SPELL_ACETUM_PRIMUS       = 9;
   public final static int SPELL_ACID_ARROW          = 10;
   public final static int SPELL_ACID_BREATH         = 11;
   public final static int SPELL_ACID_BLAST          = 12;
   public final static int SPELL_ACUTE_VISION        = 13;
   public final static int SPELL_ADAMANTITE_GOLEM    = 14;
   public final static int SPELL_AID                 = 15;
   public final static int SPELL_ANIMATE_DEAD        = 16;
   public final static int SPELL_ARMOR               = 17;
   public final static int SPELL_ASSIST              = 18;
   public final static int SPELL_ASTRAL_WALK         = 19;
   public final static int SPELL_BARK_SKIN           = 20;
   public final static int SPELL_BLACK_DEATH         = 21;
   public final static int SPELL_BLADE_BARRIER       = 22;
   public final static int SPELL_BLESS               = 23;
   public final static int SPELL_BLESS_WEAPON        = 24;
   public final static int SPELL_BLINDNESS           = 25;
   public final static int SPELL_BLUEFIRE            = 26;
   public final static int SPELL_BURNING_HANDS       = 27;
   public final static int SPELL_CALL_LIGHTNING      = 28;
   public final static int SPELL_CALM                = 29;
   public final static int SPELL_CANCELLATION        = 30;
   public final static int SPELL_CAUSE_CRITICAL      = 31;
   public final static int SPELL_CAUSE_LIGHT         = 32;
   public final static int SPELL_CAUSE_SERIOUS       = 33;
   public final static int SPELL_CAUSTIC_FONT        = 34;
   public final static int SPELL_CHAIN_LIGHTNING     = 35;
   public final static int SPELL_CHARM_PERSON        = 36;
   public final static int SPELL_CHILL_TOUCH         = 37;
   public final static int SPELL_COLOUR_SPRAY        = 38;
   public final static int SPELL_CONTINUAL_LIGHT     = 39;
   public final static int SPELL_CONTROL_UNDEAD      = 40;
   public final static int SPELL_CONTROL_WEATHER     = 41;
   public final static int SPELL_CORRUPTION          = 42;
   public final static int SPELL_CREATE_FOOD         = 43;
   public final static int SPELL_CREATE_ROSE         = 44;
   public final static int SPELL_CREATE_SPRING       = 45;
   public final static int SPELL_CREATE_WATER        = 46;
   public final static int SPELL_CURE_BLINDNESS      = 47;
   public final static int SPELL_CURE_CRITICAL       = 48;
   public final static int SPELL_CURE_DISEASE        = 49;
   public final static int SPELL_CURE_LIGHT          = 50;
   public final static int SPELL_CURE_POISON         = 51;
   public final static int SPELL_CURE_SERIOUS        = 52;
   public final static int SPELL_CURSE               = 53;
   public final static int SPELL_CURSED_LANDS        = 54;
   public final static int SPELL_DEADLY_VENOM        = 55;
   public final static int SPELL_DEAFEN              = 56;
   public final static int SPELL_DEMONFIRE           = 57;
   public final static int SPELL_DESERT_FIST         = 58;
   public final static int SPELL_DETECT_EVIL         = 59;
   public final static int SPELL_DETECT_GOOD         = 60;
   public final static int SPELL_HIDDEN_PERCEPTION   = 61;
   public final static int SPELL_DETECT_INVIS        = 62;
   public final static int SPELL_DETECT_MAGIC        = 63;
   public final static int SPELL_DETECT_POISON       = 64;
   public final static int SPELL_DETECT_UNDEAD       = 65;
   public final static int SPELL_DISINTEGRATE        = 66;
   public final static int SPELL_DISPEL_EVIL         = 67;
   public final static int SPELL_DISPEL_GOOD         = 68;
   public final static int SPELL_DISPEL_MAGIC        = 69;
   public final static int SPELL_DISRUPTION          = 70;
   public final static int SPELL_DRAGON_SKIN         = 71;
   public final static int SPELL_DRAGONS_BREATH      = 72;
   public final static int SPELL_EARTHQUAKE          = 73;
   public final static int SPELL_ENCHANT_ARMOR       = 74;
   public final static int SPELL_ENCHANT_WEAPON      = 75;
   public final static int SPELL_ENERGY_DRAIN        = 76;
   public final static int SPELL_ENHANCED_ARMOR      = 77;
   public final static int SPELL_ETHERAL_FIST        = 78;
   public final static int SPELL_FAERIE_FIRE         = 79;
   public final static int SPELL_FAERIE_FOG          = 80;
   public final static int SPELL_FARSIGHT            = 81;
   public final static int SPELL_FEAR                = 82;
   public final static int SPELL_FIREBALL            = 83;
   public final static int SPELL_FIRE_BREATH         = 84;
   public final static int SPELL_FIRE_PROOF          = 85;
   public final static int SPELL_FIRE_SHIELD         = 86;
   public final static int SPELL_FLAMESTRIKE         = 87;
   public final static int SPELL_FLOATING_DISC       = 88;
   public final static int SPELL_FLY                 = 89;
   public final static int SPELL_FRENZY              = 90;
   public final static int SPELL_FROST_BREATH        = 91;
   public final static int SPELL_GALVANIC_WHIP       = 92;
   public final static int SPELL_GAS_BREATH          = 93;
   public final static int SPELL_GATE                = 94;
   public final static int SPELL_GIANT_STRENGTH      = 95;
   public final static int SPELL_GROUP_DEFENSE       = 96;
   public final static int SPELL_GROUP_HEAL          = 97;
   public final static int SPELL_HAND_OF_UNDEAD      = 98;
   public final static int SPELL_HARM                = 99;
   public final static int SPELL_HASTE               = 100;
   public final static int SPELL_HEAL                = 101;
   public final static int SPELL_HEALING_LIGHT       = 102;
   public final static int SPELL_HEAT_METAL          = 103;
   public final static int SPELL_HELICAL_FLOW        = 104;
   public final static int SPELL_HELLFIRE            = 105;
   public final static int SPELL_HOLY_WORD           = 106;
   public final static int SPELL_HURRICANE           = 107;
   public final static int SPELL_ICEBALL             = 108;
   public final static int SPELL_IDENTIFY            = 109;
   public final static int SPELL_IMPROVED_DETECT     = 110;
   public final static int SPELL_IMPROVED_INVIS      = 111;
   public final static int SPELL_INFRAVISION         = 112;
   public final static int SPELL_INSANITY            = 113;
   public final static int SPELL_INSPIRE             = 114;
   public final static int SPELL_INVISIBILITY        = 115;
   public final static int SPELL_IRON_GOLEM          = 116;
   public final static int SPELL_KNOCK               = 117;
   public final static int SPELL_KNOW_ALIGNMENT      = 118;
   public final static int SPELL_LESSER_GOLEM        = 119;
   public final static int SPELL_LETHARGIC_MIST      = 120;
   public final static int SPELL_LIGHTNING_BOLT      = 121;
   public final static int SPELL_LIGHTNING_BREATH    = 122;
   public final static int SPELL_LIGHTNING_SHIELD    = 123;
   public final static int SPELL_LINK                = 124;
   public final static int SPELL_LION_HELP           = 125;
   public final static int SPELL_LOCATE_OBJECT       = 126;
   public final static int SPELL_MAGIC_JAR           = 127;
   public final static int SPELL_MAGIC_MISSILE       = 128;
   public final static int SPELL_MAGIC_RESISTANCE    = 129;
   public final static int SPELL_MAGNETIC_TRUST      = 130;
   public final static int SPELL_MASS_HEALING        = 131;
   public final static int SPELL_MASS_INVIS          = 132;
   public final static int SPELL_MASS_SANCTUARY      = 133;
   public final static int SPELL_MASTER_HEALING      = 134;
   public final static int SPELL_MELD_INTO_STONE     = 135;
   public final static int SPELL_MEND                = 136;
   public final static int SPELL_MIND_LIGHT          = 137;
   public final static int SPELL_MIND_WRACK          = 138;
   public final static int SPELL_MIND_WRENCH         = 139;
   public final static int SPELL_MIST_WALK           = 140;
   public final static int SPELL_MYSTERIOUS_DREAM    = 141;
   public final static int SPELL_NEXUS               = 142;
   public final static int SPELL_PASS_DOOR           = 143;
   public final static int SPELL_PLAGUE              = 144;
   public final static int SPELL_POISON              = 145;
   public final static int SPELL_POLYMORPH           = 146;
   public final static int SPELL_PORTAL              = 147;
   public final static int SPELL_POWER_WORD_KILL     = 148;
   public final static int SPELL_HAND_OF_GOD         = 149;
   public final static int SPELL_POWER_WORD_STUN     = 150;
   public final static int SPELL_PROTECTION_COLD     = 151;
   public final static int SPELL_PROTECTION_EVIL     = 152;
   public final static int SPELL_PROTECTION_GOOD     = 153;
   public final static int SPELL_PROTECTION_HEAT     = 154;
   public final static int SPELL_PROTECTION_NEGATIVE = 155;
   public final static int SPELL_PROTECTIVE_SHIELD   = 156;
   public final static int SPELL_QUANTUM_SPIKE       = 157;
   public final static int SPELL_RANGER_STAFF        = 158;
   public final static int SPELL_RAY_OF_TRUTH        = 159;
   public final static int SPELL_RECHARGE            = 160;
   public final static int SPELL_REFRESH             = 161;
   public final static int SPELL_REMOVE_CURSE        = 162;
   public final static int SPELL_REMOVE_FEAR         = 163;
   public final static int SPELL_RESILIENCE          = 164;
   public final static int SPELL_RESTORING_LIGHT     = 165;
   public final static int SPELL_SANCTIFY_LANDS      = 166;
   public final static int SPELL_SANCTUARY           = 167;
   public final static int SPELL_SANDSTORM           = 168;
   public final static int SPELL_SCREAM              = 169;
   public final static int SPELL_SEVERITY_FORCE      = 170;
   public final static int SPELL_SHIELD              = 171;
   public final static int SPELL_SHIELDING           = 172;
   public final static int SPELL_SHOCKING_GRASP      = 173;
   public final static int SPELL_SHOCKING_TRAP       = 174;
   public final static int SPELL_SLEEP               = 175;
   public final static int SPELL_SLOW                = 176;
   public final static int SPELL_SOLAR_FLIGHT        = 177;
   public final static int SPELL_SONIC_RESONANCE     = 178;
   public final static int SPELL_SPECTRAL_FUROR      = 179;
   public final static int SPELL_STONE_GOLEM         = 180;
   public final static int SPELL_STONE_SKIN          = 181;
   public final static int SPELL_SULFURUS_SPRAY      = 182;
   public final static int SPELL_SUMMON              = 183;
   public final static int SPELL_SUMMON_SHADOW       = 184;
   public final static int SPELL_SUPERIOR_HEAL       = 185;
   public final static int SPELL_TELEPORT            = 186;
   public final static int SPELL_TURN                = 187;
   public final static int SPELL_VENTRILOQUATE       = 188;
   public final static int SPELL_VAMPIRIC_BLAST      = 189;
   public final static int SPELL_WEAKEN              = 190;
   public final static int SPELL_WEB                 = 191;
   public final static int SPELL_WITCH_CURSE         = 192;
   public final static int SPELL_WORD_OF_RECALL      = 193;
   public final static int SPELL_WRATH               = 194;
   public final static FlagStructure[] spellTexts= new FlagStructure[195];

   public final static byte SEX_NONE   = 0;
   public final static byte SEX_MALE   = 1;
   public final static byte SEX_FEMALE = 2;
   public final static byte SEX_EITHER = 3;
   public final static String[] sexTexts = {"none", "male", "female", "either"};

   public final static byte POSITION_DEAD     = 0;
   public final static byte POSITION_MORTAL   = 1;
   public final static byte POSITION_INCAP    = 2;
   public final static byte POSITION_STUNNED  = 3;
   public final static byte POSITION_SLEEPING = 4;
   public final static byte POSITION_RESTING  = 5;
   public final static byte POSITION_SITTING  = 6;
   public final static byte POSITION_FIGHTING = 7;
   public final static byte POSITION_STANDING = 8;
   public final static String[] positionTexts =
     {"dead","mortal","incap","stun","sleep","rest","sit","fight","stand"};

   public final static byte SIZE_TINY       = 0;
   public final static byte SIZE_SMALL      = 1;
   public final static byte SIZE_MEDIUM     = 2;
   public final static byte SIZE_LARGE      = 3;
   public final static byte SIZE_HUGE       = 4;
   public final static byte SIZE_GIANT      = 5;
   public final static byte SIZE_GARGANTUAN = 6;
   public static FlagStructure[] mobileSizeTexts = new FlagStructure[7];
   public static FlagStructure[] doorSizeTexts = new FlagStructure[7];

   public static final byte RACE_TYPE_AARAKOCRA       = 0;
   public static final byte RACE_TYPE_AIR_ELEMENTAL   = 1;
   public static final byte RACE_TYPE_ARIAL           = 2;
   public static final byte RACE_TYPE_BAT             = 3;
   public static final byte RACE_TYPE_BEAR            = 4;
   public static final byte RACE_TYPE_BEHOLDER        = 5;
   public static final byte RACE_TYPE_BLACK_DRAGON    = 6;
   public static final byte RACE_TYPE_BLUE_DRAGON     = 7;
   public static final byte RACE_TYPE_BRASS_DRAGON    = 8;
   public static final byte RACE_TYPE_BRONZE_DRAGON   = 9;
   public static final byte RACE_TYPE_CAT             = 10;
   public static final byte RACE_TYPE_CENTAUR         = 11;
   public static final byte RACE_TYPE_CENTIPEDE       = 12;
   public static final byte RACE_TYPE_CLOUD_GIANT     = 13;
   public static final byte RACE_TYPE_COPPER_DRAGON   = 14;
   public static final byte RACE_TYPE_DEATH_KNIGHT    = 15;
   public static final byte RACE_TYPE_DARK_ELF        = 16;
   public static final byte RACE_TYPE_DOG             = 17;
   public static final byte RACE_TYPE_DOLL            = 18;
   public static final byte RACE_TYPE_DRACOLICH       = 19;
   public static final byte RACE_TYPE_DRACONIAN       = 20;
   public static final byte RACE_TYPE_DRAGON          = 21;
   public static final byte RACE_TYPE_DROW            = 22;
   public static final byte RACE_TYPE_DUERGAR         = 23;
   public static final byte RACE_TYPE_DWARF           = 24;
   public static final byte RACE_TYPE_EARTH_ELEMENTAL = 25;
   public static final byte RACE_TYPE_ELF             = 26;
   public static final byte RACE_TYPE_FELAR           = 27;
   public static final byte RACE_TYPE_FIDO            = 28;
   public static final byte RACE_TYPE_FIRE_ELEMENTAL  = 29;
   public static final byte RACE_TYPE_FIRE_GIANT      = 30;
   public static final byte RACE_TYPE_FISH            = 31;
   public static final byte RACE_TYPE_FOX             = 32;
   public static final byte RACE_TYPE_FROST_GIANT     = 33;
   public static final byte RACE_TYPE_GARGOYLE        = 34;
   public static final byte RACE_TYPE_GITHYANKI       = 35;
   public static final byte RACE_TYPE_GHOST           = 36;
   public static final byte RACE_TYPE_GHOUL           = 37;
   public static final byte RACE_TYPE_GIANT           = 38;
   public static final byte RACE_TYPE_GNOLL           = 39;
   public static final byte RACE_TYPE_GNOME           = 40;
   public static final byte RACE_TYPE_GOBLIN          = 41;
   public static final byte RACE_TYPE_GOLD_DRAGON     = 42;
   public static final byte RACE_TYPE_GOLEM           = 43;
   public static final byte RACE_TYPE_GREEN_DRAGON    = 44;
   public static final byte RACE_TYPE_HALF_ELF        = 45;
   public static final byte RACE_TYPE_HARPY           = 46;
   public static final byte RACE_TYPE_HOBGOBLIN       = 47;
   public static final byte RACE_TYPE_HORSE           = 48;
   public static final byte RACE_TYPE_HUMAN           = 49;
   public static final byte RACE_TYPE_KOBOLD          = 50;
   public static final byte RACE_TYPE_LAMIA           = 51;
   public static final byte RACE_TYPE_LICH            = 52;
   public static final byte RACE_TYPE_LION            = 53;
   public static final byte RACE_TYPE_LIZARD          = 54;
   public static final byte RACE_TYPE_MINOTAUR        = 55;
   public static final byte RACE_TYPE_MODRON          = 56;
   public static final byte RACE_TYPE_OGRE            = 57;
   public static final byte RACE_TYPE_ORC             = 58;
   public static final byte RACE_TYPE_PIG             = 59;
   public static final byte RACE_TYPE_RABBIT          = 60;
   public static final byte RACE_TYPE_RED_DRAGON      = 61;
   public static final byte RACE_TYPE_ROCKSEER        = 62;
   public static final byte RACE_TYPE_SATYR           = 63;
   public static final byte RACE_TYPE_SCHOOL_MONSTER  = 64;
   public static final byte RACE_TYPE_SILVER_DRAGON   = 65;
   public static final byte RACE_TYPE_SNAKE           = 66;
   public static final byte RACE_TYPE_SONG_BIRD       = 67;
   public static final byte RACE_TYPE_SPRITE          = 68;
   public static final byte RACE_TYPE_STORM_GIANT     = 69;
   public static final byte RACE_TYPE_SVIRFNEBLI      = 70;
   public static final byte RACE_TYPE_TIGER           = 71;
   public static final byte RACE_TYPE_TROLL           = 72;
   public static final byte RACE_TYPE_UNDEAD          = 73;
   public static final byte RACE_TYPE_WATER_ELEMENTAL = 74;
   public static final byte RACE_TYPE_WATER_FOWL      = 75;
   public static final byte RACE_TYPE_WHITE_DRAGON    = 76;
   public static final byte RACE_TYPE_WOLF            = 77;
   public static final byte RACE_TYPE_WYVERN          = 78;
   public static final byte RACE_TYPE_ZOMBIE          = 79;
   public static MobileRaceFlagsValues[] mobileRecommendedRaceFlagsValues = new MobileRaceFlagsValues[80];

   public static final byte ROOM_FLAG_NO_RECALL    = 0;
   public static final byte ROOM_FLAG_NO_MAGIC     = 1;
   public static final byte ROOM_FLAG_NOSUMMON     = 2;
   public static final byte ROOM_FLAG_NOWHERE      = 3;
   public static final byte ROOM_FLAG_DARK         = 4;
   public static final byte ROOM_FLAG_INDOORS      = 5;
   public static final byte ROOM_FLAG_BANK         = 6;
   public static final byte ROOM_FLAG_PET_SHOP     = 7;
   public static final byte ROOM_FLAG_NO_MOB       = 8;
   public static final byte ROOM_FLAG_SAFE         = 9;
   public static final byte ROOM_FLAG_PRIVATE      = 10;
   public static final byte ROOM_FLAG_SOLITARY     = 11;
   public static final byte ROOM_FLAG_NEWBIES_ONLY = 12;
   public static final byte ROOM_FLAG_HEROES_ONLY  = 13;
   public static final byte ROOM_FLAG_GODS_ONLY    = 14;
   public static final byte ROOM_FLAG_IMP_ONLY     = 15;
   public static final byte ROOM_FLAG_LAW          = 16;
   public static final byte ROOM_FLAG_REGISTRY     = 17;
   public static final byte ROOM_FLAG_RANDOM       = 18;
   public static FlagStructure[] roomFlagsTexts = new FlagStructure[19];

   public static final byte ROOM_SECTOR_INSIDE    = 0;
   public static final byte ROOM_SECTOR_CITY      = 1;
   public static final byte ROOM_SECTOR_FIELD     = 2;
   public static final byte ROOM_SECTOR_FOREST    = 3;
   public static final byte ROOM_SECTOR_HILL      = 4;
   public static final byte ROOM_SECTOR_MOUNTAIN  = 5;
   public static final byte ROOM_SECTOR_WATER     = 6;
   public static final byte ROOM_SECTOR_DEEPWATER = 7;
   public static final byte ROOM_SECTOR_UNUSED    = 8;
   public static final byte ROOM_SECTOR_AIR       = 9;
   public static final byte ROOM_SECTOR_DESERT    = 10;
   public static FlagStructure[] roomSectorTexts = new FlagStructure[11];

   public final static boolean EXIT_WITHOUT_DOOR = false;
   public final static boolean EXIT_WITH_DOOR    = true;

   public final static byte DOOR_STATE_OPENED = 0;
   public final static byte DOOR_STATE_CLOSED = 1;
   public final static byte DOOR_STATE_LOCKED = 2;
   public static FlagStructure[] doorStateTexts = new FlagStructure[3];

   public final static byte DOOR_RESETS_OPENED       = 0;
   public final static byte DOOR_RESETS_CLOSED       = 1;
   public final static byte DOOR_RESETS_LOCKED       = 2;
   public final static byte DOOR_NO_PASSDOOR         = 3;
   public final static byte DOOR_EASY_TO_PICK        = 4;
   public final static byte DOOR_HARD_TO_PICK        = 5;
   public final static byte DOOR_INFURIATING_TO_PICK = 6;
   public final static byte DOOR_CANNOT_BE_PICKED    = 7;
   public final static byte DOOR_CANNOT_BE_CLOSED    = 8;
   public final static byte DOOR_CANNOT_BE_LOCKED    = 9;
   public static FlagStructure[] doorResetsTexts = new FlagStructure[10];

   public final static byte WEAPON_TYPE_EXOTIC  = 0;
   public final static byte WEAPON_TYPE_SWORD   = 1;
   public final static byte WEAPON_TYPE_DAGGER  = 2;
   public final static byte WEAPON_TYPE_SPEAR   = 3;
   public final static byte WEAPON_TYPE_MACE    = 4;
   public final static byte WEAPON_TYPE_AXE     = 5;
   public final static byte WEAPON_TYPE_FLAIL   = 6;
   public final static byte WEAPON_TYPE_WHIP    = 7;
   public final static byte WEAPON_TYPE_POLEARM = 8;
   public final static byte WEAPON_TYPE_BOW     = 9;
   public final static byte WEAPON_TYPE_ARROW   = 10;
   public final static byte WEAPON_TYPE_LANCE   = 11;
   public final static String[] weaponTypeTexts = {"exotic","sword","dagger","spear","mace","axe","flail",
                                                   "whip","polearm","bow","arrow","lance"};

   public final static byte WEAPON_FLAG_FLAMING    = 0;
   public final static byte WEAPON_FLAG_FROST      = 1;
   public final static byte WEAPON_FLAG_VAMPIRIC   = 2;
   public final static byte WEAPON_FLAG_SHARP      = 3;
   public final static byte WEAPON_FLAG_VORPAL     = 4;
   public final static byte WEAPON_FLAG_TWO_HANDED = 5;
   public final static byte WEAPON_FLAG_SHOCKING   = 6;
   public final static byte WEAPON_FLAG_POISONED   = 7;
   public final static byte WEAPON_FLAG_HOLY       = 8;
   public final static byte WEAPON_FLAG_KATANA     = 9;
   public final static byte WEAPON_FLAG_NIGHTMARE  = 10;
   public static FlagStructure[] weaponFlagsTexts = new FlagStructure[11];

   public final static byte DAMAGE_TYPE_PIERCE = 0;
   public final static byte DAMAGE_TYPE_BASH   = 1;
   public final static byte DAMAGE_TYPE_SLASH  = 2;
   public final static byte DAMAGE_TYPE_MAGIC  = 3;
   public final static String[] damageTypeTexts = {"pierce", "bash", "slash", "magic"};
   public static String[][] damageTexts = null;

   public final static byte DAMAGE_TYPE_PIERCE_BITE    = 0;
   public final static byte DAMAGE_TYPE_PIERCE_CHOMP   = 1;
   public final static byte DAMAGE_TYPE_PIERCE_PECK    = 2;
   public final static byte DAMAGE_TYPE_PIERCE_PIERCE  = 3;
   public final static byte DAMAGE_TYPE_PIERCE_SCRATCH = 4;
   public final static byte DAMAGE_TYPE_PIERCE_STAB    = 5;
   public final static byte DAMAGE_TYPE_PIERCE_STING   = 6;
   public final static byte DAMAGE_TYPE_PIERCE_THRUST  = 7;
   public final static String[] damageTypePierceTexts =
          {"bite", "chomp", "peck", "pierce", "scratch", "stab", "sting", "thrust"};

   public final static String[] weaponDamageTypeTexts =
          {"hit","slice","stab","slash","whip","claw","blast","pound","crush","grep","bite",
           "pierce","suction"};


   public final static byte DAMAGE_TYPE_BASH_BEATING = 0;
   public final static byte DAMAGE_TYPE_BASH_BLAST   = 1;
   public final static byte DAMAGE_TYPE_BASH_POUND   = 2;
   public final static byte DAMAGE_TYPE_BASH_CHARGE  = 3;
   public final static byte DAMAGE_TYPE_BASH_CRUSH   = 4;
   public final static byte DAMAGE_TYPE_BASH_PECK    = 5;
   public final static byte DAMAGE_TYPE_BASH_PUNCH   = 6;
   public final static byte DAMAGE_TYPE_BASH_SLAP    = 7;
   public final static byte DAMAGE_TYPE_BASH_SMASH   = 8;
   public final static byte DAMAGE_TYPE_BASH_SUCTION = 9;
   public final static byte DAMAGE_TYPE_BASH_THWACK  = 10;
   public final static String[] damageTypeBashTexts =
          {"beating", "blast", "pound", "charge", "crush", "peck", "punch", "slap","smash","suction","thwack"};

   public final static byte DAMAGE_TYPE_SLASH_CLAW   = 0;
   public final static byte DAMAGE_TYPE_SLASH_CLEAVE = 1;
   public final static byte DAMAGE_TYPE_SLASH_GREP   = 2;
   public final static byte DAMAGE_TYPE_SLASH_SLASH  = 3;
   public final static byte DAMAGE_TYPE_SLASH_SLICE  = 4;
   public final static byte DAMAGE_TYPE_SLASH_WHIP   = 5;
   public final static String[] damageTypeSlashTexts =
          {"claw", "cleave", "grep", "slash", "slice", "whip"};

   public final static byte DAMAGE_TYPE_MAGIC_ACID_ACIDIC_BITE        = 0;
   public final static byte DAMAGE_TYPE_MAGIC_ACID_DIGESTION          = 1;
   public final static byte DAMAGE_TYPE_MAGIC_ACID_SLIME              = 2;
   public final static byte DAMAGE_TYPE_MAGIC_COLD_CHILL              = 3;
   public final static byte DAMAGE_TYPE_MAGIC_COLD_FREEZING_BITE      = 4;
   public final static byte DAMAGE_TYPE_MAGIC_ENERGY_MAGIC            = 5;
   public final static byte DAMAGE_TYPE_MAGIC_ENERGY_WRATH            = 6;
   public final static byte DAMAGE_TYPE_MAGIC_FIRE_FLAME              = 7;
   public final static byte DAMAGE_TYPE_MAGIC_FIRE_FLAMING_BITE       = 8;
   public final static byte DAMAGE_TYPE_MAGIC_HOLY_DIVINE_POWER       = 9;
   public final static byte DAMAGE_TYPE_MAGIC_LIGHTNING_SHOCK         = 10;
   public final static byte DAMAGE_TYPE_MAGIC_LIGHTNING_SHOCKING_BITE = 11;
   public final static byte DAMAGE_TYPE_MAGIC_NEGATIVE_LIFE_DRAIN     = 12;
   public final static String[] damageTypeMagicTexts =
          {"acbite", "digestion", "slime", "chill", "frbite", "magic", "wrath","flame","flaming",
           "flbite","divine","shock","shbite","drain"};

   public final static byte MOBILE_ACT_FLAG_IS_NPC        = 0;
   public final static byte MOBILE_ACT_FLAG_SENTINEL      = 1;
   public final static byte MOBILE_ACT_FLAG_SCAVENGER     = 2;
   public final static byte MOBILE_ACT_FLAG_AGGRESSIVE    = 3;
   public final static byte MOBILE_ACT_FLAG_STAY_AREA     = 4;
   public final static byte MOBILE_ACT_FLAG_WIMPY         = 5;
   public final static byte MOBILE_ACT_FLAG_PET           = 6;
   public final static byte MOBILE_ACT_FLAG_TRAIN         = 7;
   public final static byte MOBILE_ACT_FLAG_PRACTICE      = 8;
   public final static byte MOBILE_ACT_FLAG_HUNTER        = 9;
   public final static byte MOBILE_ACT_FLAG_UNDEAD        = 10;
   public final static byte MOBILE_ACT_FLAG_CLERIC        = 11;
   public final static byte MOBILE_ACT_FLAG_MAGE          = 12;
   public final static byte MOBILE_ACT_FLAG_THIEF         = 13;
   public final static byte MOBILE_ACT_FLAG_WARRIOR       = 14;
   public final static byte MOBILE_ACT_FLAG_NOALIGN       = 15;
   public final static byte MOBILE_ACT_FLAG_NOPURGE       = 16;
   public final static byte MOBILE_ACT_FLAG_OUTDOORS      = 17;
   public final static byte MOBILE_ACT_FLAG_INDOORS       = 18;
   public final static byte MOBILE_ACT_FLAG_RIDEABLE      = 19;
   public final static byte MOBILE_ACT_FLAG_IS_HEALER     = 20;
   public final static byte MOBILE_ACT_FLAG_GAIN          = 21;
   public final static byte MOBILE_ACT_FLAG_UPDATE_ALWAYS = 22;
   public final static byte MOBILE_ACT_FLAG_IS_CHANGER    = 23;
   public final static byte MOBILE_ACT_FLAG_NOTRACK       = 24;
   public static FlagStructure[] mobileActFlagTexts = new FlagStructure[25];

   public final static byte MOBILE_AFF_FLAG_BLIND        = 0;
   public final static byte MOBILE_AFF_FLAG_INVISIBLE    = 1;
   public final static byte MOBILE_AFF_FLAG_DETECT_EVIL  = 2;
   public final static byte MOBILE_AFF_FLAG_DETECT_INVIS = 3;
   public final static byte MOBILE_AFF_FLAG_DETECT_MAGIC = 4;
   public final static byte MOBILE_AFF_FLAG_DETECT_HIDE  = 5;
   public final static byte MOBILE_AFF_FLAG_DETECT_GOOD  = 6;
   public final static byte MOBILE_AFF_FLAG_SANCTUARY    = 7;
   public final static byte MOBILE_AFF_FLAG_FAERIE_FIRE  = 8;
   public final static byte MOBILE_AFF_FLAG_INFRAVISION  = 9;
   public final static byte MOBILE_AFF_FLAG_PROTECT_EVIL = 10;
   public final static byte MOBILE_AFF_FLAG_PROTECT_GOOD = 11;
   public final static byte MOBILE_AFF_FLAG_SNEAKING     = 12;
   public final static byte MOBILE_AFF_FLAG_HIDING       = 13;
   public final static byte MOBILE_AFF_FLAG_FLYING       = 14;
   public final static byte MOBILE_AFF_FLAG_PASS_DOOR    = 15;
   public final static byte MOBILE_AFF_FLAG_HASTE        = 16;
   public final static byte MOBILE_AFF_FLAG_DARK_VISION  = 17;
   public final static byte MOBILE_AFF_FLAG_SWIMMING     = 18;
   public final static byte MOBILE_AFF_FLAG_REGENERATION = 19;
   public static FlagStructure[] mobileAffFlagTexts = new FlagStructure[20];

   public final static byte MOBILE_OFF_FLAG_AREA_ATTACK = 0;
   public final static byte MOBILE_OFF_FLAG_BACKSTAB    = 1;
   public final static byte MOBILE_OFF_FLAG_BASH        = 2;
   public final static byte MOBILE_OFF_FLAG_BERSERK     = 3;
   public final static byte MOBILE_OFF_FLAG_DISARM      = 4;
   public final static byte MOBILE_OFF_FLAG_DODGE       = 5;
   public final static byte MOBILE_OFF_FLAG_FADE        = 6;
   public final static byte MOBILE_OFF_FLAG_FAST        = 7;
   public final static byte MOBILE_OFF_FLAG_KICK        = 8;
   public final static byte MOBILE_OFF_FLAG_KICK_DIRT   = 9;
   public final static byte MOBILE_OFF_FLAG_PARRY       = 10;
   public final static byte MOBILE_OFF_FLAG_RESCUE      = 11;
   public final static byte MOBILE_OFF_FLAG_TAIL        = 12;
   public final static byte MOBILE_OFF_FLAG_TRIP        = 13;
   public final static byte MOBILE_OFF_FLAG_CRUSH       = 14;
   public static FlagStructure[] mobileOffFlagTexts = new FlagStructure[15];

   public static final byte MOBILE_ASSISTS_ALL     = 0;
   public static final byte MOBILE_ASSISTS_ALIGN   = 1;
   public static final byte MOBILE_ASSISTS_RACE    = 2;
   public static final byte MOBILE_ASSISTS_PLAYERS = 3;
   public static final byte MOBILE_ASSISTS_GUARD   = 4;
   public static final byte MOBILE_ASSISTS_VNUM    = 5;
   public static FlagStructure[] mobileAssistsTexts = new FlagStructure[6];

   public final static byte IMM_RES_VULN_SUMMON    = 0;
   public final static byte IMM_RES_VULN_CHARM     = 1;
   public final static byte IMM_RES_VULN_MAGIC     = 2;
   public final static byte IMM_RES_VULN_WEAPONS   = 3;
   public final static byte IMM_RES_VULN_BASH      = 4;
   public final static byte IMM_RES_VULN_PIERCE    = 5;
   public final static byte IMM_RES_VULN_SLASH     = 6;
   public final static byte IMM_RES_VULN_FIRE      = 7;
   public final static byte IMM_RES_VULN_COLD      = 8;
   public final static byte IMM_RES_VULN_LIGHTNING = 9;
   public final static byte IMM_RES_VULN_ACID      = 10;
   public final static byte IMM_RES_VULN_POISON    = 11;
   public final static byte IMM_RES_VULN_NEGATIVE  = 12;
   public final static byte IMM_RES_VULN_HOLY      = 13;
   public final static byte IMM_RES_VULN_ENERGY    = 14;
   public final static byte IMM_RES_VULN_MENTAL    = 15;
   public final static byte IMM_RES_VULN_DISEASE   = 16;
   public final static byte IMM_RES_VULN_DROWNING  = 17;
   public final static byte IMM_RES_VULN_LIGHT     = 18;
   public final static byte IMM_RES_VULN_SOUND     = 19;
   public final static byte IMM_RES_VULN_WOOD      = 20;
   public final static byte IMM_RES_VULN_SILVER    = 21;
   public final static byte IMM_RES_VULN_IRON      = 22;
   public static FlagStructure[] immResVulnFlagTexts = new FlagStructure[23];

   public final static byte MOBILE_CORPSE_FLAG_EDIBLE        = 0;
   public final static byte MOBILE_CORPSE_FLAG_POISON        = 1;
   public final static byte MOBILE_CORPSE_FLAG_MAGICAL       = 2;
   public final static byte MOBILE_CORPSE_FLAG_INSTANT_DECAY = 3;
   public final static byte MOBILE_CORPSE_FLAG_OTHER         = 4;
   public static FlagStructure[] mobileCorpseFlagTexts  = new FlagStructure[5];

   public final static byte MOBILE_FORM_FLAG_ANIMAL     = 0;
   public final static byte MOBILE_FORM_FLAG_SENTIENT   = 1;
   public final static byte MOBILE_FORM_FLAG_UNDEAD     = 2;
   public final static byte MOBILE_FORM_FLAG_CONSTRUCT  = 3;
   public final static byte MOBILE_FORM_FLAG_MIST       = 4;
   public final static byte MOBILE_FORM_FLAG_INTANGIBLE = 5;
   public final static byte MOBILE_FORM_FLAG_BIPED      = 6;
   public final static byte MOBILE_FORM_FLAG_CENTAUR    = 7;
   public final static byte MOBILE_FORM_FLAG_INSECT     = 8;
   public final static byte MOBILE_FORM_FLAG_SPIDER     = 9;
   public final static byte MOBILE_FORM_FLAG_CRUSTACEAN = 10;
   public final static byte MOBILE_FORM_FLAG_WORM       = 11;
   public final static byte MOBILE_FORM_FLAG_BLOB       = 12;
   public final static byte MOBILE_FORM_FLAG_MAMMAL     = 13;
   public final static byte MOBILE_FORM_FLAG_BIRD       = 14;
   public final static byte MOBILE_FORM_FLAG_REPTILE    = 15;
   public final static byte MOBILE_FORM_FLAG_SNAKE      = 16;
   public final static byte MOBILE_FORM_FLAG_DRAGON     = 17;
   public final static byte MOBILE_FORM_FLAG_AMPHIBIAN  = 18;
   public final static byte MOBILE_FORM_FLAG_FISH       = 19;
   public final static byte MOBILE_FORM_FLAG_COLD_BLOOD = 20;
   public static FlagStructure[] mobileFormFlagTexts  = new FlagStructure[21];

   public final static byte MOBILE_PART_FLAG_HEAD      = 0;
   public final static byte MOBILE_PART_FLAG_ARMS      = 1;
   public final static byte MOBILE_PART_FLAG_LEGS      = 2;
   public final static byte MOBILE_PART_FLAG_HEART     = 3;
   public final static byte MOBILE_PART_FLAG_BRAINS    = 4;
   public final static byte MOBILE_PART_FLAG_GUTS      = 5;
   public final static byte MOBILE_PART_FLAG_HANDS     = 6;
   public final static byte MOBILE_PART_FLAG_FEET      = 7;
   public final static byte MOBILE_PART_FLAG_FINGERS   = 8;
   public final static byte MOBILE_PART_FLAG_EAR       = 9;
   public final static byte MOBILE_PART_FLAG_EYE       = 10;
   public final static byte MOBILE_PART_FLAG_TONGUE    = 11;
   public final static byte MOBILE_PART_FLAG_EYESTALK  = 12;
   public final static byte MOBILE_PART_FLAG_TENTACLES = 13;
   public final static byte MOBILE_PART_FLAG_FINS      = 14;
   public final static byte MOBILE_PART_FLAG_WINGS     = 15;
   public final static byte MOBILE_PART_FLAG_TAIL      = 16;
   public final static byte MOBILE_PART_FLAG_CLAWS     = 17;
   public final static byte MOBILE_PART_FLAG_FANGS     = 18;
   public final static byte MOBILE_PART_FLAG_HORNS     = 19;
   public final static byte MOBILE_PART_FLAG_SCALES    = 20;
   public final static byte MOBILE_PART_FLAG_TUSKS     = 21;
   public static FlagStructure[] mobilePartsFlagTexts = new FlagStructure[22];

   public static MobileLevelValues[] mobileLevelRecommendedValues = new MobileLevelValues[1+95]; //95 levels

   public final static byte WEAR_NONE         = 0  ;
   public final static byte WEAR_LIGHT        = 1  ;
   public final static byte WEAR_LEFT_FINGER  = 2  ;
   public final static byte WEAR_RIGHT_FINGER = 3  ;
   public final static byte WEAR_NECK_1       = 4  ;
   public final static byte WEAR_NECK_2       = 5  ;
   public final static byte WEAR_TORSO        = 6  ;
   public final static byte WEAR_HEAD         = 7  ;
   public final static byte WEAR_LEGS         = 8  ;
   public final static byte WEAR_FEET         = 9  ;
   public final static byte WEAR_HANDS        = 10  ;
   public final static byte WEAR_ARMS         = 11  ;
   public final static byte WEAR_SHIELD       = 12  ;
   public final static byte WEAR_ABOUT_BODY   = 13  ;
   public final static byte WEAR_WAIST        = 14  ;
   public final static byte WEAR_LEFT_WRIST   = 15  ;
   public final static byte WEAR_RIGHT_WRIST  = 16  ;
   public final static byte WEAR_WIELD        = 17  ;
   public final static byte WEAR_HELD         = 18  ;
   public final static byte WEAR_FLOATING     = 19  ;
   public final static byte WEAR_TATOO        = 20  ;
   public final static byte WEAR_SECOND_WIELD = 21  ;
   public final static byte WEAR_STUCK_IN     = 22  ;
   public static FlagStructure[] wearTexts = new FlagStructure[23];

   public static String[] objectMaterialTexts =
   {"none", "unknown","plastic","slime","jelly","wax","rubber","oil","balm","cream","leather","kid leather","fur",
    "snakeskin","gut","food","meat","bread","wood","hardwood","softwood","bamboo","ebony","cork",
    "horn","light","sponge","elastic","silk","satin","lace", "laen", "wool", "linen","canvas","cloth",
    "velvet","felt","paper","parchment","vellum","hemp","feathers","webbing","cardboard", "steel",
    "gold","silver","mithril","platinum","iron","lead","copper","electrum","bronze","brass","wire",
    "tin","pewter","metal","dark","porcelain","ivory","marble","stone","quartz","corundum","flint",
    "lodestone","granite","enamel","obsidian","adamantite","glass","pottery","crystal","ice","bone",
    "shell","coral","energy","fire","air","water","acid","coal","sandstone","clay","ash","earth",
    "diamond","etherealness","nothingness","dragonscale","blue dragonscale","black dragonscale",
    "white dragonscale","red dragonscale"};

   public final static byte LIQUID_WATER            = 0;
   public final static byte LIQUID_BEER             = 1;
   public final static byte LIQUID_RED_WINE         = 2;
   public final static byte LIQUID_ALE              = 3;
   public final static byte LIQUID_DARK_ALE         = 4;
   public final static byte LIQUID_WHISKY           = 5;
   public final static byte LIQUID_LEMONADE         = 6;
   public final static byte LIQUID_FIREBREATHER     = 7;
   public final static byte LIQUID_LOCAL_SPECIALITY = 8;
   public final static byte LIQUID_SLIME_MOLD_JUICE = 9;
   public final static byte LIQUID_MILK             = 10;
   public final static byte LIQUID_TEA              = 11;
   public final static byte LIQUID_COFFEE           = 12;
   public final static byte LIQUID_BLOOD            = 13;
   public final static byte LIQUID_SALT_WATER       = 14;
   public final static byte LIQUID_COKE             = 15;
   public final static byte LIQUID_ROOT_BEER        = 16;
   public final static byte LIQUID_ELVISH_WINE      = 17;
   public final static byte LIQUID_WHITE_WINE       = 18;
   public final static byte LIQUID_CHAMPAGNE        = 19;
   public final static byte LIQUID_MEAD             = 20;
   public final static byte LIQUID_ROSE_WINE        = 21;
   public final static byte LIQUID_BENEDICTINE_WINE = 22;
   public final static byte LIQUID_VODKA            = 23;
   public final static byte LIQUID_CRANBERRY_JUICE  = 24;
   public final static byte LIQUID_ORANGE_JUICE     = 25;
   public final static byte LIQUID_ABSINTHE         = 26;
   public final static byte LIQUID_BRANDY           = 27;
   public final static byte LIQUID_AQUAVIT          = 28;
   public final static byte LIQUID_SCHNAPPS         = 29;
   public final static byte LIQUID_ICEWINE          = 30;
   public final static byte LIQUID_AMONTILLADO      = 31;
   public final static byte LIQUID_SHERRY           = 32;
   public final static byte LIQUID_FRAMBOISE        = 33;
   public final static byte LIQUID_RUM              = 34;
   public final static byte LIQUID_CORDIAL          = 35;
   public static LiquidStructure[] liquids = new LiquidStructure[36];

   public final static byte OBJECT_TYPE_NONE      = 0;
   public final static byte OBJECT_TYPE_LIGHT     = 1;
   public final static byte OBJECT_TYPE_SCROLL    = 2;
   public final static byte OBJECT_TYPE_WAND      = 3;
   public final static byte OBJECT_TYPE_STAFF     = 4;
   public final static byte OBJECT_TYPE_WEAPON    = 5;
   public final static byte OBJECT_TYPE_TREASURE  = 6;
   public final static byte OBJECT_TYPE_ARMOR     = 7;
   public final static byte OBJECT_TYPE_POTION    = 8;
   public final static byte OBJECT_TYPE_CLOTHING  = 9;
   public final static byte OBJECT_TYPE_FURNITURE = 10;
   public final static byte OBJECT_TYPE_TRASH     = 11;
   public final static byte OBJECT_TYPE_CONTAINER = 12;
   public final static byte OBJECT_TYPE_DRINK_CON = 13;
   public final static byte OBJECT_TYPE_KEY       = 14;
   public final static byte OBJECT_TYPE_FOOD      = 15;
   public final static byte OBJECT_TYPE_MONEY     = 16;
   public final static byte OBJECT_TYPE_BOAT      = 17;
   public final static byte OBJECT_TYPE_CORPSE_NPC= 18;
   public final static byte OBJECT_TYPE_CORPSE_PC = 19;
   public final static byte OBJECT_TYPE_FOUNTAIN  = 20;
   public final static byte OBJECT_TYPE_PILL      = 21;
   public final static byte OBJECT_TYPE_PROTECT   = 22;
   public final static byte OBJECT_TYPE_MAP       = 23;
   public final static byte OBJECT_TYPE_PORTAL    = 24;
   public final static byte OBJECT_TYPE_WARP_STONE= 25;
   public final static byte OBJECT_TYPE_ROOM_KEY  = 26;
   public final static byte OBJECT_TYPE_GEM       = 27;
   public final static byte OBJECT_TYPE_JEWELRY   = 28;
   public final static byte OBJECT_TYPE_JUKEBOX   = 29;
   public final static byte OBJECT_TYPE_TATOO     = 30;
   public final static byte OBJECT_TYPE_BOMB      = 31;
   public static FlagStructure[] objectTypeTexts = new FlagStructure[32];

   public final static byte OBJECT_BULK_CLOTHING          = 0;
   public final static byte OBJECT_BULK_BUCKLER           = 1;
   public final static byte OBJECT_BULK_HARD_LEATHER      = 2;
   public final static byte OBJECT_BULK_SMALL_SHIELD      = 3;
   public final static byte OBJECT_BULK_CHAINMAIL         = 4;
   public final static byte OBJECT_BULK_MEDIUM_SHIELD     = 5;
   public final static byte OBJECT_BULK_PLATEMAIL         = 6;
   public final static byte OBJECT_BULK_KITE_SHIELD       = 7;
   public final static byte OBJECT_BULK_LIGHT_PLATE_ARMOR = 8;
   public final static byte OBJECT_BULK_TOWER_SHIELD      = 9;
   public final static byte OBJECT_BULK_HEAVY_PLATE_ARMOR = 10;
   public static FlagStructure[] objectBulkTexts = new FlagStructure[11];

   public final static byte OBJECT_PORTAL_GATEFLAG_NORMAL_EXIT = 0;
   public final static byte OBJECT_PORTAL_GATEFLAG_NO_CURSE    = 1;
   public final static byte OBJECT_PORTAL_GATEFLAG_GO_WITH     = 2;
   public final static byte OBJECT_PORTAL_GATEFLAG_GATE_BUGGY  = 3;
   public final static byte OBJECT_PORTAL_GATEFLAG_RANDOM      = 4;
   public static FlagStructure[] objectPortalGateFlagTexts = new FlagStructure[5];

   public final static byte OBJECT_FURNITURE_FLAG_STAND_AT   = 0;
   public final static byte OBJECT_FURNITURE_FLAG_STAND_ON   = 1;
   public final static byte OBJECT_FURNITURE_FLAG_STAND_IN   = 2;
   public final static byte OBJECT_FURNITURE_FLAG_SIT_AT     = 3;
   public final static byte OBJECT_FURNITURE_FLAG_SIT_ON     = 4;
   public final static byte OBJECT_FURNITURE_FLAG_SIT_IN     = 5;
   public final static byte OBJECT_FURNITURE_FLAG_REST_AT    = 6;
   public final static byte OBJECT_FURNITURE_FLAG_REST_ON    = 7;
   public final static byte OBJECT_FURNITURE_FLAG_REST_IN    = 8;
   public final static byte OBJECT_FURNITURE_FLAG_SLEEP_AT   = 9;
   public final static byte OBJECT_FURNITURE_FLAG_SLEEP_ON   = 10;
   public final static byte OBJECT_FURNITURE_FLAG_SLEEP_IN   = 11;
   public final static byte OBJECT_FURNITURE_FLAG_PUT_AT     = 12;
   public final static byte OBJECT_FURNITURE_FLAG_PUT_ON     = 13;
   public final static byte OBJECT_FURNITURE_FLAG_PUT_IN     = 14;
   public final static byte OBJECT_FURNITURE_FLAG_PUT_INSIDE = 15;
   public final static byte OBJECT_FURNITURE_FLAG_WRITE_ON   = 16;
   public static FlagStructure[] objectFurnitureFlagTexts = new FlagStructure[17];

   public final static byte EXTRAFLAG_GLOWING      = 0;
   public final static byte EXTRAFLAG_HUMMING      = 1;
   public final static byte EXTRAFLAG_DARK         = 2;
   public final static byte EXTRAFLAG_LOCK         = 3;
   public final static byte EXTRAFLAG_EVIL         = 4;
   public final static byte EXTRAFLAG_INVISIBLE    = 5;
   public final static byte EXTRAFLAG_MAGIC        = 6;
   public final static byte EXTRAFLAG_NODROP       = 7;
   public final static byte EXTRAFLAG_BLESS        = 8;
   public final static byte EXTRAFLAG_ANTI_GOOD    = 9;
   public final static byte EXTRAFLAG_ANTI_EVIL    = 10;
   public final static byte EXTRAFLAG_ANTI_NEUTRAL = 11;
   public final static byte EXTRAFLAG_NOREMOVE     = 12;
   public final static byte EXTRAFLAG_INVENTORY    = 13;
   public final static byte EXTRAFLAG_NOPURGE      = 14;
   public final static byte EXTRAFLAG_ROT_DEATH    = 15;
   public final static byte EXTRAFLAG_VIS_DEATH    = 16;
   public final static byte EXTRAFLAG_NOSAC        = 17;
   public final static byte EXTRAFLAG_NONMETAL     = 18;
   public final static byte EXTRAFLAG_NOLOCATE     = 19;
   public final static byte EXTRAFLAG_MELT_DROP    = 20;
   public final static byte EXTRAFLAG_HAD_TIMER    = 21;
   public final static byte EXTRAFLAG_SELL_EXTRACT = 22;
   public final static byte EXTRAFLAG_BURN_PROOF   = 23;
   public final static byte EXTRAFLAG_NOUNCURSE    = 24;
   public final static byte EXTRAFLAG_NOSELL       = 25;
   public final static byte EXTRAFLAG_VIS_ROT      = 26;
   public static FlagStructure[] extraFlagTexts = new FlagStructure[27];

   public final static byte WEAR_FLAG_TAKE       = 0;
   public final static byte WEAR_FLAG_FINGER     = 1;
   public final static byte WEAR_FLAG_NECK       = 2;
   public final static byte WEAR_FLAG_BODY       = 3;
   public final static byte WEAR_FLAG_HEAD       = 4;
   public final static byte WEAR_FLAG_LEGS       = 5;
   public final static byte WEAR_FLAG_FEET       = 6;
   public final static byte WEAR_FLAG_HANDS      = 7;
   public final static byte WEAR_FLAG_ARMS       = 8;
   public final static byte WEAR_FLAG_SHIELD     = 9;
   public final static byte WEAR_FLAG_ABOUT_BODY = 10;
   public final static byte WEAR_FLAG_WAIST      = 11;
   public final static byte WEAR_FLAG_WRIST      = 12;
   public final static byte WEAR_FLAG_WIELD      = 13;
   public final static byte WEAR_FLAG_HOLD       = 14;
   public final static byte WEAR_FLAG_NOSAC      = 15;
   public final static byte WEAR_FLAG_FLOAT      = 16;
   public final static byte WEAR_FLAG_TATOO      = 17;
   public final static byte WEAR_FLAG_TWO_HANDS  = 18;
   public static FlagStructure[] wearFlagTexts = new FlagStructure[19];

   public final static byte APPLY_NONE          = 0;
   public final static byte APPLY_STRENGTH      = 1;
   public final static byte APPLY_DEXTERITY     = 2;
   public final static byte APPLY_INTELLIGENCE  = 3;
   public final static byte APPLY_WISDOM        = 4;
   public final static byte APPLY_CONSTITUTION  = 5;
   public final static byte APPLY_CHARISMA      = 6;
   public final static byte APPLY_CLASS         = 7;
   public final static byte APPLY_LEVEL         = 8;
   public final static byte APPLY_AGE           = 9;
   public final static byte APPLY_HEIGHT        = 10;
   public final static byte APPLY_WEIGHT        = 11;
   public final static byte APPLY_MANA          = 12;
   public final static byte APPLY_HITPOINTS     = 13;
   public final static byte APPLY_MOVEMENT      = 14;
   public final static byte APPLY_GOLD          = 15;
   public final static byte APPLY_EXP           = 16;
   public final static byte APPLY_ARMOR_CLASS   = 17;
   public final static byte APPLY_HITROLL       = 18;
   public final static byte APPLY_DAMROLL       = 19;
   public final static byte APPLY_SAVES		= 20;
   public final static byte APPLY_SAVING_PARA	= 20;
   public final static byte APPLY_SAVING_ROD	= 21;
   public final static byte APPLY_SAVING_PETRI	= 22;
   public final static byte APPLY_SAVING_BREATH	= 23;
   public final static byte APPLY_SAVING_SPELL	= 24;
   public final static byte APPLY_SPELL_AFFECT	= 25;
   public final static byte APPLY_SIZE		= 26;
   public final static byte APPLY_SEX           = 27;
   public static FlagStructure[] applyTexts = new FlagStructure[28];

   public final static byte AFF_BLIND       = 0;
   public final static byte AFF_INVISIBLE   = 1;
   public final static byte AFF_IMP_INVIS   = 2;
   public final static byte AFF_FADE        = 3;
   public final static byte AFF_SCREAM      = 4;
   public final static byte AFF_BLOODTHIRST = 5;
   public final static byte AFF_STUN        = 6;
   public final static byte AFF_SANCTUARY   = 7;
   public final static byte AFF_FAERIE_FIRE = 8;
   public final static byte AFF_INFRARED    = 9;
   public final static byte AFF_CURSE       = 10;
   public final static byte AFF_CORRUPTION  = 11;
   public final static byte AFF_POISON      = 12;
   public final static byte AFF_PROTECT_EVIL= 13;
   public final static byte AFF_PROTECT_GOOD= 14;
   public final static byte AFF_SNEAK       = 15;
   public final static byte AFF_HIDE        = 16;
   public final static byte AFF_SLEEP       = 17;
   public final static byte AFF_CHARM       = 18;
   public final static byte AFF_FLYING      = 19;
   public final static byte AFF_PASS_DOOR   = 20;
   public final static byte AFF_HASTE       = 21;
   public final static byte AFF_CALM        = 22;
   public final static byte AFF_PLAGUE      = 23;
   public final static byte AFF_WEAKEN      = 24;
   public final static byte AFF_WEAKEN_STUN = 25;
   public final static byte AFF_BERSERK     = 26;
   public final static byte AFF_SWIM        = 27;
   public final static byte AFF_REGENERATION= 28;
   public final static byte AFF_SLOW        = 29;
   public final static byte AFF_CAMOUFLAGE  = 30;
   public final static byte AFF_FLAMING     = 31;
   public final static byte AFF_HOLD        = 32;
   public final static byte AFF_PROTECT     = 33;
   public final static byte AFF_PARALYSIS   = 34;
   public static FlagStructure[] affTexts = new FlagStructure[35];

   public final static byte DET_DETECT_IMP_INVIS = 0;//35;
   public final static byte DET_DETECT_FADE      = 1;//36;
   public final static byte DET_DETECT_EVIL      = 2;//37;
   public final static byte DET_DETECT_INVISIBLE = 3;//38;
   public final static byte DET_DETECT_MAGIC     = 4;//39;
   public final static byte DET_DETECT_HIDDEN    = 5;//40;
   public final static byte DET_DETECT_GOOD      = 6;//41;
   public final static byte DET_DETECT_SNEAK     = 7;//42;
   public final static byte DET_DETECT_UNDEAD    = 8;//43;
   public final static byte DET_DARK_VISION      = 9;//44;
   public final static byte DET_ACUTE_VISION     = 10;//45;
   public static FlagStructure[] detTexts = new FlagStructure[11];

   public final static int WELL_KNOWN_MOB_CITYGUARD        = 0;
   public final static int WELL_KNOWN_MOB_VAGABOND         = 1;
   public final static int WELL_KNOWN_MOB_CAT              = 2;
   public final static int WELL_KNOWN_MOB_FIDO             = 3;
   public final static int WELL_KNOWN_MOB_SAGE             = 4;
   public final static int WELL_KNOWN_MOB_VAMPIRE          = 5;
   public final static int WELL_KNOWN_MOB_SHADOW           = 6;
   public final static int WELL_KNOWN_MOB_SPECIAL_GUARD    = 7;
   public final static int WELL_KNOWN_MOB_BEAR             = 8;
   public final static int WELL_KNOWN_MOB_DEMON            = 9;
   public final static int WELL_KNOWN_MOB_NIGHTWALKER      = 10;
   public final static int WELL_KNOWN_MOB_STALKER          = 11;
   public final static int WELL_KNOWN_MOB_SQUIRE           = 12;
   public final static int WELL_KNOWN_MOB_MIRROR_IMAGE     = 13;
   public final static int WELL_KNOWN_MOB_UNDEAD           = 14;
   public final static int WELL_KNOWN_MOB_LION             = 15;
   public final static int WELL_KNOWN_MOB_WOLF             = 16;
   public final static int WELL_KNOWN_MOB_SUM_SHADOW       = 17;
   public final static int WELL_KNOWN_MOB_LESSER_GOLEM     = 18;
   public final static int WELL_KNOWN_MOB_STONE_GOLEM      = 19;
   public final static int WELL_KNOWN_MOB_IRON_GOLEM       = 20;
   public final static int WELL_KNOWN_MOB_ADAMANTITE_GOLEM = 21;
   public final static int WELL_KNOWN_MOB_HUNTER           = 22;
   public final static int WELL_KNOWN_MOB_PATROLMAN        = 23;
   public final static int WELL_KNOWN_MOB_TROLLS           = 24;
   public final static int WELL_KNOWN_MOB_OGRES            = 25;
   public static WellKnownMob[] wellKnownMobs = new WellKnownMob[26];

   public final static int WELL_KNOWN_OBJ_SILVER_ONE        = 0;
   public final static int WELL_KNOWN_OBJ_GOLD_ONE          = 1;
   public final static int WELL_KNOWN_OBJ_GOLD_SOME         = 2;
   public final static int WELL_KNOWN_OBJ_SILVER_SOME       = 3;
   public final static int WELL_KNOWN_OBJ_COINS             = 4;
   public final static int WELL_KNOWN_OBJ_CORPSE_NPC        = 5;
   public final static int WELL_KNOWN_OBJ_CORPSE_PC         = 6;
   public final static int WELL_KNOWN_OBJ_SEVERED_HEAD      = 7;
   public final static int WELL_KNOWN_OBJ_TORN_HEAD         = 8;
   public final static int WELL_KNOWN_OBJ_SLICED_ARM        = 9;
   public final static int WELL_KNOWN_OBJ_SLICED_LEG        = 10;
   public final static int WELL_KNOWN_OBJ_GUTS              = 11;
   public final static int WELL_KNOWN_OBJ_BRAINS            = 12;
   public final static int WELL_KNOWN_OBJ_MUSHROOMS         = 13;
   public final static int WELL_KNOWN_OBJ_LIGHT_BALL        = 14;
   public final static int WELL_KNOWN_OBJ_SPRING            = 15;
   public final static int WELL_KNOWN_OBJ_DISC              = 16;
   public final static int WELL_KNOWN_OBJ_PORTAL            = 17;
   public final static int WELL_KNOWN_OBJ_ROSE              = 18;
   public final static int WELL_KNOWN_OBJ_WHITE_ROSE        = 19;
   public final static int WELL_KNOWN_OBJ_PIT               = 20;
   public final static int WELL_KNOWN_OBJ_SCHOOL_MACE       = 21;
   public final static int WELL_KNOWN_OBJ_SCHOOL_DAGGER     = 22;
   public final static int WELL_KNOWN_OBJ_SCHOOL_SWORD      = 23;
   public final static int WELL_KNOWN_OBJ_SCHOOL_SPEAR      = 24;
   public final static int WELL_KNOWN_OBJ_SCHOOL_STAFF      = 25;
   public final static int WELL_KNOWN_OBJ_SCHOOL_AXE        = 26;
   public final static int WELL_KNOWN_OBJ_SCHOOL_FLAIL      = 27;
   public final static int WELL_KNOWN_OBJ_SCHOOL_WHIP       = 28;
   public final static int WELL_KNOWN_OBJ_SCHOOL_POLEARM    = 29;
   public final static int WELL_KNOWN_OBJ_SCHOOL_BOW        = 30;
   public final static int WELL_KNOWN_OBJ_SCHOOL_LANCE      = 31;
   public final static int WELL_KNOWN_OBJ_SCHOOL_VEST       = 32;
   public final static int WELL_KNOWN_OBJ_SCHOOL_SHIELD     = 33;
   public final static int WELL_KNOWN_OBJ_SCHOOL_BANNER     = 34;
   public final static int WELL_KNOWN_OBJ_MAP               = 35;
   public final static int WELL_KNOWN_OBJ_NMAP1             = 36;
   public final static int WELL_KNOWN_OBJ_NMAP2             = 37;
   public final static int WELL_KNOWN_OBJ_MAP_NT            = 38;
   public final static int WELL_KNOWN_OBJ_MAP_OFCOL         = 39;
   public final static int WELL_KNOWN_OBJ_MAP_SM            = 40;
   public final static int WELL_KNOWN_OBJ_MAP_TITAN         = 41;
   public final static int WELL_KNOWN_OBJ_MAP_OLD           = 42;
   public final static int WELL_KNOWN_OBJ_WHISTLE           = 43;
   public final static int WELL_KNOWN_OBJ_POTION_VIAL       = 44;
   public final static int WELL_KNOWN_OBJ_STEAK             = 45;
   public final static int WELL_KNOWN_OBJ_RANGER_STAFF      = 46;
   public final static int WELL_KNOWN_OBJ_RANGER_ARROW      = 47;
   public final static int WELL_KNOWN_OBJ_RANGER_BOW        = 48;
   public final static int WELL_KNOWN_OBJ_RULER_BADGE       = 49;
   public final static int WELL_KNOWN_OBJ_RULER_SHIELD1     = 50;
   public final static int WELL_KNOWN_OBJ_RULER_SHIELD2     = 51;
   public final static int WELL_KNOWN_OBJ_RULER_SHIELD3     = 52;
   public final static int WELL_KNOWN_OBJ_RULER_SHIELD4     = 53;
   public final static int WELL_KNOWN_OBJ_CHAOS_BLADE       = 54;
   public final static int WELL_KNOWN_OBJ_DRAGONDAGGER      = 55;
   public final static int WELL_KNOWN_OBJ_DRAGONMACE        = 56;
   public final static int WELL_KNOWN_OBJ_PLATE             = 57;
   public final static int WELL_KNOWN_OBJ_DRAGONSWORD       = 58;
   public final static int WELL_KNOWN_OBJ_DRAGONLANCE       = 59;
   public final static int WELL_KNOWN_OBJ_PONCHO            = 60;
   public final static int WELL_KNOWN_OBJ_BATTLE_THRONE     = 61;
   public final static int WELL_KNOWN_OBJ_SHALAFI_ALTAR     = 62;
   public final static int WELL_KNOWN_OBJ_CHAOS_ALTAR       = 63;
   public final static int WELL_KNOWN_OBJ_INVADER_SKULL     = 64;
   public final static int WELL_KNOWN_OBJ_KNIGHT_ALTAR      = 65;
   public final static int WELL_KNOWN_OBJ_RULER_STAND       = 66;
   public final static int WELL_KNOWN_OBJ_LIONS_ALTAR       = 67;
   public final static int WELL_KNOWN_OBJ_HUNTER_ALTAR      = 68;
   public final static int WELL_KNOWN_OBJ_POTION_SILVER     = 69;
   public final static int WELL_KNOWN_OBJ_POTION_GOLDEN     = 70;
   public final static int WELL_KNOWN_OBJ_POTION_SWIRLING   = 71;
   public final static int WELL_KNOWN_OBJ_KATANA_SWORD      = 72;
   public final static int WELL_KNOWN_OBJ_EYED_SWORD        = 73;
   public final static int WELL_KNOWN_OBJ_FIRE_SHIELD       = 74;
   public final static int WELL_KNOWN_OBJ_MAGIC_JAR         = 75;
   public final static int WELL_KNOWN_OBJ_HAMMER            = 76;
   public final static int WELL_KNOWN_OBJ_FINE              = 77;
   public final static int WELL_KNOWN_OBJ_CHUNK_IRON        = 78;
   public final static int WELL_KNOWN_OBJ_TATTOO_APOLLON    = 79;
   public final static int WELL_KNOWN_OBJ_TATTOO_ZEUS       = 80;
   public final static int WELL_KNOWN_OBJ_TATTOO_SIEBELE    = 81;
   public final static int WELL_KNOWN_OBJ_TATTOO_HEPHAESTUS = 82;
   public final static int WELL_KNOWN_OBJ_TATTOO_EHRUMEN    = 83;
   public final static int WELL_KNOWN_OBJ_TATTOO_AHRUMAZDA  = 84;
   public final static int WELL_KNOWN_OBJ_TATTOO_DEIMOS     = 85;
   public final static int WELL_KNOWN_OBJ_TATTOO_PHOBOS     = 86;
   public final static int WELL_KNOWN_OBJ_TATTOO_ODIN       = 87;
   public final static int WELL_KNOWN_OBJ_TATTOO_MARS       = 88;
   public final static int WELL_KNOWN_OBJ_TATTOO_ATHENA     = 89;
   public final static int WELL_KNOWN_OBJ_TATTOO_GOKTENGRI  = 90;
   public final static int WELL_KNOWN_OBJ_TATTOO_HERA       = 91;
   public final static int WELL_KNOWN_OBJ_TATTOO_VENUS      = 92;
   public final static int WELL_KNOWN_OBJ_TATTOO_ARES       = 93;
   public final static int WELL_KNOWN_OBJ_TATTOO_PROMETHEUS = 94;
   public final static int WELL_KNOWN_OBJ_TATTOO_EROS       = 95;
   public static WellKnownObj[] wellKnownObjs = new WellKnownObj[96];


   public AnatoliaConsts()
   {
     spellTexts[SPELL_AMNESIA]             = new FlagStructure("amnesia",null,"538");
     spellTexts[SPELL_ATTRACT_OTHER]       = new FlagStructure("attract other",null,"580");
     spellTexts[SPELL_DRAGON_BREATH]       = new FlagStructure("dragon breath",null,"563");
     spellTexts[SPELL_DRAGON_STRENGTH]     = new FlagStructure("dragon strength",null,"562");
     spellTexts[SPELL_GENERAL_PURPOSE]     = new FlagStructure("general purpose",null,"401");
     spellTexts[SPELL_HIGH_EXPLOSIVE]      = new FlagStructure("high explosive",null,"402");
     spellTexts[SPELL_LOVE_POTION]         = new FlagStructure("love potion",null,"666");
     spellTexts[SPELL_REMOVE_TATTOO]       = new FlagStructure("remove tattoo",null,"552");
     spellTexts[SPELL_TATTOO]              = new FlagStructure("tattoo",null,"551");
     spellTexts[SPELL_ACETUM_PRIMUS]       = new FlagStructure("acetum primus",null,"654");
     spellTexts[SPELL_ACID_ARROW]          = new FlagStructure("acid arrow",null,"644");
     spellTexts[SPELL_ACID_BREATH]         = new FlagStructure("acid breath",null,"200");
     spellTexts[SPELL_ACID_BLAST]          = new FlagStructure("acid blast",null,"70");
     spellTexts[SPELL_ACUTE_VISION]        = new FlagStructure("acute vision",null,"514");
     spellTexts[SPELL_ADAMANTITE_GOLEM]    = new FlagStructure("adamantite golem",null,"665");
     spellTexts[SPELL_AID]                 = new FlagStructure("aid",null,"680");
     spellTexts[SPELL_ANIMATE_DEAD]        = new FlagStructure("animate dead",null,"581");
     spellTexts[SPELL_ARMOR]               = new FlagStructure("armor",null,"1");
     spellTexts[SPELL_ASSIST]              = new FlagStructure("assist",null,"670");
     spellTexts[SPELL_ASTRAL_WALK]         = new FlagStructure("astral walk",null,"622");
     spellTexts[SPELL_BARK_SKIN]           = new FlagStructure("bark skin",null,"515");
     spellTexts[SPELL_BLACK_DEATH]         = new FlagStructure("black death",null,"677");
     spellTexts[SPELL_BLADE_BARRIER]       = new FlagStructure("blade barrier",null,"679");
     spellTexts[SPELL_BLESS]               = new FlagStructure("bless",null,"3");
     spellTexts[SPELL_BLESS_WEAPON]        = new FlagStructure("bless weapon",null,"637");
     spellTexts[SPELL_BLINDNESS]           = new FlagStructure("blindness",null,"4");
     spellTexts[SPELL_BLUEFIRE]            = new FlagStructure("bluefire",null,"660");
     spellTexts[SPELL_BURNING_HANDS]       = new FlagStructure("burning hands",null,"5");
     spellTexts[SPELL_CALL_LIGHTNING]      = new FlagStructure("call lightning",null,"6");
     spellTexts[SPELL_CALM]                = new FlagStructure("calm ",null,"509");
     spellTexts[SPELL_CANCELLATION]        = new FlagStructure("cancellation",null,"507");
     spellTexts[SPELL_CAUSE_CRITICAL]      = new FlagStructure("cause critical",null,"63");
     spellTexts[SPELL_CAUSE_LIGHT]         = new FlagStructure("cause light",null,"62");
     spellTexts[SPELL_CAUSE_SERIOUS]       = new FlagStructure("cause serious",null,"64");
     spellTexts[SPELL_CAUSTIC_FONT]        = new FlagStructure("caustic font",null,"653");
     spellTexts[SPELL_CHAIN_LIGHTNING]     = new FlagStructure("chain lightning",null,"500");
     spellTexts[SPELL_CHARM_PERSON]        = new FlagStructure("person",null,"7");
     spellTexts[SPELL_CHILL_TOUCH]         = new FlagStructure("charm touch",null,"8");
     spellTexts[SPELL_COLOUR_SPRAY]        = new FlagStructure("colour spray",null,"10");
     spellTexts[SPELL_CONTINUAL_LIGHT]     = new FlagStructure("continual light",null,"57");
     spellTexts[SPELL_CONTROL_UNDEAD]      = new FlagStructure("control undead",null,"669");
     spellTexts[SPELL_CONTROL_WEATHER]     = new FlagStructure("control weather",null,"11");
     spellTexts[SPELL_CORRUPTION]          = new FlagStructure("corruption",null,"671");
     spellTexts[SPELL_CREATE_FOOD]         = new FlagStructure("create food",null,"12");
     spellTexts[SPELL_CREATE_ROSE]         = new FlagStructure("create rose",null,"511");
     spellTexts[SPELL_CREATE_SPRING]       = new FlagStructure("create spring",null,"80");
     spellTexts[SPELL_CREATE_WATER]        = new FlagStructure("create water",null,"13");
     spellTexts[SPELL_CURE_BLINDNESS]      = new FlagStructure("cure blindness",null,"14");
     spellTexts[SPELL_CURE_CRITICAL]       = new FlagStructure("cure critical",null,"15");
     spellTexts[SPELL_CURE_DISEASE]        = new FlagStructure("cure disease",null,"501");
     spellTexts[SPELL_CURE_LIGHT]          = new FlagStructure("cure light",null,"16");
     spellTexts[SPELL_CURE_POISON]         = new FlagStructure("cure poison",null,"43");
     spellTexts[SPELL_CURE_SERIOUS]        = new FlagStructure("cure serious",null,"61");
     spellTexts[SPELL_CURSE]               = new FlagStructure("curse",null,"17");
     spellTexts[SPELL_CURSED_LANDS]        = new FlagStructure("cursed lands",null,"675");
     spellTexts[SPELL_DEADLY_VENOM]        = new FlagStructure("deadly venom",null,"674");
     spellTexts[SPELL_DEAFEN]              = new FlagStructure("deafen",null,"570");
     spellTexts[SPELL_DEMONFIRE]           = new FlagStructure("demonfire",null,"505");
     spellTexts[SPELL_DESERT_FIST]         = new FlagStructure("desert fist",null,"681");
     spellTexts[SPELL_DETECT_EVIL]         = new FlagStructure("detect evil",null,"18");
     spellTexts[SPELL_DETECT_GOOD]         = new FlagStructure("detect good",null,"513");
     spellTexts[SPELL_HIDDEN_PERCEPTION]   = new FlagStructure("hidden perception",null,"44");
     spellTexts[SPELL_DETECT_INVIS]        = new FlagStructure("detect invis",null,"19");
     spellTexts[SPELL_DETECT_MAGIC]        = new FlagStructure("detect magic",null,"20");
     spellTexts[SPELL_DETECT_POISON]       = new FlagStructure("detect poison",null,"21");
     spellTexts[SPELL_DETECT_UNDEAD]       = new FlagStructure("detect undead",null,"594");
     spellTexts[SPELL_DISINTEGRATE]        = new FlagStructure("disintegrate",null,"574");
     spellTexts[SPELL_DISPEL_EVIL]         = new FlagStructure("dispel evil",null,"22");
     spellTexts[SPELL_DISPEL_GOOD]         = new FlagStructure("dispel good",null,"512");
     spellTexts[SPELL_DISPEL_MAGIC]        = new FlagStructure("dispel magic",null,"59");
     spellTexts[SPELL_DISRUPTION]          = new FlagStructure("disruption",null,"648");
     spellTexts[SPELL_DRAGON_SKIN]         = new FlagStructure("dragon skin",null,"612");
     spellTexts[SPELL_DRAGONS_BREATH]      = new FlagStructure("dragons breath",null,"576");
     spellTexts[SPELL_EARTHQUAKE]          = new FlagStructure("earthquake",null,"23");
     spellTexts[SPELL_ENCHANT_ARMOR]       = new FlagStructure("enchant armor",null,"510");
     spellTexts[SPELL_ENCHANT_WEAPON]      = new FlagStructure("enchant werapon",null,"24");
     spellTexts[SPELL_ENERGY_DRAIN]        = new FlagStructure("energy drain",null,"25");
     spellTexts[SPELL_ENHANCED_ARMOR]      = new FlagStructure("enhanced armor",null,"583");
     spellTexts[SPELL_ETHERAL_FIST]        = new FlagStructure("etheral fist",null,"645");
     spellTexts[SPELL_FAERIE_FIRE]         = new FlagStructure("faerie fire",null,"72");
     spellTexts[SPELL_FAERIE_FOG]          = new FlagStructure("faerie fog",null,"73");
     spellTexts[SPELL_FARSIGHT]            = new FlagStructure("farsight",null,"521");
     spellTexts[SPELL_FEAR]                = new FlagStructure("fear",null,"598");
     spellTexts[SPELL_FIREBALL]            = new FlagStructure("fireball",null,"26");
     spellTexts[SPELL_FIRE_BREATH]         = new FlagStructure("fire breath",null,"201");
     spellTexts[SPELL_FIRE_PROOF]          = new FlagStructure("fireproof",null,"523");
     spellTexts[SPELL_FIRE_SHIELD]         = new FlagStructure("fire shield",null,"601");
     spellTexts[SPELL_FLAMESTRIKE]         = new FlagStructure("flamestrike",null,"65");
     spellTexts[SPELL_FLOATING_DISC]       = new FlagStructure("floating disc",null,"522");
     spellTexts[SPELL_FLY]                 = new FlagStructure("fly",null,"56");
     spellTexts[SPELL_FRENZY]              = new FlagStructure("frenzy",null,"504");
     spellTexts[SPELL_FROST_BREATH]        = new FlagStructure("frost breath",null,"202");
     spellTexts[SPELL_GALVANIC_WHIP]       = new FlagStructure("galvanic whip",null,"655");
     spellTexts[SPELL_GAS_BREATH]          = new FlagStructure("gas breath",null,"203");
     spellTexts[SPELL_GATE]                = new FlagStructure("gate",null,"83");
     spellTexts[SPELL_GIANT_STRENGTH]      = new FlagStructure("giant strength",null,"39");
     spellTexts[SPELL_GROUP_DEFENSE]       = new FlagStructure("group defense",null,"586");
     spellTexts[SPELL_GROUP_HEAL]          = new FlagStructure("group heal",null,"642");
     spellTexts[SPELL_HAND_OF_UNDEAD]      = new FlagStructure("hand of undead",null,"647");
     spellTexts[SPELL_HARM]                = new FlagStructure("harm",null,"27");
     spellTexts[SPELL_HASTE]               = new FlagStructure("haste",null,"502");
     spellTexts[SPELL_HEAL]                = new FlagStructure("heal",null,"28");
     spellTexts[SPELL_HEALING_LIGHT]       = new FlagStructure("healing light",null,"613");
     spellTexts[SPELL_HEAT_METAL]          = new FlagStructure("heat metal",null,"516");
     spellTexts[SPELL_HELICAL_FLOW]        = new FlagStructure("helical flow",null,"661");
     spellTexts[SPELL_HELLFIRE]            = new FlagStructure("hellfire",null,"520");
     spellTexts[SPELL_HOLY_WORD]           = new FlagStructure("holy word",null,"506");
     spellTexts[SPELL_HURRICANE]           = new FlagStructure("hurricane",null,"672");
     spellTexts[SPELL_ICEBALL]             = new FlagStructure("iceball",null,"513");
     spellTexts[SPELL_IDENTIFY]            = new FlagStructure("identify",null,"53");
     spellTexts[SPELL_IMPROVED_DETECT]     = new FlagStructure("improved detect",null,"626");
     spellTexts[SPELL_IMPROVED_INVIS]      = new FlagStructure("improved invis",null,"627");
     spellTexts[SPELL_INFRAVISION]         = new FlagStructure("infravision",null,"77");
     spellTexts[SPELL_INSANITY]            = new FlagStructure("insanity",null,"616");
     spellTexts[SPELL_INSPIRE]             = new FlagStructure("inspire",null,"587");
     spellTexts[SPELL_INVISIBILITY]        = new FlagStructure("invisibility",null,"28");
     spellTexts[SPELL_IRON_GOLEM]          = new FlagStructure("iron golem",null,"664");
     spellTexts[SPELL_KNOCK]               = new FlagStructure("knock",null,"603");
     spellTexts[SPELL_KNOW_ALIGNMENT]      = new FlagStructure("knwon alignment",null,"58");
     spellTexts[SPELL_LESSER_GOLEM]        = new FlagStructure("lesser golem",null,"662");
     spellTexts[SPELL_LETHARGIC_MIST]      = new FlagStructure("lethargic mist",null,"676");
     spellTexts[SPELL_LIGHTNING_BOLT]      = new FlagStructure("lightning bolt",null,"30");
     spellTexts[SPELL_LIGHTNING_BREATH]    = new FlagStructure("lightning breath",null,"204");
     spellTexts[SPELL_LIGHTNING_SHIELD]    = new FlagStructure("lightning shield",null,"614");
     spellTexts[SPELL_LINK]                = new FlagStructure("link",null,"588");
     spellTexts[SPELL_LION_HELP]           = new FlagStructure("lion help",null,"595");
     spellTexts[SPELL_LOCATE_OBJECT]       = new FlagStructure("locate object",null,"31");
     spellTexts[SPELL_MAGIC_JAR]           = new FlagStructure("magic jar",null,"596");
     spellTexts[SPELL_MAGIC_MISSILE]       = new FlagStructure("magic missile",null,"32");
     spellTexts[SPELL_MAGIC_RESISTANCE]    = new FlagStructure("magic resistance",null,"605");
     spellTexts[SPELL_MAGNETIC_TRUST]      = new FlagStructure("magnetic trust",null,"656");
     spellTexts[SPELL_MASS_HEALING]        = new FlagStructure("mass healing",null,"508");
     spellTexts[SPELL_MASS_INVIS]          = new FlagStructure("mass invis",null,"69");
     spellTexts[SPELL_MASS_SANCTUARY]      = new FlagStructure("mass sanctuary",null,"589");
     spellTexts[SPELL_MASTER_HEALING]      = new FlagStructure("master healing",null,"641");
     spellTexts[SPELL_MELD_INTO_STONE]     = new FlagStructure("meld into stone",null,"584");
     spellTexts[SPELL_MEND]                = new FlagStructure("mend",null,"590");
     spellTexts[SPELL_MIND_LIGHT]          = new FlagStructure("mind light",null,"82");
     spellTexts[SPELL_MIND_WRACK]          = new FlagStructure("mind wrack",null,"650");
     spellTexts[SPELL_MIND_WRENCH]         = new FlagStructure("mind wrench",null,"651");
     spellTexts[SPELL_MIST_WALK]           = new FlagStructure("mist walk",null,"658");
     spellTexts[SPELL_MYSTERIOUS_DREAM]    = new FlagStructure("mysterious dream",null,"678");
     spellTexts[SPELL_NEXUS]               = new FlagStructure("nexus",null,"520");
     spellTexts[SPELL_PASS_DOOR]           = new FlagStructure("pass door",null,"74");
     spellTexts[SPELL_PLAGUE]              = new FlagStructure("plague",null,"503");
     spellTexts[SPELL_POISON]              = new FlagStructure("poison",null,"33");
     spellTexts[SPELL_POLYMORPH]           = new FlagStructure("polymorph",null,"639");
     spellTexts[SPELL_PORTAL]              = new FlagStructure("portal",null,"519");
     spellTexts[SPELL_POWER_WORD_KILL]     = new FlagStructure("power word kill",null,"604");
     spellTexts[SPELL_HAND_OF_GOD]         = new FlagStructure("hand of god",null,"500");
     spellTexts[SPELL_POWER_WORD_STUN]     = new FlagStructure("power word stun",null,"625");
     spellTexts[SPELL_PROTECTION_COLD]     = new FlagStructure("protection cold",null,"600");
     spellTexts[SPELL_PROTECTION_EVIL]     = new FlagStructure("protection evil",null,"34");
     spellTexts[SPELL_PROTECTION_GOOD]     = new FlagStructure("protection good",null,"666");
     spellTexts[SPELL_PROTECTION_HEAT]     = new FlagStructure("protection heat",null,"599");
     spellTexts[SPELL_PROTECTION_NEGATIVE] = new FlagStructure("protection negative",null,"636");
     spellTexts[SPELL_PROTECTIVE_SHIELD]   = new FlagStructure("protection shield",null,"572");
     spellTexts[SPELL_QUANTUM_SPIKE]       = new FlagStructure("quantum spike",null,"657");
     spellTexts[SPELL_RANGER_STAFF]        = new FlagStructure("ranger staff",null,"519");
     spellTexts[SPELL_RAY_OF_TRUTH]        = new FlagStructure("ray of truth",null,"518");
     spellTexts[SPELL_RECHARGE]            = new FlagStructure("recharge",null,"517");
     spellTexts[SPELL_REFRESH]             = new FlagStructure("refresh",null,"81");
     spellTexts[SPELL_REMOVE_CURSE]        = new FlagStructure("remove curse",null,"35");
     spellTexts[SPELL_REMOVE_FEAR]         = new FlagStructure("remove fear",null,"582");
     spellTexts[SPELL_RESILIENCE]          = new FlagStructure("resilience",null,"638");
     spellTexts[SPELL_RESTORING_LIGHT]     = new FlagStructure("restoring light",null,"643");
     spellTexts[SPELL_SANCTIFY_LANDS]      = new FlagStructure("sanctify lands",null,"673");
     spellTexts[SPELL_SANCTUARY]           = new FlagStructure("sanctuary",null,"36");
     spellTexts[SPELL_SANDSTORM]           = new FlagStructure("sandstorm",null,"577");
     spellTexts[SPELL_SCREAM]              = new FlagStructure("scream",null,"578");
     spellTexts[SPELL_SEVERITY_FORCE]      = new FlagStructure("severity force",null,"622");
     spellTexts[SPELL_SHIELD]              = new FlagStructure("shield",null,"67");
     spellTexts[SPELL_SHIELDING]           = new FlagStructure("shielding",null,"591");
     spellTexts[SPELL_SHOCKING_GRASP]      = new FlagStructure("shocking grasp",null,"53");
     spellTexts[SPELL_SHOCKING_TRAP]       = new FlagStructure("shocking trap",null,"615");
     spellTexts[SPELL_SLEEP]               = new FlagStructure("sleep",null,"38");
     spellTexts[SPELL_SLOW]                = new FlagStructure("slow",null,"515");
     spellTexts[SPELL_SOLAR_FLIGHT]        = new FlagStructure("solar flight",null,"659");
     spellTexts[SPELL_SONIC_RESONANCE]     = new FlagStructure("sonic resonance",null,"649");
     spellTexts[SPELL_SPECTRAL_FUROR]      = new FlagStructure("spectral furor",null,"646");
     spellTexts[SPELL_STONE_GOLEM]         = new FlagStructure("stone golem",null,"663");
     spellTexts[SPELL_STONE_SKIN]          = new FlagStructure("stone skin",null,"66");
     spellTexts[SPELL_SULFURUS_SPRAY]      = new FlagStructure("sulfurus spray",null,"652");
     spellTexts[SPELL_SUMMON]              = new FlagStructure("summon",null,"40");
     spellTexts[SPELL_SUMMON_SHADOW]       = new FlagStructure("summon shadow",null,"620");
     spellTexts[SPELL_SUPERIOR_HEAL]       = new FlagStructure("superior heal",null,"640");
     spellTexts[SPELL_TELEPORT]            = new FlagStructure("teleport",null,"2");
     spellTexts[SPELL_TURN]                = new FlagStructure("turn",null,"597");
     spellTexts[SPELL_VENTRILOQUATE]       = new FlagStructure("ventriloquate",null,"41");
     spellTexts[SPELL_VAMPIRIC_BLAST]      = new FlagStructure("vampiric blast",null,"611");
     spellTexts[SPELL_WEAKEN]              = new FlagStructure("weaken",null,"68");
     spellTexts[SPELL_WEB]                 = new FlagStructure("web",null,"592");
     spellTexts[SPELL_WITCH_CURSE]         = new FlagStructure("witch curse",null,"602");
     spellTexts[SPELL_WORD_OF_RECALL]      = new FlagStructure("word of recall",null,"42");
     spellTexts[SPELL_WRATH]               = new FlagStructure("wrath",null,"553");

     objectConditionTexts[OBJECT_CONDITION_EXCELLENT] = new FlagStructure("excellent",null,"P");
     objectConditionTexts[OBJECT_CONDITION_GOOD]      = new FlagStructure("good"     ,null,"G");
     objectConditionTexts[OBJECT_CONDITION_FINE]      = new FlagStructure("fine"     ,null,"A");
     objectConditionTexts[OBJECT_CONDITION_AVERAGE]   = new FlagStructure("average"  ,null,"W");
     objectConditionTexts[OBJECT_CONDITION_POOR]      = new FlagStructure("poor"     ,null,"D");
     objectConditionTexts[OBJECT_CONDITION_FRAGILE]   = new FlagStructure("fragile"  ,null,"B");
     objectConditionTexts[OBJECT_CONDITION_BROKEN]    = new FlagStructure("broken"   ,null,"R");

     containerFlagTexts[CONT_CLOSEABLE] = new FlagStructure("Closeable"   , "Container can be closed"       , "A");
     containerFlagTexts[CONT_PICKPROOF] = new FlagStructure("Pick proof"  , "Container cannot be picked"    , "B");
     containerFlagTexts[CONT_CLOSED]    = new FlagStructure("Closed"      , "Container is closed"           , "C");
     containerFlagTexts[CONT_LOCKED]    = new FlagStructure("Locked"      , "Container is locked"           , "D");
     containerFlagTexts[CONT_PUT_ON]    = new FlagStructure("Put on"      , "You can put somthing on"       , "E");
     containerFlagTexts[CONT_FOR_ARROW] = new FlagStructure("For arrows"  , "Container can contain arrows"  , "F");

     roomFlagsTexts[ROOM_FLAG_NO_RECALL]    = new FlagStructure("No recall"   , "Players cannot use 'recall' command to leave this room"       , "N");
     roomFlagsTexts[ROOM_FLAG_NO_MAGIC]     = new FlagStructure("No magic"    , "No spells can be cast in this room"                           , "W");
     roomFlagsTexts[ROOM_FLAG_NOSUMMON]     = new FlagStructure("No summon"   , "Any (N)PC in this room cannot summon nor be summoned"         , "X");
     roomFlagsTexts[ROOM_FLAG_NOWHERE]      = new FlagStructure("Nowhere"     , null                                                           , "T");
     roomFlagsTexts[ROOM_FLAG_DARK]         = new FlagStructure("Dark"        , "A light source must be carried to see in this room"           , "A");
     roomFlagsTexts[ROOM_FLAG_INDOORS]      = new FlagStructure("Indoors"     , "Room is inside(i.e. not affected by weather)"                 , "D");
     roomFlagsTexts[ROOM_FLAG_BANK]         = new FlagStructure("Bank"        , "(c|g)balance, (c|g)deposit and (c|g)withdraw commands allowed", "U");
     roomFlagsTexts[ROOM_FLAG_PET_SHOP]     = new FlagStructure("Pet shop"    , "Room is a pet shop"                                           , "M");
     roomFlagsTexts[ROOM_FLAG_NO_MOB]       = new FlagStructure("No mob"      , "Monsters cannot enter this room"                              , "C");
     roomFlagsTexts[ROOM_FLAG_SAFE]         = new FlagStructure("Safe"        , "Safe from pkilling and aggressive mobs"                       , "K");
     roomFlagsTexts[ROOM_FLAG_PRIVATE]      = new FlagStructure("Private"     , "Room is limited to two characters (i.e. chat room)"           , "J");
     roomFlagsTexts[ROOM_FLAG_SOLITARY]     = new FlagStructure("Solitary"    , "One character only can enter this room"                       , "L");
     roomFlagsTexts[ROOM_FLAG_NEWBIES_ONLY] = new FlagStructure("Newbies only", "Level 10 and less only"                                       , "R");
     roomFlagsTexts[ROOM_FLAG_HEROES_ONLY]  = new FlagStructure("Heroes only" , "Level 91 and more only"                                       , "Q");
     roomFlagsTexts[ROOM_FLAG_GODS_ONLY]    = new FlagStructure("Gods only"   , "Level 95 and more only"                                       , "P");
     roomFlagsTexts[ROOM_FLAG_IMP_ONLY]     = new FlagStructure("Imp only"    , "Level 100 only"                                               , "O");
     roomFlagsTexts[ROOM_FLAG_LAW]          = new FlagStructure("Law"         , null                                                           , "S");
     roomFlagsTexts[ROOM_FLAG_REGISTRY]     = new FlagStructure("Registry"    , null                                                           , "b");
     roomFlagsTexts[ROOM_FLAG_RANDOM]       = new FlagStructure("Random exits", "the exits of the room randomize regularly (it's a \"Void\")"  , null);

     roomSectorTexts[ROOM_SECTOR_INSIDE]    = new FlagStructure("Inside(1)"    , "Room costs 1 move point"                                             , null);
     roomSectorTexts[ROOM_SECTOR_CITY]      = new FlagStructure("City(2)"      , "Room costs 2 move points"                                            , null);
     roomSectorTexts[ROOM_SECTOR_FIELD]     = new FlagStructure("Field(2)"     , "Room costs 2 move points"                                            , null);
     roomSectorTexts[ROOM_SECTOR_FOREST]    = new FlagStructure("Forest(3)"    , "Room costs 3 move points"                                            , null);
     roomSectorTexts[ROOM_SECTOR_HILL]      = new FlagStructure("Hills(4)"     , "Room costs 4 move points"                                            , null);
     roomSectorTexts[ROOM_SECTOR_MOUNTAIN]  = new FlagStructure("Mountain(6)"  , "Room costs 6 move points"                                            , null);
     roomSectorTexts[ROOM_SECTOR_WATER]     = new FlagStructure("Water(4)"     , "(swimmable) Room costs 4 move points"                                , null);
     roomSectorTexts[ROOM_SECTOR_DEEPWATER] = new FlagStructure("Deep Water(0)", "Boat required"                                                       , null);
     roomSectorTexts[ROOM_SECTOR_UNUSED]    = new FlagStructure("Unused"       , "Unused"                                                              , null);
     roomSectorTexts[ROOM_SECTOR_AIR]       = new FlagStructure("Air(0)"       , "Fly spell required"                                                  , null);
     roomSectorTexts[ROOM_SECTOR_DESERT]    = new FlagStructure("Desert(9)"    , "Room costs 9 move points; will eventually affect thirst and recovery", null);

     doorStateTexts[DOOR_STATE_OPENED] = new FlagStructure("Opened", null, null);
     doorStateTexts[DOOR_STATE_CLOSED] = new FlagStructure("Closed", null, null);
     doorStateTexts[DOOR_STATE_LOCKED] = new FlagStructure("Locked", null, null);

     doorSizeTexts[SIZE_TINY]       = new FlagStructure("tiny"      , "Particle, atoms"  , "tiny");
     doorSizeTexts[SIZE_SMALL]      = new FlagStructure("small"     , "svirfnebli, gnome", "small");
     doorSizeTexts[SIZE_MEDIUM]     = new FlagStructure("medium"    , "troll"            , "medium");
     doorSizeTexts[SIZE_LARGE]      = new FlagStructure("Large"     , "human, elf, dwarf", "large");
     doorSizeTexts[SIZE_HUGE]       = new FlagStructure("Huge"      , "giant"            , "huge");
     doorSizeTexts[SIZE_GIANT]      = new FlagStructure("GIANT"     , "dragon"           , "giant");
     doorSizeTexts[SIZE_GARGANTUAN] = new FlagStructure("GARGANTUAN", "dragon"           , "gargantuan");

     doorResetsTexts[DOOR_RESETS_OPENED]       = new FlagStructure("Door may be opened, closed, not locked, resets to opened", null, "A");
     doorResetsTexts[DOOR_RESETS_CLOSED]       = new FlagStructure("Door resets to closed"                                   , null, "B");
     doorResetsTexts[DOOR_RESETS_LOCKED]       = new FlagStructure("Door resets to locked"                                   , null, "C");
     doorResetsTexts[DOOR_NO_PASSDOOR]         = new FlagStructure("Door unaffected by \"pass door\""                        , null, "F");
     doorResetsTexts[DOOR_EASY_TO_PICK]        = new FlagStructure("Door with lock easy to pick"                             , null, "G");
     doorResetsTexts[DOOR_HARD_TO_PICK]        = new FlagStructure("Door with lock hard to pick"                             , null, "H");
     doorResetsTexts[DOOR_INFURIATING_TO_PICK] = new FlagStructure("Door with lock infuriating to pick"                      , null, "I");
     doorResetsTexts[DOOR_CANNOT_BE_PICKED]    = new FlagStructure("Door with unpickable lock (key required)"                , null, "J");
     doorResetsTexts[DOOR_CANNOT_BE_CLOSED]    = new FlagStructure("Door cannot be closed"                                   , null, "K");
     doorResetsTexts[DOOR_CANNOT_BE_LOCKED]    = new FlagStructure("Door cannot be locked"                                   , null, "L");


     weaponFlagsTexts[WEAPON_FLAG_FLAMING]    = new FlagStructure("Flaming"   ,""              ,"A");
     weaponFlagsTexts[WEAPON_FLAG_FROST]      = new FlagStructure("Frost"     ,""              ,"B");
     weaponFlagsTexts[WEAPON_FLAG_VAMPIRIC]   = new FlagStructure("Vampiric"  ,"(Energy drain)","C");
     weaponFlagsTexts[WEAPON_FLAG_SHARP]      = new FlagStructure("Sharp"     ,""              ,"D");
     weaponFlagsTexts[WEAPON_FLAG_VORPAL]     = new FlagStructure("Vorpal"    ,""              ,"E");
     weaponFlagsTexts[WEAPON_FLAG_TWO_HANDED] = new FlagStructure("Two-handed",""              ,"F");
     weaponFlagsTexts[WEAPON_FLAG_SHOCKING]   = new FlagStructure("Shocking"  ,"(Electrical)"  ,"G");
     weaponFlagsTexts[WEAPON_FLAG_POISONED]   = new FlagStructure("Poisoned"  ,""              ,"H");
     weaponFlagsTexts[WEAPON_FLAG_HOLY]       = new FlagStructure("Holy"      ,""              ,"I");
     weaponFlagsTexts[WEAPON_FLAG_KATANA]     = new FlagStructure("Katana"    ,""              ,"J");
     weaponFlagsTexts[WEAPON_FLAG_NIGHTMARE]  = new FlagStructure("Nightmare" ,""              ,"K");

     damageTexts = new String[damageTypeTexts.length][];
     damageTexts[DAMAGE_TYPE_PIERCE] = damageTypePierceTexts;
     damageTexts[DAMAGE_TYPE_BASH]   = damageTypeBashTexts;
     damageTexts[DAMAGE_TYPE_SLASH]  = damageTypeSlashTexts;
     damageTexts[DAMAGE_TYPE_MAGIC]  = damageTypeMagicTexts;

     mobileActFlagTexts[MOBILE_ACT_FLAG_IS_NPC]        = new FlagStructure("NPC"          ,"Mobile is an NPC (set automatically by the game)"            ,"A");
     mobileActFlagTexts[MOBILE_ACT_FLAG_SENTINEL]      = new FlagStructure("sentinel"     ,"Mobile doesn't wander"                                       ,"B");
     mobileActFlagTexts[MOBILE_ACT_FLAG_SCAVENGER]     = new FlagStructure("scavenger"    ,"Mobile picks up items on the floor"                          ,"C");
     mobileActFlagTexts[MOBILE_ACT_FLAG_AGGRESSIVE]    = new FlagStructure("aggressive"   ,"Mobile attacks any character in the same room"               ,"F");
     mobileActFlagTexts[MOBILE_ACT_FLAG_STAY_AREA]     = new FlagStructure("stay area"    ,"Mobile will not leave a zone (this should be set)"           ,"G");
     mobileActFlagTexts[MOBILE_ACT_FLAG_WIMPY]         = new FlagStructure("wimpy"        ,"Mobile will fly when badly hurt"                             ,"H");
     mobileActFlagTexts[MOBILE_ACT_FLAG_PET]           = new FlagStructure("pet"          ,"Mobile is a pet (and hence safe from attack)"                ,"I");
     mobileActFlagTexts[MOBILE_ACT_FLAG_TRAIN]         = new FlagStructure("train"        ,"Mobile can train statistics"                                 ,"J");
     mobileActFlagTexts[MOBILE_ACT_FLAG_PRACTICE]      = new FlagStructure("practice"     ,"Mobile can practice statistics"                              ,"K");
     mobileActFlagTexts[MOBILE_ACT_FLAG_HUNTER]        = new FlagStructure("hunter"       ,""                                                            ,"L");
     mobileActFlagTexts[MOBILE_ACT_FLAG_UNDEAD]        = new FlagStructure("undead"       ,"Mobile has special undead powers (i.e. life draining)"       ,"O");
     mobileActFlagTexts[MOBILE_ACT_FLAG_CLERIC]        = new FlagStructure("cleric"       ,"Mobile has cleric casting powers"                            ,"Q");
     mobileActFlagTexts[MOBILE_ACT_FLAG_MAGE]          = new FlagStructure("mage"         ,"Mobile has mage casting powers"                              ,"R");
     mobileActFlagTexts[MOBILE_ACT_FLAG_THIEF]         = new FlagStructure("thief"        ,"Mobile has thief combat skills (backstab, etc.)"             ,"S");
     mobileActFlagTexts[MOBILE_ACT_FLAG_WARRIOR]       = new FlagStructure("warrior"      ,"Mobile has warrior combat skills (disarm, parry, etc.)"      ,"T");
     mobileActFlagTexts[MOBILE_ACT_FLAG_NOALIGN]       = new FlagStructure("noalign"      ,"Mobile is unaligned (unintelligent animals, golems, etc.)"   ,"U");
     mobileActFlagTexts[MOBILE_ACT_FLAG_NOPURGE]       = new FlagStructure("nopurge"      ,"Mobile isn't removed by the purge command"                   ,"V");
     mobileActFlagTexts[MOBILE_ACT_FLAG_OUTDOORS]      = new FlagStructure("outdoors"     ,"Mobile will not wander outside a building"                   ,"W");
     mobileActFlagTexts[MOBILE_ACT_FLAG_INDOORS]       = new FlagStructure("indoors"      ,"Mobile will not wander into a building"                      ,"Y");
     mobileActFlagTexts[MOBILE_ACT_FLAG_RIDEABLE]      = new FlagStructure("rideable"     ,"Mobile can be ridden"                                        ,"Z");
     mobileActFlagTexts[MOBILE_ACT_FLAG_IS_HEALER]     = new FlagStructure("healer"       ,"Mobile can heal characters (i.e. the heal command)"          ,"a");
     mobileActFlagTexts[MOBILE_ACT_FLAG_GAIN]          = new FlagStructure("gain"         ,"Mobile can grant new skills (i.e. the gain command)"         ,"b");
     mobileActFlagTexts[MOBILE_ACT_FLAG_UPDATE_ALWAYS] = new FlagStructure("update"       ,"Mobile is always updated, even in idle zones (rarely needed)","c");
     mobileActFlagTexts[MOBILE_ACT_FLAG_IS_CHANGER]    = new FlagStructure("changer"      ,"Mobile can change coins (i.e. Otho the Money Changer)"       ,"d");
     mobileActFlagTexts[MOBILE_ACT_FLAG_NOTRACK]       = new FlagStructure("no Track"     ,""                                                            ,"e");

     mobileAffFlagTexts[MOBILE_AFF_FLAG_BLIND]        = new FlagStructure("blind"        ,"Mobile is blind"                                        ,"A");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_INVISIBLE]    = new FlagStructure("invisible"    ,"Mobile is invisible"                                    ,"B");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_DETECT_EVIL]  = new FlagStructure("detect evil"  ,"Mobile can sense evil"                                  ,"C");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_DETECT_INVIS] = new FlagStructure("detect invis" ,"Mobile can see invisible."                              ,"D");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_DETECT_MAGIC] = new FlagStructure("detect magic" ,"Mobile can see magic"                                   ,"E");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_DETECT_HIDE]  = new FlagStructure("detect hide"  ,"Mobile can see hidden (sneaking/hiding) characters"     ,"F");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_DETECT_GOOD]  = new FlagStructure("detect good"  ,"Mobile can sense good"                                  ,"G");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_SANCTUARY]    = new FlagStructure("sanctuary"    ,"Mobile is protected by a sanctuary spell."              ,"H");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_FAERIE_FIRE]  = new FlagStructure("faerie fire"  ,"Mobile is surrounded by faerie fire (a hindrance)"      ,"I");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_INFRAVISION]  = new FlagStructure("infravision"  ,"Mobile can see heat sources in the dark"                ,"J");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_PROTECT_EVIL] = new FlagStructure("protect evil" ,"Mobile takes less damage from evil characters"          ,"N");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_PROTECT_GOOD] = new FlagStructure("protect good" ,"Mobile takes less damage from good characters"          ,"O");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_SNEAKING]     = new FlagStructure("sneaking"     ,"Mobile is sneaking (hard to detect while moving)"       ,"P");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_HIDING]       = new FlagStructure("hiding"       ,"Mobile is hiding (cannot be seen without detect hidden)","Q");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_FLYING]       = new FlagStructure("flying"       ,"Mobile is flying"                                       ,"T");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_PASS_DOOR]    = new FlagStructure("pass door"    ,"Mobile can walk through closed doors"                   ,"U");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_HASTE]        = new FlagStructure("haste"        ,"Mobile is affected by a haste spell"                    ,"V");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_DARK_VISION]  = new FlagStructure("dark vision"  ,"Mobile can see in the dark without a light source"      ,"Z");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_SWIMMING]     = new FlagStructure("swimming"     ,"Mobile is swimming (or capable of swimming)"            ,"b");
     mobileAffFlagTexts[MOBILE_AFF_FLAG_REGENERATION] = new FlagStructure("regeneration" ,"Mobile recovers hit points and mana faster than usual"  ,"c");

     mobileOffFlagTexts[MOBILE_OFF_FLAG_AREA_ATTACK] = new FlagStructure("area attack"  ,"Mobile hits all characters fighting against it. Very powerful.","A");
     mobileOffFlagTexts[MOBILE_OFF_FLAG_BACKSTAB]    = new FlagStructure("backstab"     ,"Mobile can backstab to start a combat"                         ,"B");
     mobileOffFlagTexts[MOBILE_OFF_FLAG_BASH]        = new FlagStructure("bash"         ,"Mobile can bash characters off their feet"                     ,"C");
     mobileOffFlagTexts[MOBILE_OFF_FLAG_BERSERK]     = new FlagStructure("berserk"      ,"Mobile may go berserk in a fight"                              ,"D");
     mobileOffFlagTexts[MOBILE_OFF_FLAG_DISARM]      = new FlagStructure("disarm"       ,"Mobile can disarm _without_ a weapon wielded"                  ,"E");
     mobileOffFlagTexts[MOBILE_OFF_FLAG_DODGE]       = new FlagStructure("dodge"        ,"Mobile dodges blows"                                           ,"F");
     mobileOffFlagTexts[MOBILE_OFF_FLAG_FADE]        = new FlagStructure("fade"         ,"Mobile can fade \"out of phase\" to avoid blows"               ,"G");
     mobileOffFlagTexts[MOBILE_OFF_FLAG_FAST]        = new FlagStructure("fast"         ,"Mobile is faster than most others, so has extra attacks"       ,"H");
     mobileOffFlagTexts[MOBILE_OFF_FLAG_KICK]        = new FlagStructure("kick"         ,"Mobile can kick in combat for extra damage"                    ,"I");
     mobileOffFlagTexts[MOBILE_OFF_FLAG_KICK_DIRT]   = new FlagStructure("kick dirt"    ,"Mobile kicks dirt, blinding opponents"                         ,"J");
     mobileOffFlagTexts[MOBILE_OFF_FLAG_PARRY]       = new FlagStructure("parry"        ,"Mobile can parry _without_ a weapon wielded"                   ,"K");
     mobileOffFlagTexts[MOBILE_OFF_FLAG_RESCUE]      = new FlagStructure("rescue"       ,"Mobile may rescue allies in a fight"                           ,"L");
     mobileOffFlagTexts[MOBILE_OFF_FLAG_TAIL]        = new FlagStructure("tail"         ,"Mobile can legsweep with its tail or tentacles"                ,"M");
     mobileOffFlagTexts[MOBILE_OFF_FLAG_TRIP]        = new FlagStructure("trip"         ,"Mobile trips in combat"                                        ,"N");
     mobileOffFlagTexts[MOBILE_OFF_FLAG_CRUSH]       = new FlagStructure("crush"        ,"Mobile can crush opponents in its arms"                        ,"O");

     mobileAssistsTexts[MOBILE_ASSISTS_ALL]     = new FlagStructure("all"     ,"Mobile helps all other mobiles in combat"         ,"P");
     mobileAssistsTexts[MOBILE_ASSISTS_ALIGN]   = new FlagStructure("align"   ,"Mobile assists mobiles of like alignment"         ,"Q");
     mobileAssistsTexts[MOBILE_ASSISTS_RACE]    = new FlagStructure("race"    ,"Mobile will assist other mobiles of the same race","R");
     mobileAssistsTexts[MOBILE_ASSISTS_PLAYERS] = new FlagStructure("players" ,"Mobile will assist players (by race/alignment)"   ,"S");
     mobileAssistsTexts[MOBILE_ASSISTS_GUARD]   = new FlagStructure("guard"   ,"Mobile assists as a cityguard"                    ,"T");
     mobileAssistsTexts[MOBILE_ASSISTS_VNUM]    = new FlagStructure("vnum"    ,"Mobile assists mobiles of the same number only"   ,"U");

     immResVulnFlagTexts[IMM_RES_VULN_SUMMON]    = new FlagStructure("summon"       ,"Summoning and gating magic"                            ,"A");
     immResVulnFlagTexts[IMM_RES_VULN_CHARM]     = new FlagStructure("charm"        ,"Charm spells (the beguiling spell group)"              ,"B");
     immResVulnFlagTexts[IMM_RES_VULN_MAGIC]     = new FlagStructure("magic"        ,"All magic (be very careful using this flag)"           ,"C");
     immResVulnFlagTexts[IMM_RES_VULN_WEAPONS]   = new FlagStructure("weapons"      ,"All physical attacks (be very careful using this flag)","D");
     immResVulnFlagTexts[IMM_RES_VULN_BASH]      = new FlagStructure("bash"         ,"Blunt weapons"                                         ,"E");
     immResVulnFlagTexts[IMM_RES_VULN_PIERCE]    = new FlagStructure("pierce"       ,"Piercing weapons"                                      ,"F");
     immResVulnFlagTexts[IMM_RES_VULN_SLASH]     = new FlagStructure("slash"        ,"Slashing weapons"                                      ,"G");
     immResVulnFlagTexts[IMM_RES_VULN_FIRE]      = new FlagStructure("Fire"         ,"Flame and heat attacks and spells"                     ,"H");
     immResVulnFlagTexts[IMM_RES_VULN_COLD]      = new FlagStructure("Cold"         ,"Cold and ice attacks and spells"                       ,"I");
     immResVulnFlagTexts[IMM_RES_VULN_LIGHTNING] = new FlagStructure("Lightning"    ,"Electrical attacks and spells"                         ,"J");
     immResVulnFlagTexts[IMM_RES_VULN_ACID]      = new FlagStructure("Acid"         ,"Corrosive attacks and spells"                          ,"K");
     immResVulnFlagTexts[IMM_RES_VULN_POISON]    = new FlagStructure("Poison"       ,"Venoms and toxic vapors"                               ,"L");
     immResVulnFlagTexts[IMM_RES_VULN_NEGATIVE]  = new FlagStructure("Negative"     ,"Life draining attacks and spells, or unholy energies"  ,"M");
     immResVulnFlagTexts[IMM_RES_VULN_HOLY]      = new FlagStructure("Holy"         ,"Holy or blessed attacks"                               ,"N");
     immResVulnFlagTexts[IMM_RES_VULN_ENERGY]    = new FlagStructure("Energy"       ,"Generic magical force (i.e. magic missile)"            ,"O");
     immResVulnFlagTexts[IMM_RES_VULN_MENTAL]    = new FlagStructure("Mental"       ,"Mental attacks (such as a mind flayer's mind blasts)"  ,"P");
     immResVulnFlagTexts[IMM_RES_VULN_DISEASE]   = new FlagStructure("Disease"      ,"Disease, from the common cold to the black death"      ,"Q");
     immResVulnFlagTexts[IMM_RES_VULN_DROWNING]  = new FlagStructure("Drowning"     ,"Watery attacks and suffocation"                        ,"R");
     immResVulnFlagTexts[IMM_RES_VULN_LIGHT]     = new FlagStructure("Light"        ,"Light-based attacks, whether blinding or cutting"      ,"S");
     immResVulnFlagTexts[IMM_RES_VULN_SOUND]     = new FlagStructure("Sound"        ,"Sonic attacks and weapons, or deafening noises"        ,"T");
     immResVulnFlagTexts[IMM_RES_VULN_WOOD]      = new FlagStructure("Wood"         ,"Wooden weapons and creatures"                          ,"X");
     immResVulnFlagTexts[IMM_RES_VULN_SILVER]    = new FlagStructure("Silver"       ,"Silver or mithril weapons and creatures"               ,"Y");
     immResVulnFlagTexts[IMM_RES_VULN_IRON]      = new FlagStructure("Iron"         ,"Iron and steel weapons and creatures"                  ,"Z");

     mobileSizeTexts[SIZE_TINY]       = new FlagStructure("tiny"      , "Particle, atoms"  , "tiny");
     mobileSizeTexts[SIZE_SMALL]      = new FlagStructure("small"     , "svirfnebli, gnome", "small");
     mobileSizeTexts[SIZE_MEDIUM]     = new FlagStructure("medium"    , "troll"            , "medium");
     mobileSizeTexts[SIZE_LARGE]      = new FlagStructure("Large"     , "human, elf, dwarf", "large");
     mobileSizeTexts[SIZE_HUGE]       = new FlagStructure("Huge"      , "giant"            , "huge");
     mobileSizeTexts[SIZE_GIANT]      = new FlagStructure("GIANT"     , "dragon"           , "giant");
     mobileSizeTexts[SIZE_GARGANTUAN] = new FlagStructure("GARGANTUAN", "dragon"           , "gargantuan");

     mobileCorpseFlagTexts[MOBILE_CORPSE_FLAG_EDIBLE]        = new FlagStructure("edible"   ,"Mobile can be eaten"                                     ,"A");
     mobileCorpseFlagTexts[MOBILE_CORPSE_FLAG_POISON]        = new FlagStructure("poison"   ,"Mobile is poisonous when eaten (should also be edible)"  ,"B");
     mobileCorpseFlagTexts[MOBILE_CORPSE_FLAG_MAGICAL]       = new FlagStructure("magical*" ,"Mobile's magic nature causes strange effects when eaten" ,"C");
     mobileCorpseFlagTexts[MOBILE_CORPSE_FLAG_INSTANT_DECAY] = new FlagStructure("vanishes","Mobile vanishes after death (i.e. a wraith)"              ,"D");
     mobileCorpseFlagTexts[MOBILE_CORPSE_FLAG_OTHER]         = new FlagStructure("other*"   ,"Mobile is not flesh and blood (defined by material type)","E");

     mobileFormFlagTexts[MOBILE_FORM_FLAG_ANIMAL]     = new FlagStructure("animal"       ,"Mobile is a \"dumb\" animal"                             , "G");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_SENTIENT]   = new FlagStructure("sentient"     ,"Mobile is capable of higher reasoning"                   , "H");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_UNDEAD]     = new FlagStructure("undead"       ,"Mobile is an undead, and not truly alive at all"         , "I");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_CONSTRUCT]  = new FlagStructure( "construct"    ,"Mobile is a magical construct, such as a golem"         , "J");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_MIST]       = new FlagStructure("mist"         ,"Mobile is a partially material mist"                     , "K");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_INTANGIBLE] = new FlagStructure("intangible"   ,"Mobile is immaterial (like a ghost)"                     , "L");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_BIPED]      = new FlagStructure("biped"        ,"Mobile is bipedal (like a human)"                        , "M");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_CENTAUR]    = new FlagStructure("centaur"      ,"Mobile has a humanoid torso, but a beast's lower body"   , "N");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_INSECT]     = new FlagStructure("insect"       ,"Mobile is an insect"                                     , "O");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_SPIDER]     = new FlagStructure("spider"       ,"Mobile is an arachnid"                                   , "P");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_CRUSTACEAN] = new FlagStructure("crustacean"   ,"Mobile is a crustacean (i.e. a crab or lobster)"         , "Q");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_WORM]       = new FlagStructure("worm"         ,"Mobile is a worm, that is a tube-shaped invertebrate"    , "R");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_BLOB]       = new FlagStructure("blob"         ,"Mobile is a formless blob (when used with mist, a cloud)", "S");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_MAMMAL]     = new FlagStructure("mammal"       ,"Mobile is a mammal"                                      , "V");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_BIRD]       = new FlagStructure("bird"         ,"Mobile is a bird"                                        , "W");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_REPTILE]    = new FlagStructure("reptile"      ,"Mobile is a reptile (and should be cold-blooded)"        , "X");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_SNAKE]      = new FlagStructure("snake"        ,"Mobile is a snake (and should be a reptile)"             , "Y");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_DRAGON]     = new FlagStructure("dragon"       ,"Mobile is a dragon"                                      , "Z");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_AMPHIBIAN]  = new FlagStructure("amphibian"    ,"Mobile is an amphibian (and should be able to swim)"     , "a");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_FISH]       = new FlagStructure("fish"         ,"Mobile is a fish (and should be able to swim)"           , "b");
     mobileFormFlagTexts[MOBILE_FORM_FLAG_COLD_BLOOD] = new FlagStructure("cold blood"   ,"Mobile is cold-blooded, cannot be seen with infravis."   , "c");

     mobilePartsFlagTexts[MOBILE_PART_FLAG_HEAD]      = new FlagStructure("head"         ,"Mobile has a head"                                      ,"A");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_ARMS]      = new FlagStructure("arms"         ,"Mobile has arm(s) (usually assumed to be 2)"            ,"B");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_LEGS]      = new FlagStructure("legs"         ,"Mobile has leg(s)"                                      ,"C");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_HEART]     = new FlagStructure("heart"        ,"Mobile has a heart"                                     ,"D");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_BRAINS]    = new FlagStructure("brains"       ,"Mobile has brains (not all mobs with heads have brains)","E");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_GUTS]      = new FlagStructure("guts"         ,"Mobile has intestines"                                  ,"F");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_HANDS]     = new FlagStructure("hands"        ,"Mobile has hands capable of manipulating objects"       ,"G");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_FEET]      = new FlagStructure("feet"         ,"Mobile has feet"                                        ,"H");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_FINGERS]   = new FlagStructure("fingers"      ,"Mobile has fingers capable of wearing rings"            ,"I");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_EAR]       = new FlagStructure("ear"          ,"Mobile has ear(s)"                                      ,"J");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_EYE]       = new FlagStructure("eye"          ,"Mobile has eye(s)"                                      ,"K");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_TONGUE]    = new FlagStructure("tongue"       ,"Mobile has a _long_ tongue (like a lizard)"             ,"L");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_EYESTALK]  = new FlagStructure("eyestalks"    ,"Mobile has eyestalks (it should also have eyes)"        ,"M");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_TENTACLES] = new FlagStructure("tentacles"    ,"Mobile has one or more tentacles"                       ,"N");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_FINS]      = new FlagStructure("fins"         ,"Mobile has fins"                                        ,"O");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_WINGS]     = new FlagStructure("wings"        ,"Mobile has wings"                                       ,"P");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_TAIL]      = new FlagStructure("tail"         ,"Mobile has a usable tail (no stubs)"                    ,"Q");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_CLAWS]     = new FlagStructure("claws"        ,"Mobile has combat-capable claws"                        ,"U");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_FANGS]     = new FlagStructure("fangs"        ,"Mobile has combat-capable teeth"                        ,"V");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_HORNS]     = new FlagStructure("horns"        ,"Mobile has horns, not necessarily dangerous ones"       ,"W");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_SCALES]    = new FlagStructure("scales"       ,"Mobile is covered with scales"                          ,"X");
     mobilePartsFlagTexts[MOBILE_PART_FLAG_TUSKS]     = new FlagStructure("tusks"        ,"Mobile has some teeth elongated into tusks"             ,"Y");

     wearTexts[WEAR_NONE]         = new FlagStructure("Inventory"      , null, "-1");
     wearTexts[WEAR_LIGHT]        = new FlagStructure("Light"          , null, "0");
     wearTexts[WEAR_LEFT_FINGER]  = new FlagStructure("Left Finger"    , null, "1");
     wearTexts[WEAR_RIGHT_FINGER] = new FlagStructure("Right Finger"   , null, "2");
     wearTexts[WEAR_NECK_1]       = new FlagStructure("Neck(1)"        , null, "3");
     wearTexts[WEAR_NECK_2]       = new FlagStructure("Neck(2)"        , null, "4");
     wearTexts[WEAR_TORSO]        = new FlagStructure("On torso"       , null, "5");
     wearTexts[WEAR_HEAD]         = new FlagStructure("Head"           , null, "6");
     wearTexts[WEAR_LEGS]         = new FlagStructure("Legs"           , null, "7");
     wearTexts[WEAR_FEET]         = new FlagStructure("Feet"           , null, "8");
     wearTexts[WEAR_HANDS]        = new FlagStructure("Hands"          , null, "9");
     wearTexts[WEAR_ARMS]         = new FlagStructure("Arms"           , null, "10");
     wearTexts[WEAR_SHIELD]       = new FlagStructure("As shield"      , null, "11");
     wearTexts[WEAR_ABOUT_BODY]   = new FlagStructure("About body"     , null, "12");
     wearTexts[WEAR_WAIST]        = new FlagStructure("Waist"          , null, "13");
     wearTexts[WEAR_LEFT_WRIST]   = new FlagStructure("Left wrist"     , null, "14");
     wearTexts[WEAR_RIGHT_WRIST]  = new FlagStructure("Right wrist"    , null, "15");
     wearTexts[WEAR_WIELD]        = new FlagStructure("Wielded"        , null, "16");
     wearTexts[WEAR_HELD]         = new FlagStructure("Held"           , null, "17");
     wearTexts[WEAR_FLOATING]     = new FlagStructure("Floating nearby", null, "18");
     wearTexts[WEAR_TATOO]        = new FlagStructure("Tatoo"          , null, "19");
     wearTexts[WEAR_SECOND_WIELD] = new FlagStructure("Second wield"   , null, "20");
     wearTexts[WEAR_STUCK_IN]     = new FlagStructure("Stuck in"       , null, "21");

     liquids[LIQUID_WATER]            = new LiquidStructure("water"                  ,"clear",           0, 1, 10);
     liquids[LIQUID_BEER]             = new LiquidStructure("beer"                   ,"amber",          12, 1,  8);
     liquids[LIQUID_RED_WINE]         = new LiquidStructure("red wine"               ,"burgundy",       30, 1,  8);
     liquids[LIQUID_ALE]              = new LiquidStructure("ale"                    ,"brown",          15, 1,  8);
     liquids[LIQUID_DARK_ALE]         = new LiquidStructure("dark ale"               ,"dark",           16, 1,  8);
     liquids[LIQUID_WHISKY]           = new LiquidStructure("whisky"                 ,"golden",        120, 1,  5);
     liquids[LIQUID_LEMONADE]         = new LiquidStructure("lemonade"               ,"pink",            0, 1,  9);
     liquids[LIQUID_FIREBREATHER]     = new LiquidStructure("firebreather"           ,"boiling",       190, 0,  4);
     liquids[LIQUID_LOCAL_SPECIALITY] = new LiquidStructure("local specialty"        ,"clear",         151, 1,  3);
     liquids[LIQUID_SLIME_MOLD_JUICE] = new LiquidStructure("slime mold juice"       ,"green",           0, 2, -8);
     liquids[LIQUID_MILK]             = new LiquidStructure("milk"                   ,"white",           0, 2,  9);
     liquids[LIQUID_TEA]              = new LiquidStructure("tea"                    ,"tan",             0, 1,  8);
     liquids[LIQUID_COFFEE]           = new LiquidStructure("coffee"                 ,"black",           0, 1,  8);
     liquids[LIQUID_BLOOD]            = new LiquidStructure("blood"                  ,"red",             0, 2, -1);
     liquids[LIQUID_SALT_WATER]       = new LiquidStructure("salt water"             ,"clear",           0, 1, -2);
     liquids[LIQUID_COKE]             = new LiquidStructure("coke"                   ,"brown",           0, 2,  9);
     liquids[LIQUID_ROOT_BEER]        = new LiquidStructure("root beer"              ,"brown",           0, 2,  9);
     liquids[LIQUID_ELVISH_WINE]      = new LiquidStructure("elvish wine"            ,"green",          35, 2,  8);
     liquids[LIQUID_WHITE_WINE]       = new LiquidStructure("white wine"             ,"golden",         28, 1,  8);
     liquids[LIQUID_CHAMPAGNE]        = new LiquidStructure("champagne"              ,"golden",         32, 1,  8);
     liquids[LIQUID_MEAD]             = new LiquidStructure("mead"                   ,"honey-colored",  34, 2,  8);
     liquids[LIQUID_ROSE_WINE]        = new LiquidStructure("rose wine"              ,"pink",           26, 1,  8);
     liquids[LIQUID_BENEDICTINE_WINE] = new LiquidStructure("benedictine wine"       ,"burgundy",       40, 1,  8);
     liquids[LIQUID_VODKA]            = new LiquidStructure("vodka"                  ,"clear",         130, 1,  5);
     liquids[LIQUID_CRANBERRY_JUICE]  = new LiquidStructure("cranberry juice"        ,"red",             0, 1,  9);
     liquids[LIQUID_ORANGE_JUICE]     = new LiquidStructure("orange juice"           ,"orange",          0, 2,  9);
     liquids[LIQUID_ABSINTHE]         = new LiquidStructure("absinthe"               ,"green",         200, 1,  4);
     liquids[LIQUID_BRANDY]           = new LiquidStructure("brandy"                 ,"golden ",        80, 1,  5);
     liquids[LIQUID_AQUAVIT]          = new LiquidStructure("aquavit"                ,"clear",         140, 1,  5);
     liquids[LIQUID_SCHNAPPS]         = new LiquidStructure("schnapps"               ,"clear",          90, 1,  5);
     liquids[LIQUID_ICEWINE]          = new LiquidStructure("icewine"                ,"purple",         50, 2,  6);
     liquids[LIQUID_AMONTILLADO]      = new LiquidStructure("amontillado"            ,"burgundy",       35, 2,  8);
     liquids[LIQUID_SHERRY]           = new LiquidStructure("sherry"                 ,"red ",           38, 2,  7);
     liquids[LIQUID_FRAMBOISE]        = new LiquidStructure("framboise"              ,"red",            50, 1,  7);
     liquids[LIQUID_RUM]              = new LiquidStructure("rum"                    ,"amber",         151, 1,  4);
     liquids[LIQUID_CORDIAL]          = new LiquidStructure("cordial"                ,"clear",         100, 1,  5);

     objectTypeTexts[OBJECT_TYPE_NONE]      = new FlagStructure("none"      , null, "-1");
     objectTypeTexts[OBJECT_TYPE_LIGHT]     = new FlagStructure("light"     , null, "1");
     objectTypeTexts[OBJECT_TYPE_SCROLL]    = new FlagStructure("scroll"    , null, "2");
     objectTypeTexts[OBJECT_TYPE_WAND]      = new FlagStructure("wand"      , null, "3");
     objectTypeTexts[OBJECT_TYPE_STAFF]     = new FlagStructure("staff"     , null, "4");
     objectTypeTexts[OBJECT_TYPE_WEAPON]    = new FlagStructure("weapon"    , null, "5");
     objectTypeTexts[OBJECT_TYPE_TREASURE]  = new FlagStructure("treasure"  , null, "8");
     objectTypeTexts[OBJECT_TYPE_ARMOR]     = new FlagStructure("armor"     , null, "9");
     objectTypeTexts[OBJECT_TYPE_POTION]    = new FlagStructure("potion"    , null, "10");
     objectTypeTexts[OBJECT_TYPE_CLOTHING]  = new FlagStructure("clothing"  , null, "11");
     objectTypeTexts[OBJECT_TYPE_FURNITURE] = new FlagStructure("furniture" , null, "12");
     objectTypeTexts[OBJECT_TYPE_TRASH]     = new FlagStructure("trash"     , null, "13");
     objectTypeTexts[OBJECT_TYPE_CONTAINER] = new FlagStructure("container" , null, "15");
     objectTypeTexts[OBJECT_TYPE_DRINK_CON] = new FlagStructure("drink"     , null, "17");
     objectTypeTexts[OBJECT_TYPE_KEY]       = new FlagStructure("key"       , null, "18");
     objectTypeTexts[OBJECT_TYPE_FOOD]      = new FlagStructure("food"      , null, "19");
     objectTypeTexts[OBJECT_TYPE_MONEY]     = new FlagStructure("money"     , null, "20");
     objectTypeTexts[OBJECT_TYPE_BOAT]      = new FlagStructure("boat"      , null, "22");
     objectTypeTexts[OBJECT_TYPE_CORPSE_NPC]= new FlagStructure("npc_corpse", null, "23");
     objectTypeTexts[OBJECT_TYPE_CORPSE_PC] = new FlagStructure("pc_corpse" , null, "24");
     objectTypeTexts[OBJECT_TYPE_FOUNTAIN]  = new FlagStructure("fountain"  , null, "25");
     objectTypeTexts[OBJECT_TYPE_PILL]      = new FlagStructure("pill"      , null, "26");
     objectTypeTexts[OBJECT_TYPE_PROTECT]   = new FlagStructure("protect"   , null, "27");
     objectTypeTexts[OBJECT_TYPE_MAP]       = new FlagStructure("map"       , null, "28");
     objectTypeTexts[OBJECT_TYPE_PORTAL]    = new FlagStructure("portal"    , null, "29");
     objectTypeTexts[OBJECT_TYPE_WARP_STONE]= new FlagStructure("warp_stone", null, "30");
     objectTypeTexts[OBJECT_TYPE_ROOM_KEY]  = new FlagStructure("room_key"  , null, "31");
     objectTypeTexts[OBJECT_TYPE_GEM]       = new FlagStructure("gem"       , null, "32");
     objectTypeTexts[OBJECT_TYPE_JEWELRY]   = new FlagStructure("jewelry"   , null, "33");
     objectTypeTexts[OBJECT_TYPE_JUKEBOX]   = new FlagStructure("jukebox"   , null, "34");
     objectTypeTexts[OBJECT_TYPE_TATOO]     = new FlagStructure("tatoo"     , null, "35");
     objectTypeTexts[OBJECT_TYPE_BOMB]      = new FlagStructure("bomb"      , null, "36");

     objectBulkTexts[OBJECT_BULK_CLOTHING]          = new FlagStructure("Clothing"         , null, "0");
     objectBulkTexts[OBJECT_BULK_BUCKLER]           = new FlagStructure("Buckler"          , null, "0");
     objectBulkTexts[OBJECT_BULK_HARD_LEATHER]      = new FlagStructure("Hard leather"     , null, "1");
     objectBulkTexts[OBJECT_BULK_SMALL_SHIELD]      = new FlagStructure("Small shield"     , null, "1");
     objectBulkTexts[OBJECT_BULK_CHAINMAIL]         = new FlagStructure("Scale/Chainmail"  , null, "2");
     objectBulkTexts[OBJECT_BULK_MEDIUM_SHIELD]     = new FlagStructure("Medium shield"    , null, "2");
     objectBulkTexts[OBJECT_BULK_PLATEMAIL]         = new FlagStructure("Platemail"        , null, "3");
     objectBulkTexts[OBJECT_BULK_KITE_SHIELD]       = new FlagStructure("Kite shield"      , null, "3");
     objectBulkTexts[OBJECT_BULK_LIGHT_PLATE_ARMOR] = new FlagStructure("Light plate armor", null, "4");
     objectBulkTexts[OBJECT_BULK_TOWER_SHIELD]      = new FlagStructure("Tower shield"     , null, "4");
     objectBulkTexts[OBJECT_BULK_HEAVY_PLATE_ARMOR] = new FlagStructure("Heavy plate armor", null, "5");

     objectPortalGateFlagTexts[OBJECT_PORTAL_GATEFLAG_NORMAL_EXIT] = new FlagStructure("Normal exit","should be set if no other portal flags are used"    ,"A");
     objectPortalGateFlagTexts[OBJECT_PORTAL_GATEFLAG_NO_CURSE]    = new FlagStructure("No curse"   ,"can't be used by people who are cursed"             ,"B");
     objectPortalGateFlagTexts[OBJECT_PORTAL_GATEFLAG_GO_WITH]     = new FlagStructure("Go with"    ,"portal entrance moves with the person using it"     ,"C");
     objectPortalGateFlagTexts[OBJECT_PORTAL_GATEFLAG_GATE_BUGGY]  = new FlagStructure("Gate buggy" ,"chance it will teleport the person to a random room","D");
     objectPortalGateFlagTexts[OBJECT_PORTAL_GATEFLAG_RANDOM]      = new FlagStructure("Random"     ,"will always teleport the person to a random room"   ,"E");

     objectFurnitureFlagTexts[OBJECT_FURNITURE_FLAG_STAND_AT]   = new FlagStructure("Stand at"  ,null,"A");
     objectFurnitureFlagTexts[OBJECT_FURNITURE_FLAG_STAND_ON]   = new FlagStructure("Stand on"  ,null,"B");
     objectFurnitureFlagTexts[OBJECT_FURNITURE_FLAG_STAND_IN]   = new FlagStructure("Stand in"  ,null,"C");
     objectFurnitureFlagTexts[OBJECT_FURNITURE_FLAG_SIT_AT]     = new FlagStructure("Sit at"    ,null,"D");
     objectFurnitureFlagTexts[OBJECT_FURNITURE_FLAG_SIT_ON]     = new FlagStructure("Sit on"    ,null,"E");
     objectFurnitureFlagTexts[OBJECT_FURNITURE_FLAG_SIT_IN]     = new FlagStructure("Sit in"    ,null,"F");
     objectFurnitureFlagTexts[OBJECT_FURNITURE_FLAG_REST_AT]    = new FlagStructure("Rest at"   ,null,"G");
     objectFurnitureFlagTexts[OBJECT_FURNITURE_FLAG_REST_ON]    = new FlagStructure("Rest on"   ,null,"H");
     objectFurnitureFlagTexts[OBJECT_FURNITURE_FLAG_REST_IN]    = new FlagStructure("Rest in"   ,null,"I");
     objectFurnitureFlagTexts[OBJECT_FURNITURE_FLAG_SLEEP_AT]   = new FlagStructure("Sleep at"  ,null,"J");
     objectFurnitureFlagTexts[OBJECT_FURNITURE_FLAG_SLEEP_ON]   = new FlagStructure("Sleep on"  ,null,"K");
     objectFurnitureFlagTexts[OBJECT_FURNITURE_FLAG_SLEEP_IN]   = new FlagStructure("Sleep in"  ,null,"L");
     objectFurnitureFlagTexts[OBJECT_FURNITURE_FLAG_PUT_AT]     = new FlagStructure("Put at"    ,null,"M");
     objectFurnitureFlagTexts[OBJECT_FURNITURE_FLAG_PUT_ON]     = new FlagStructure("Put on"    ,null,"N");
     objectFurnitureFlagTexts[OBJECT_FURNITURE_FLAG_PUT_IN]     = new FlagStructure("Put in"    ,null,"O");
     objectFurnitureFlagTexts[OBJECT_FURNITURE_FLAG_PUT_INSIDE] = new FlagStructure("Put inside",null,"P");
     objectFurnitureFlagTexts[OBJECT_FURNITURE_FLAG_WRITE_ON]   = new FlagStructure("Write on"  ,null,"Q");

     extraFlagTexts[EXTRAFLAG_GLOWING]      = new FlagStructure("Glowing"     , null                                                     , "A");
     extraFlagTexts[EXTRAFLAG_HUMMING]      = new FlagStructure("Humming"     , null                                                     , "B");
     extraFlagTexts[EXTRAFLAG_DARK]         = new FlagStructure("Dark"        , "(hidden)"                                               , "C");
     extraFlagTexts[EXTRAFLAG_LOCK]         = new FlagStructure("Lock"        , null                                                     , "D");
     extraFlagTexts[EXTRAFLAG_EVIL]         = new FlagStructure("Evil"        , null                                                     , "E");
     extraFlagTexts[EXTRAFLAG_INVISIBLE]    = new FlagStructure("Invisible"   , null                                                     , "F");
     extraFlagTexts[EXTRAFLAG_MAGIC]        = new FlagStructure("Magic"       , null                                                     , "G");
     extraFlagTexts[EXTRAFLAG_NODROP]       = new FlagStructure("No drop"     , null                                                     , "H");
     extraFlagTexts[EXTRAFLAG_BLESS]        = new FlagStructure("Bless"       , null                                                     , "I");
     extraFlagTexts[EXTRAFLAG_ANTI_GOOD]    = new FlagStructure("Anti-good"   , null                                                     , "J");
     extraFlagTexts[EXTRAFLAG_ANTI_EVIL]    = new FlagStructure("Anti-evil"   , null                                                     , "K");
     extraFlagTexts[EXTRAFLAG_ANTI_NEUTRAL] = new FlagStructure("Anti-neutral", null                                                     , "L");
     extraFlagTexts[EXTRAFLAG_NOREMOVE]     = new FlagStructure("No remove"   , "Cannot be removed"                                      , "M");
     extraFlagTexts[EXTRAFLAG_INVENTORY]    = new FlagStructure("Inventory"   , "ONLY FOR OBJECTS BEING SOLD IN SHOPS (prevents run-out)", "N");
     extraFlagTexts[EXTRAFLAG_NOPURGE]      = new FlagStructure("No purge"    , "Should be set for \"stationary\" objects like furniture", "O");
     extraFlagTexts[EXTRAFLAG_ROT_DEATH]    = new FlagStructure("Rot death"   , "Joker against death"                                    , "P");
     extraFlagTexts[EXTRAFLAG_VIS_DEATH]    = new FlagStructure("Vis death"   , "Visible only when his holder is dead"                   , "Q");
     extraFlagTexts[EXTRAFLAG_NOSAC]        = new FlagStructure("No sacrifice", "Cannot be sacrified"                                    , "R");
     extraFlagTexts[EXTRAFLAG_NONMETAL]     = new FlagStructure("Non metal"   , null                                                     , "S");
     extraFlagTexts[EXTRAFLAG_NOLOCATE]     = new FlagStructure("No locate"   , "Cannot be located with \"locate\" spell"                , "T");
     extraFlagTexts[EXTRAFLAG_MELT_DROP]    = new FlagStructure("Melt drop"   , "dissolves when droped"                                  , "U");
     extraFlagTexts[EXTRAFLAG_HAD_TIMER]    = new FlagStructure("Had timer"   , null                                                     , "V");
     extraFlagTexts[EXTRAFLAG_SELL_EXTRACT] = new FlagStructure("Sell extract", "ONLY FOR GEMS STORES (buy-price is sold-price)"         , "W");
     extraFlagTexts[EXTRAFLAG_BURN_PROOF]   = new FlagStructure("Burn proof"  , "Not destroyed by fire"                                  , "Y");
     extraFlagTexts[EXTRAFLAG_NOUNCURSE]    = new FlagStructure("No uncurse"  , null                                                     , "Z");
     extraFlagTexts[EXTRAFLAG_NOSELL]       = new FlagStructure("No sell"     , null                                                     , "a");
     extraFlagTexts[EXTRAFLAG_VIS_ROT]      = new FlagStructure("Vis rot"     , "Only visible when corpse decomposes"                    , "b");

     wearFlagTexts[WEAR_FLAG_TAKE]       = new FlagStructure("Take"           , null, "A");
     wearFlagTexts[WEAR_FLAG_FINGER]     = new FlagStructure("Finger"         , null, "B");
     wearFlagTexts[WEAR_FLAG_NECK]       = new FlagStructure("Neck"           , null, "C");
     wearFlagTexts[WEAR_FLAG_BODY]       = new FlagStructure("On torso"       , null, "D");
     wearFlagTexts[WEAR_FLAG_HEAD]       = new FlagStructure("Head"           , null, "E");
     wearFlagTexts[WEAR_FLAG_LEGS]       = new FlagStructure("Legs"           , null, "F");
     wearFlagTexts[WEAR_FLAG_FEET]       = new FlagStructure("Feet"           , null, "G");
     wearFlagTexts[WEAR_FLAG_HANDS]      = new FlagStructure("Hands"          , null, "H");
     wearFlagTexts[WEAR_FLAG_ARMS]       = new FlagStructure("Arms"           , null, "I");
     wearFlagTexts[WEAR_FLAG_SHIELD]     = new FlagStructure("As shield"      , null, "J");
     wearFlagTexts[WEAR_FLAG_ABOUT_BODY] = new FlagStructure("About body"     , null, "K");
     wearFlagTexts[WEAR_FLAG_WAIST]      = new FlagStructure("Waist"          , null, "L");
     wearFlagTexts[WEAR_FLAG_WRIST]      = new FlagStructure("Left wrist"     , null, "M");
     wearFlagTexts[WEAR_FLAG_WIELD]      = new FlagStructure("Wielded"        , null, "N");
     wearFlagTexts[WEAR_FLAG_HOLD]       = new FlagStructure("Held"           , null, "O");
     wearFlagTexts[WEAR_FLAG_NOSAC]      = new FlagStructure("No sac"         , null, "P");
     wearFlagTexts[WEAR_FLAG_FLOAT]      = new FlagStructure("Floating nearby", null, "Q");
     wearFlagTexts[WEAR_FLAG_TATOO]      = new FlagStructure("Tatoo"          , null, "R");
     wearFlagTexts[WEAR_FLAG_TWO_HANDS]  = new FlagStructure("Two hands"      , null, "S");

     applyTexts[APPLY_NONE]          = new FlagStructure("None"          , null, Integer.toString(APPLY_NONE));
     applyTexts[APPLY_STRENGTH]      = new FlagStructure("Strength"      , null, Integer.toString(APPLY_STRENGTH));
     applyTexts[APPLY_DEXTERITY]     = new FlagStructure("Dexterity"     , null, Integer.toString(APPLY_DEXTERITY));
     applyTexts[APPLY_INTELLIGENCE]  = new FlagStructure("Intelligence"  , null, Integer.toString(APPLY_WISDOM));
     applyTexts[APPLY_WISDOM]        = new FlagStructure("Wisdom"        , null, Integer.toString(APPLY_WISDOM));
     applyTexts[APPLY_CONSTITUTION]  = new FlagStructure("Constitution"  , null, Integer.toString(APPLY_CHARISMA));
     applyTexts[APPLY_CHARISMA]      = new FlagStructure("Charisma"      , null, Integer.toString(APPLY_CHARISMA));
     applyTexts[APPLY_CLASS]         = new FlagStructure("Class"         , null, Integer.toString(APPLY_LEVEL));
     applyTexts[APPLY_LEVEL]         = new FlagStructure("Level"         , null, Integer.toString(APPLY_LEVEL));
     applyTexts[APPLY_AGE]           = new FlagStructure("Age"           , null, Integer.toString(APPLY_AGE));
     applyTexts[APPLY_HEIGHT]        = new FlagStructure("Height"        , null, Integer.toString(APPLY_WEIGHT));
     applyTexts[APPLY_WEIGHT]        = new FlagStructure("Weight"        , null, Integer.toString(APPLY_WEIGHT));
     applyTexts[APPLY_MANA]          = new FlagStructure("Mana"          , null, Integer.toString(APPLY_MANA));
     applyTexts[APPLY_HITPOINTS]     = new FlagStructure("Hitpoints"     , null, Integer.toString(APPLY_HITPOINTS));
     applyTexts[APPLY_MOVEMENT]      = new FlagStructure("Movement"      , null, Integer.toString(APPLY_MOVEMENT));
     applyTexts[APPLY_GOLD]          = new FlagStructure("Gold"          , null, Integer.toString(APPLY_GOLD));
     applyTexts[APPLY_EXP]           = new FlagStructure("Exp"           , null, Integer.toString(APPLY_EXP));
     applyTexts[APPLY_ARMOR_CLASS]   = new FlagStructure("Armor class"   , null, Integer.toString(APPLY_ARMOR_CLASS));
     applyTexts[APPLY_HITROLL]       = new FlagStructure("Hitroll"       , null, Integer.toString(APPLY_HITROLL));
     applyTexts[APPLY_DAMROLL]       = new FlagStructure("Damroll"       , null, Integer.toString(APPLY_DAMROLL));
     applyTexts[APPLY_SAVES]         = new FlagStructure("Saves"         , null, Integer.toString(APPLY_SAVES));
     applyTexts[APPLY_SAVING_PARA]   = new FlagStructure("Save vs para"  , null, Integer.toString(APPLY_SAVING_PARA));
     applyTexts[APPLY_SAVING_ROD]    = new FlagStructure("Save vs rod"   , null, Integer.toString(APPLY_SAVING_PETRI));
     applyTexts[APPLY_SAVING_PETRI]  = new FlagStructure("Save vs petri" , null, Integer.toString(APPLY_SAVING_PETRI));
     applyTexts[APPLY_SAVING_BREATH] = new FlagStructure("Save vs breath", null, Integer.toString(APPLY_SAVING_BREATH));
     applyTexts[APPLY_SAVING_SPELL]  = new FlagStructure("Save vs spell" , null, Integer.toString(APPLY_SAVING_SPELL));
     applyTexts[APPLY_SPELL_AFFECT]  = new FlagStructure("spell affect"  , null, Integer.toString(APPLY_SPELL_AFFECT));
     applyTexts[APPLY_SIZE]          = new FlagStructure("Size"          , null, Integer.toString(APPLY_SIZE));
     applyTexts[APPLY_SEX]           = new FlagStructure("Sex"           , null, Integer.toString(APPLY_SEX));

     affTexts[AFF_BLIND]            = new FlagStructure("Blind"           , null, "A");
     affTexts[AFF_INVISIBLE]        = new FlagStructure("Invisible"       , null, "B");
     affTexts[AFF_IMP_INVIS]        = new FlagStructure("Only IMP can see", null, "C");
     affTexts[AFF_FADE]             = new FlagStructure("Fade"            , null, "D");
     affTexts[AFF_SCREAM]           = new FlagStructure("Scream"          , null, "E");
     affTexts[AFF_BLOODTHIRST]      = new FlagStructure("Bloodthirst"     , null, "F");
     affTexts[AFF_STUN]             = new FlagStructure("Stun"            , null, "G");
     affTexts[AFF_SANCTUARY]        = new FlagStructure("Sanctuary"       , null, "H");
     affTexts[AFF_FAERIE_FIRE]      = new FlagStructure("Faerie fire"     , null, "I");
     affTexts[AFF_INFRARED]         = new FlagStructure("Infrared"        , null, "J");
     affTexts[AFF_CURSE]            = new FlagStructure("Curse"           , null, "K");
     affTexts[AFF_CORRUPTION]       = new FlagStructure("Corruption"      , null, "K");
     affTexts[AFF_POISON]           = new FlagStructure("Poison"          , null, "M");
     affTexts[AFF_PROTECT_EVIL]     = new FlagStructure("Protection evil" , null, "N");
     affTexts[AFF_PROTECT_GOOD]     = new FlagStructure("Protection good" , null, "O");
     affTexts[AFF_SNEAK]            = new FlagStructure("Sneak"           , null, "P");
     affTexts[AFF_HIDE]             = new FlagStructure("Hide"            , null, "Q");
     affTexts[AFF_SLEEP]            = new FlagStructure("Sleep"           , null, "R");
     affTexts[AFF_CHARM]            = new FlagStructure("Charm"           , null, "S");
     affTexts[AFF_FLYING]           = new FlagStructure("Flying"          , null, "T");
     affTexts[AFF_PASS_DOOR]        = new FlagStructure("Pass door"       , null, "U");
     affTexts[AFF_HASTE]            = new FlagStructure("Haste"           , null, "V");
     affTexts[AFF_CALM]             = new FlagStructure("Calm"            , null, "W");
     affTexts[AFF_PLAGUE]           = new FlagStructure("Plague"          , null, "X");
     affTexts[AFF_WEAKEN]           = new FlagStructure("Weaken"          , null, "Y");
     affTexts[AFF_WEAKEN_STUN]      = new FlagStructure("Weaken stun"     , null, "Z");
     affTexts[AFF_BERSERK]          = new FlagStructure("Berserk"         , null, "a");
     affTexts[AFF_SWIM]             = new FlagStructure("Swim"            , null, "b");
     affTexts[AFF_REGENERATION]     = new FlagStructure("Regeneration"    , null, "c");
     affTexts[AFF_SLOW]             = new FlagStructure("Slow"            , null, "d");
     affTexts[AFF_CAMOUFLAGE]       = new FlagStructure("Camouflage"      , null, "e");
     //affTexts[AFF_FLAMING]          = new FlagStructure("Flaming"         , null, "A");
     //affTexts[AFF_HOLD]             = new FlagStructure("Hold"            , null, "B");
     //affTexts[AFF_PROTECT]          = new FlagStructure("Protect"         , null, "C");
     //affTexts[AFF_PARALYSIS]        = new FlagStructure("Paralysis"       , null, "D");
     affTexts[AFF_FLAMING]          = new FlagStructure("Flaming"         , null, "#");
     affTexts[AFF_HOLD]             = new FlagStructure("Hold"            , null, "#");
     affTexts[AFF_PROTECT]          = new FlagStructure("Protect"         , null, "#");
     affTexts[AFF_PARALYSIS]        = new FlagStructure("Paralysis"       , null, "#");

     detTexts[DET_DETECT_IMP_INVIS] = new FlagStructure("Detect imp invis", null, "A");
     detTexts[DET_DETECT_FADE]      = new FlagStructure("Detect fade"     , null, "B");
     detTexts[DET_DETECT_EVIL]      = new FlagStructure("Detect evil"     , null, "C");
     detTexts[DET_DETECT_INVISIBLE] = new FlagStructure("Detect invisible", null, "D");
     detTexts[DET_DETECT_MAGIC]     = new FlagStructure("Detect magic"    , null, "E");
     detTexts[DET_DETECT_HIDDEN]    = new FlagStructure("Detect hidden"   , null, "F");
     detTexts[DET_DETECT_GOOD]      = new FlagStructure("Detect good"     , null, "G");
     detTexts[DET_DETECT_SNEAK]     = new FlagStructure("Detect sneak"    , null, "H");
     detTexts[DET_DETECT_UNDEAD]    = new FlagStructure("Detect undead"   , null, "I");
     detTexts[DET_DARK_VISION]      = new FlagStructure("Dark vision"     , null, "Z");
     detTexts[DET_ACUTE_VISION]     = new FlagStructure("Acute vision"    , null, "e");

     mobileLevelRecommendedValues[0]  = new MobileLevelValues(0,0,0,0,0,0,0,0,0,0,0,0,0,0);
     mobileLevelRecommendedValues[1]  = new MobileLevelValues(0,2,6,10,5,5,100,1,4,0,9,9,9,10);
     mobileLevelRecommendedValues[2]  = new MobileLevelValues(0,2,7,21,5,5,100,1,5,0,8,8,8,10);
     mobileLevelRecommendedValues[3]  = new MobileLevelValues(0,2,6,35,5,5,100,1,6,0,7,7,7,10);
     mobileLevelRecommendedValues[4]  = new MobileLevelValues(0,2,7,46,5,5,100,1,5,1,6,6,6,9);
     mobileLevelRecommendedValues[5]  = new MobileLevelValues(0,2,6,60,5,5,100,1,6,1,5,5,5,9);
     mobileLevelRecommendedValues[6]  = new MobileLevelValues(0,2,7,71,5,5,100,1,7,1,4,4,4,8);
     mobileLevelRecommendedValues[7]  = new MobileLevelValues(0,2,6,85,5,5,100,1,8,1,4,4,4,8);
     mobileLevelRecommendedValues[8]  = new MobileLevelValues(0,2,7,96,5,5,100,1,7,2,3,3,3,8);
     mobileLevelRecommendedValues[9]  = new MobileLevelValues(0,2,6,110,5,5,100,1,8,2,2,2,2,7);
     mobileLevelRecommendedValues[10] = new MobileLevelValues(0,2,7,121,5,5,100,2,4,2,1,1,1,7);
     mobileLevelRecommendedValues[11] = new MobileLevelValues(0,2,8,134,5,5,100,1,10,2,1,1,1,7);
     mobileLevelRecommendedValues[12] = new MobileLevelValues(0,2,10,150,5,5,100,1,10,3,0,0,0,7);
     mobileLevelRecommendedValues[13] = new MobileLevelValues(0,2,10,170,5,5,100,2,5,3,-1,-1,-1,6);
     mobileLevelRecommendedValues[14] = new MobileLevelValues(0,2,10,190,5,5,100,1,12,3,-1,-1,-1,6);
     mobileLevelRecommendedValues[15] = new MobileLevelValues(0,3,9,208,5,5,100,2,6,3,-2,-2,-2,6);
     mobileLevelRecommendedValues[16] = new MobileLevelValues(0,3,9,233,5,5,1000,2,6,4,-2,-2,-2,6);
     mobileLevelRecommendedValues[17] = new MobileLevelValues(0,3,9,258,5,5,1000,3,4,4,-3,-3,-3,6);
     mobileLevelRecommendedValues[18] = new MobileLevelValues(0,3,9,283,5,5,1000,2,7,4,-3,-3,-3,6);
     mobileLevelRecommendedValues[19] = new MobileLevelValues(0,3,9,308,5,5,1000,2,7,4,-3,-3,-3,6);
     mobileLevelRecommendedValues[20] = new MobileLevelValues(0,3,9,333,5,5,1000,2,8,5,-4,-4,-4,5);
     mobileLevelRecommendedValues[21] = new MobileLevelValues(0,4,10,360,5,5,1000,4,4,5,-5,-5,-5,5);
     mobileLevelRecommendedValues[22] = new MobileLevelValues(0,5,10,400,5,5,1000,4,4,6,-5,-5,-5,5);
     mobileLevelRecommendedValues[23] = new MobileLevelValues(0,5,10,450,5,5,1000,3,6,6,-6,-6,-6,4);
     mobileLevelRecommendedValues[24] = new MobileLevelValues(0,5,10,500,5,5,1000,2,10,6,-6,-6,-6,4);
     mobileLevelRecommendedValues[25] = new MobileLevelValues(0,5,10,550,5,5,1000,2,10,7,-7,-7,-7,4);
     mobileLevelRecommendedValues[26] = new MobileLevelValues(0,5,10,600,5,5,1000,3,7,7,-7,-7,-7,4);
     mobileLevelRecommendedValues[27] = new MobileLevelValues(0,5,10,650,5,5,1000,5,4,7,-8,-8,-8,4);
     mobileLevelRecommendedValues[28] = new MobileLevelValues(0,6,12,703,5,5,1000,2,12,8,-8,-8,-8,4);
     mobileLevelRecommendedValues[29] = new MobileLevelValues(0,6,12,778,5,5,1000,2,12,8,-9,-9,-9,3);
     mobileLevelRecommendedValues[30] = new MobileLevelValues(0,6,12,853,5,5,1000,4,6,8,-9,-9,-9,3);
     mobileLevelRecommendedValues[31] = new MobileLevelValues(0,6,12,928,5,5,1000,4,6,9,-10,-10,-10,3);
     mobileLevelRecommendedValues[32] = new MobileLevelValues(0,10,10,1000,5,5,1000,6,4,9,-10,-10,-10,3);
     mobileLevelRecommendedValues[33] = new MobileLevelValues(0,10,10,1100,5,5,1000,6,4,10,-11,-11,-11,3);
     mobileLevelRecommendedValues[34] = new MobileLevelValues(0,10,10,1200,5,5,1000,4,7,10,-11,-11,-11,3);
     mobileLevelRecommendedValues[35] = new MobileLevelValues(0,10,10,1300,5,5,1000,4,7,11,-11,-11,-11,3);
     mobileLevelRecommendedValues[36] = new MobileLevelValues(0,10,10,1400,5,5,1000,3,10,11,-12,-12,-12,3);
     mobileLevelRecommendedValues[37] = new MobileLevelValues(0,10,10,1500,5,5,1000,3,10,12,-12,-12,-12,3);
     mobileLevelRecommendedValues[38] = new MobileLevelValues(0,10,10,1600,5,5,1000,5,6,12,-13,-13,-13,3);
     mobileLevelRecommendedValues[39] = new MobileLevelValues(0,15,10,1700,5,5,1000,5,6,13,-13,-13,-13,3);
     mobileLevelRecommendedValues[40] = new MobileLevelValues(0,15,10,1850,5,5,1000,4,8,13,-13,-13,-13,3);
     mobileLevelRecommendedValues[41] = new MobileLevelValues(0,25,10,2000,5,5,1000,4,8,14,-14,-14,-14,3);
     mobileLevelRecommendedValues[42] = new MobileLevelValues(0,25,10,2250,5,5,1000,3,12,14,-14,-14,-14,3);
     mobileLevelRecommendedValues[43] = new MobileLevelValues(0,25,10,2500,5,5,1000,3,12,15,-15,-15,-15,2);
     mobileLevelRecommendedValues[44] = new MobileLevelValues(0,25,10,2750,5,5,1000,8,4,15,-15,-15,-15,2);
     mobileLevelRecommendedValues[45] = new MobileLevelValues(0,25,10,3000,5,5,1000,8,4,16,-15,-15,-15,2);
     mobileLevelRecommendedValues[46] = new MobileLevelValues(0,25,10,3250,5,5,1000,6,6,16,-16,-16,-16,2);
     mobileLevelRecommendedValues[47] = new MobileLevelValues(0,25,10,3500,5,5,1000,6,6,17,-17,-17,-17,2);
     mobileLevelRecommendedValues[48] = new MobileLevelValues(0,25,10,3750,5,5,1000,6,6,18,-18,-18,-18,1);
     mobileLevelRecommendedValues[49] = new MobileLevelValues(0,50,10,4000,5,5,1000,4,10,18,-19,-19,-19,1);
     mobileLevelRecommendedValues[50] = new MobileLevelValues(0,50,10,4250,5,5,5000,5,8,19,-20,-20,-20,0);
     mobileLevelRecommendedValues[51] = new MobileLevelValues(0,50,10,4500,5,5,5000,5,8,20,-21,-21,-21,0);
     mobileLevelRecommendedValues[52] = new MobileLevelValues(0,50,10,4750,5,5,5000,6,7,20,-22,-22,-22,0);
     mobileLevelRecommendedValues[53] = new MobileLevelValues(0,50,10,5000,5,5,5000,6,7,21,-23,-23,-23,-1);
     mobileLevelRecommendedValues[54] = new MobileLevelValues(0,50,10,5250,5,5,5000,7,6,22,-24,-24,-24,-1);
     mobileLevelRecommendedValues[55] = new MobileLevelValues(0,50,10,5500,5,5,5000,10,4,23,-25,-25,-25,-1);
     mobileLevelRecommendedValues[56] = new MobileLevelValues(0,50,10,5750,5,5,5000,10,4,24,-26,-26,-26,-2);
     mobileLevelRecommendedValues[57] = new MobileLevelValues(0,50,10,6000,5,5,5000,6,8,24,-27,-27,-27,-2);
     mobileLevelRecommendedValues[58] = new MobileLevelValues(0,50,10,6250,5,5,5000,5,10,25,-28,-28,-28,-3);
     mobileLevelRecommendedValues[59] = new MobileLevelValues(0,50,10,6500,5,5,5000,8,6,26,-29,-29,-29,-3);
     mobileLevelRecommendedValues[60] = new MobileLevelValues(0,50,10,6750,5,5,5000,8,6,28,-30,-30,-30,-3);
     mobileLevelRecommendedValues[61] = new MobileLevelValues(0,50,10,7000,5,5,5000,6,9,31,-31,-31,-31,-4);
     mobileLevelRecommendedValues[62] = new MobileLevelValues(0,50,10,7250,5,5,5000,6,9,32,-32,-32,-32,-4);
     mobileLevelRecommendedValues[63] = new MobileLevelValues(0,50,10,7500,5,5,5000,7,8,33,-33,-33,-33,-4);
     mobileLevelRecommendedValues[64] = new MobileLevelValues(0,50,10,8000,5,5,5000,8,7,34,-34,-34,-34,-4);
     mobileLevelRecommendedValues[65] = new MobileLevelValues(0,50,10,8250,5,5,5000,11,5,35,-35,-35,-35,-5);
     mobileLevelRecommendedValues[66] = new MobileLevelValues(0,50,10,8500,5,5,5000,7,9,36,-36,-36,-36,-5);
     mobileLevelRecommendedValues[67] = new MobileLevelValues(0,50,10,8750,5,5,5000,8,8,37,-37,-37,-37,-5);
     mobileLevelRecommendedValues[68] = new MobileLevelValues(0,50,10,9000,5,5,5000,7,10,38,-38,-38,-38,-6);
     mobileLevelRecommendedValues[69] = new MobileLevelValues(0,50,10,9500,5,5,5000,7,11,39,-39,-39,-39,-6);
     mobileLevelRecommendedValues[70] = new MobileLevelValues(0,50,10,9750,5,5,5000,8,10,40,-40,-40,-40,-6);
     mobileLevelRecommendedValues[71] = new MobileLevelValues(0,50,10,10000,5,5,5000,9,9,41,-41,-41,-41,-7);
     mobileLevelRecommendedValues[72] = new MobileLevelValues(0,50,10,10250,5,5,5000,8,11,42,-42,-42,-42,-7);
     mobileLevelRecommendedValues[73] = new MobileLevelValues(0,50,10,10500,5,5,5000,9,10,43,-43,-43,-43,-8);
     mobileLevelRecommendedValues[74] = new MobileLevelValues(0,50,10,10700,5,5,5000,9,10,44,-44,-44,-44,-8);
     mobileLevelRecommendedValues[75] = new MobileLevelValues(0,50,10,11000,5,5,5000,9,11,45,-45,-45,-45,-9);
     mobileLevelRecommendedValues[76] = new MobileLevelValues(0,50,10,11500,5,5,5000,10,10,46,-46,-46,-46,-9);
     mobileLevelRecommendedValues[77] = new MobileLevelValues(0,50,10,12000,5,5,5000,9,12,47,-47,-47,-47,-10);
     mobileLevelRecommendedValues[78] = new MobileLevelValues(0,50,10,12500,5,5,5000,9,12,48,-48,-48,-48,-11);
     mobileLevelRecommendedValues[79] = new MobileLevelValues(0,50,10,13000,5,5,5000,10,11,49,-49,-49,-49,-12);
     mobileLevelRecommendedValues[80] = new MobileLevelValues(0,50,10,13500,5,5,5000,13,9,50,-50,-50,-50,-13);
     mobileLevelRecommendedValues[81] = new MobileLevelValues(0,50,10,14000,5,5,5000,13,9,51,-51,-51,-51,-14);
     mobileLevelRecommendedValues[82] = new MobileLevelValues(0,50,10,14500,5,5,5000,10,12,52,-52,-52,-52,-15);
     mobileLevelRecommendedValues[83] = new MobileLevelValues(0,50,10,15000,5,5,5000,9,14,53,-53,-53,-53,-16);
     mobileLevelRecommendedValues[84] = new MobileLevelValues(0,50,10,15500,5,5,5000,9,14,54,-54,-54,-54,-18);
     mobileLevelRecommendedValues[85] = new MobileLevelValues(0,50,10,16000,5,5,5000,10,13,55,-55,-55,-55,-20);
     mobileLevelRecommendedValues[86] = new MobileLevelValues(0,50,10,16500,5,5,5000,12,11,56,-56,-56,-56,-22);
     mobileLevelRecommendedValues[87] = new MobileLevelValues(0,50,10,17000,5,5,5000,14,10,57,-57,-57,-57,-23);
     mobileLevelRecommendedValues[88] = new MobileLevelValues(0,50,10,17500,5,5,5000,10,15,58,-58,-58,-58,-24);
     mobileLevelRecommendedValues[89] = new MobileLevelValues(0,50,10,18000,5,5,5000,15,13,59,-59,-59,-59,-26);
     mobileLevelRecommendedValues[90] = new MobileLevelValues(0,50,10,18500,5,5,5000,17,9,60,-60,-60,-60,-27);
     mobileLevelRecommendedValues[91] = new MobileLevelValues(0,50,10,19000,5,5,5000,16,10,61,-61,-61,-61,-29);
     mobileLevelRecommendedValues[92] = new MobileLevelValues(0,50,10,19500,5,5,5000,10,17,62,-63,-63,-63,-31);
     mobileLevelRecommendedValues[93] = new MobileLevelValues(0,50,10,20500,5,5,5000,17,10,64,-66,-66,-66,-33);
     mobileLevelRecommendedValues[94] = new MobileLevelValues(0,50,10,21500,5,5,5000,10,18,66,-69,-69,-69,-35);
     mobileLevelRecommendedValues[95] = new MobileLevelValues(0,50,10,22500,5,5,5000,12,15,68,-71,-71,-71,-37);

     mobileRecommendedRaceFlagsValues[RACE_TYPE_AARAKOCRA]       = new MobileRaceFlagsValues("Aarakocra","C","T","H","","","","AHMV","ABCDEFGHIJKP");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_AIR_ELEMENTAL]   = new MobileRaceFlagsValues("air elemental","","","G","","DA","","AHZ","AD");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_ARIAL]           = new MobileRaceFlagsValues("arial","","T","","","","","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_BAT]             = new MobileRaceFlagsValues("bat","","T","FH","","","S","AGV","ABCDEFHJKUV");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_BEAR]            = new MobileRaceFlagsValues("bear","","","OED","","EI","","AGV","ABCDEFHJKUV");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_BEHOLDER]        = new MobileRaceFlagsValues("beholder","","c","","","C","","AHMV","AEK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_BLACK_DRAGON]    = new MobileRaceFlagsValues("black dragon","","JT","","","BEK","F","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_BLUE_DRAGON]     = new MobileRaceFlagsValues("blue dragon","","JT","","","BEJ","F","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_BRASS_DRAGON]    = new MobileRaceFlagsValues("brass dragon","","JT","","","BEH","F","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_BRONZE_DRAGON]   = new MobileRaceFlagsValues("bronze dragon","","JT","","","BEJ","F","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_CAT]             = new MobileRaceFlagsValues("cat","","","HF","","","","AGV","ACDEFHJKQUV");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_CENTAUR]         = new MobileRaceFlagsValues("centaur","","","CIJF","","D","","AGNV","ABCDEFGHIJKQ");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_CENTIPEDE]       = new MobileRaceFlagsValues("centipede","","","","","FI","E","ABGO","ACK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_CLOUD_GIANT]     = new MobileRaceFlagsValues("cloud giant","","T","","","D","PJ","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_COPPER_DRAGON]   = new MobileRaceFlagsValues("copper dragon","","JT","","","BEK","F","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_DEATH_KNIGHT]    = new MobileRaceFlagsValues("death knight","O","","EK","QLR","HM","N","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_DARK_ELF]        = new MobileRaceFlagsValues("dark-elf","","JP","","","B","Z","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_DOG]             = new MobileRaceFlagsValues("dog","","","H","","","","AGV","ACDEFHJKUV");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_DOLL]            = new MobileRaceFlagsValues("doll","","","","ILNMPQR","ES","GHKJO","AJMcc","ABCGHK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_DRACOLICH]       = new MobileRaceFlagsValues("dracolish","","JT","","BJI","HE","FI","AHZ","ACDEFGHIJKPQUVX");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_DRACONIAN]       = new MobileRaceFlagsValues("draconian","","","","","GL","I","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_DRAGON]          = new MobileRaceFlagsValues("dragon","","JT","","","HEB","FI","AHZ","ACDEFGHIJKPQUVX");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_DROW]            = new MobileRaceFlagsValues("drow","","J","","","B","Z","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_DUERGAR]         = new MobileRaceFlagsValues("duergar","","J","","","CLQ","R","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_DWARF]           = new MobileRaceFlagsValues("dwarf","","J","","","LQ","R","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_EARTH_ELEMENTAL] = new MobileRaceFlagsValues("earth elemental","","","G","","DA","","AHZ","AD");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_ELF]             = new MobileRaceFlagsValues("elf","","JP","","","B","Z","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_FELAR]           = new MobileRaceFlagsValues("felar","","J","M","","JI","HR","AHMV","ACDEFHJQUV");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_FIDO]            = new MobileRaceFlagsValues("fido","","","FR","","","C","ABGV","ACDEFHJKQV");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_FIRE_ELEMENTAL]  = new MobileRaceFlagsValues("fire elemental","","","G","H","D","I","AHZ","AD");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_FIRE_GIANT]      = new MobileRaceFlagsValues("fire giant","","","","","H","PJI","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_FISH]            = new MobileRaceFlagsValues("fish","","b","","","","IH","AHZ","ADEFJKOX");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_FOX]             = new MobileRaceFlagsValues("fox","","","HF","","","","AGV","ACDEFHJKQV");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_FROST_GIANT]     = new MobileRaceFlagsValues("frost giant","","","","","I","PJH","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_GARGOYLE]        = new MobileRaceFlagsValues("gargoyle","F","T","H","","D","IH","AGW","ACDEFHKP");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_GITHYANKI]       = new MobileRaceFlagsValues("githyanki","","J","","","","","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_GHOST]           = new MobileRaceFlagsValues("ghost","O","T","G","RMQL","D","N","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_GHOUL]           = new MobileRaceFlagsValues("ghoul","","","","Q","B","C","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_GIANT]           = new MobileRaceFlagsValues("giant","","","","","HI","PJ","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_GNOLL]           = new MobileRaceFlagsValues("gnoll","","DF","E","","","","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_GNOME]           = new MobileRaceFlagsValues("gnome","","J","","","L","E","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_GOBLIN]          = new MobileRaceFlagsValues("goblin","","J","","","Q","C","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_GOLD_DRAGON]     = new MobileRaceFlagsValues("gold dragon","","JT","","","BEHL","F","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_GOLEM]           = new MobileRaceFlagsValues("golem","","","","LQ","D","PC","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_GREEN_DRAGON]    = new MobileRaceFlagsValues("green dragon","","JT","","","BEL","F","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_HALF_ELF]        = new MobileRaceFlagsValues("half-elf","","J","","","B","Z","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_HARPY]           = new MobileRaceFlagsValues("harpy","","DFT","E","","B","","AHMV","ABCDEFGHIJKP");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_HOBGOBLIN]       = new MobileRaceFlagsValues("hobgoblin","","J","","","QL","","AHMV","ABCDEFGHIJKY");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_HORSE]           = new MobileRaceFlagsValues("horse","","","HF","","","","AGV","ACDEFJKQ");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_HUMAN]           = new MobileRaceFlagsValues("human","","","","","","","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_KOBOLD]          = new MobileRaceFlagsValues("kobold","","J","","","L","C","ABHMV","ABCDEFGHIJKQ");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_LAMIA]           = new MobileRaceFlagsValues("lamia","F","","FH","","B","","AGV","ABCDEFGHIJKQU");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_LICH]            = new MobileRaceFlagsValues("lich","","","FH","LQBJI","DH","N","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_LION]            = new MobileRaceFlagsValues("lion","B","","FH","","D","","AGV","ACDEFHJKQUV");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_LIZARD]          = new MobileRaceFlagsValues("lizard","","","","","L","I","AGXcc","ACDEFHKQV");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_MINOTAUR]        = new MobileRaceFlagsValues("minotaur","","","OHC","","","","AHMV","ABCDEFGHIJKW");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_MODRON]          = new MobileRaceFlagsValues("modron","","J","RQ","BCPNM","HIK","","H","ABCGHJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_OGRE]            = new MobileRaceFlagsValues("ogre","","J","","","Q","","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_ORC]             = new MobileRaceFlagsValues("orc","","J","","","Q","J","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_PIG]             = new MobileRaceFlagsValues("pig","","","","","","","AGV","ACDEFHJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_RABBIT]          = new MobileRaceFlagsValues("rabbit","","","FH","","","","AGV","ACDEFHJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_RED_DRAGON]      = new MobileRaceFlagsValues("red dragon","","JT","","","BEH","F","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_ROCKSEER]        = new MobileRaceFlagsValues("rockseer","","JPU","","","B","X","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_SATYR]           = new MobileRaceFlagsValues("satyr","","J","","","QX","","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_SCHOOL_MONSTER]  = new MobileRaceFlagsValues("school monster","U","","","BA","","C","AMV","ABCDEFHJKQU");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_SILVER_DRAGON]   = new MobileRaceFlagsValues("silver dragon","","JT","","","BEI","F","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_SNAKE]           = new MobileRaceFlagsValues("snake","","","","","L","I","AGXYcc","ADEFKLQVX");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_SONG_BIRD]       = new MobileRaceFlagsValues("song bird","","T","FH","","","","AGW","ACDEFHKP");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_SPRITE]          = new MobileRaceFlagsValues("sprite","","CT","G","","","ID","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_STORM_GIANT]     = new MobileRaceFlagsValues("storm giant","","T","","","J","PIH","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_SVIRFNEBLI]      = new MobileRaceFlagsValues("svirfnebli","","J","","","LQC","E","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_TIGER]           = new MobileRaceFlagsValues("tiger","","","FH","","D","","AGV","ACDEFHJKQUV");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_TROLL]           = new MobileRaceFlagsValues("troll","","Jc","D","","BE","HK","ABHMV","ABCDEFGHIJKUV");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_UNDEAD]          = new MobileRaceFlagsValues("undead","","BCDOUJ","","RQL","MO","YNS","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_WATER_ELEMENTAL] = new MobileRaceFlagsValues("water elemental","","","G","","GHI","","AHZ","AD");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_WATER_FOWL]      = new MobileRaceFlagsValues("water flow","","bT","","","R","","AGW","ACDEFHKP");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_WHITE_DRAGON]    = new MobileRaceFlagsValues("white dragon","","JT","","","BEI","F","AHMV","ABCDEFGHIJK");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_WOLF]            = new MobileRaceFlagsValues("wolf","","","FH","","","","AGV","ACDEFJKQV");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_WYVERN]          = new MobileRaceFlagsValues("wyvern","","DFT","CFH","L","","S","ABGZ","ACDEFHJKQVX");
     mobileRecommendedRaceFlagsValues[RACE_TYPE_ZOMBIE]          = new MobileRaceFlagsValues("zombie","","d","","CLB","I","D","AHMV","ABCDEFGHIJK");

     wellKnownMobs[WELL_KNOWN_MOB_CITYGUARD]        = new WellKnownMob(3060,"Cityguard");
     wellKnownMobs[WELL_KNOWN_MOB_VAGABOND]         = new WellKnownMob(3063,"Vagabond");
     wellKnownMobs[WELL_KNOWN_MOB_CAT]              = new WellKnownMob(3066,"Cat");
     wellKnownMobs[WELL_KNOWN_MOB_FIDO]             = new WellKnownMob(3062,"Fido");
     wellKnownMobs[WELL_KNOWN_MOB_SAGE]             = new WellKnownMob(3066,"Sage");
     wellKnownMobs[WELL_KNOWN_MOB_VAMPIRE]          = new WellKnownMob(3404,"Cityguard");
     wellKnownMobs[WELL_KNOWN_MOB_SHADOW]           = new WellKnownMob(10,"Vagabond");
     wellKnownMobs[WELL_KNOWN_MOB_SPECIAL_GUARD]    = new WellKnownMob(11,"Special guard");
     wellKnownMobs[WELL_KNOWN_MOB_BEAR]             = new WellKnownMob(12,"Bear");
     wellKnownMobs[WELL_KNOWN_MOB_DEMON]            = new WellKnownMob(13,"Demon");
     wellKnownMobs[WELL_KNOWN_MOB_NIGHTWALKER]      = new WellKnownMob(14,"NightWalker");
     wellKnownMobs[WELL_KNOWN_MOB_STALKER]          = new WellKnownMob(15,"Stalker");
     wellKnownMobs[WELL_KNOWN_MOB_SQUIRE]           = new WellKnownMob(16,"Squire");
     wellKnownMobs[WELL_KNOWN_MOB_MIRROR_IMAGE]     = new WellKnownMob(17,"Mirror Image");
     wellKnownMobs[WELL_KNOWN_MOB_UNDEAD]           = new WellKnownMob(18,"Undead");
     wellKnownMobs[WELL_KNOWN_MOB_LION]             = new WellKnownMob(19,"Lion");
     wellKnownMobs[WELL_KNOWN_MOB_WOLF]             = new WellKnownMob(20,"Wolf");
     wellKnownMobs[WELL_KNOWN_MOB_SUM_SHADOW]       = new WellKnownMob(26,"Sum shadow");
     wellKnownMobs[WELL_KNOWN_MOB_LESSER_GOLEM]     = new WellKnownMob(21,"Lesser Golem");
     wellKnownMobs[WELL_KNOWN_MOB_STONE_GOLEM]      = new WellKnownMob(22,"Stone Golem");
     wellKnownMobs[WELL_KNOWN_MOB_IRON_GOLEM]       = new WellKnownMob(23,"Iron golem");
     wellKnownMobs[WELL_KNOWN_MOB_ADAMANTITE_GOLEM] = new WellKnownMob(24,"Adamantite golem");
     wellKnownMobs[WELL_KNOWN_MOB_HUNTER]           = new WellKnownMob(25,"Hunter");
     wellKnownMobs[WELL_KNOWN_MOB_PATROLMAN]        = new WellKnownMob(2106,"Patrolman");
     wellKnownMobs[WELL_KNOWN_MOB_TROLLS]           = new WellKnownMob(2100,"Trolls");
     wellKnownMobs[WELL_KNOWN_MOB_OGRES]            = new WellKnownMob(2101,"Ogres");

     wellKnownObjs[WELL_KNOWN_OBJ_SILVER_ONE]        = new WellKnownObj(1, "Silver one");
     wellKnownObjs[WELL_KNOWN_OBJ_GOLD_ONE]          = new WellKnownObj(2, "Gold one");
     wellKnownObjs[WELL_KNOWN_OBJ_GOLD_SOME]         = new WellKnownObj(3, "Gold some");
     wellKnownObjs[WELL_KNOWN_OBJ_SILVER_SOME]       = new WellKnownObj(4, "Silver some");
     wellKnownObjs[WELL_KNOWN_OBJ_COINS]             = new WellKnownObj(5, "Coins");
     wellKnownObjs[WELL_KNOWN_OBJ_CORPSE_NPC]        = new WellKnownObj(10, "Corpse NPC");
     wellKnownObjs[WELL_KNOWN_OBJ_CORPSE_PC]         = new WellKnownObj(11, "Corpse PC");
     wellKnownObjs[WELL_KNOWN_OBJ_SEVERED_HEAD]      = new WellKnownObj(12, "Severed Head");
     wellKnownObjs[WELL_KNOWN_OBJ_TORN_HEAD]         = new WellKnownObj(13, "Torn head");
     wellKnownObjs[WELL_KNOWN_OBJ_SLICED_ARM]        = new WellKnownObj(14, "Sliced arm");
     wellKnownObjs[WELL_KNOWN_OBJ_SLICED_LEG]        = new WellKnownObj(15, "Sliced leg");
     wellKnownObjs[WELL_KNOWN_OBJ_GUTS]              = new WellKnownObj(16, "Guts");
     wellKnownObjs[WELL_KNOWN_OBJ_BRAINS]            = new WellKnownObj(17, "Brains");
     wellKnownObjs[WELL_KNOWN_OBJ_MUSHROOMS]         = new WellKnownObj(20, "Mushrooms");
     wellKnownObjs[WELL_KNOWN_OBJ_LIGHT_BALL]        = new WellKnownObj(21, "Light Ball");
     wellKnownObjs[WELL_KNOWN_OBJ_SPRING]            = new WellKnownObj(22, "Spring");
     wellKnownObjs[WELL_KNOWN_OBJ_DISC]              = new WellKnownObj(23, "Disc");
     wellKnownObjs[WELL_KNOWN_OBJ_PORTAL]            = new WellKnownObj(25, "Portal");
     wellKnownObjs[WELL_KNOWN_OBJ_ROSE]              = new WellKnownObj(1001, "Rose");
     wellKnownObjs[WELL_KNOWN_OBJ_WHITE_ROSE]        = new WellKnownObj(1002, "White rose");
     wellKnownObjs[WELL_KNOWN_OBJ_PIT]               = new WellKnownObj(3010, "Pit");
     wellKnownObjs[WELL_KNOWN_OBJ_SCHOOL_MACE]       = new WellKnownObj(3700, "School mace");
     wellKnownObjs[WELL_KNOWN_OBJ_SCHOOL_DAGGER]     = new WellKnownObj(3701, "School dagger");
     wellKnownObjs[WELL_KNOWN_OBJ_SCHOOL_SWORD]      = new WellKnownObj(3702, "School sword");
     wellKnownObjs[WELL_KNOWN_OBJ_SCHOOL_SPEAR]      = new WellKnownObj(3717, "School spear");
     wellKnownObjs[WELL_KNOWN_OBJ_SCHOOL_STAFF]      = new WellKnownObj(3718, "School staff");
     wellKnownObjs[WELL_KNOWN_OBJ_SCHOOL_AXE]        = new WellKnownObj(3719, "School axe");
     wellKnownObjs[WELL_KNOWN_OBJ_SCHOOL_FLAIL]      = new WellKnownObj(3720, "School flail");
     wellKnownObjs[WELL_KNOWN_OBJ_SCHOOL_WHIP]       = new WellKnownObj(3721, "School whip");
     wellKnownObjs[WELL_KNOWN_OBJ_SCHOOL_POLEARM]    = new WellKnownObj(3722, "School polearm");
     wellKnownObjs[WELL_KNOWN_OBJ_SCHOOL_BOW]        = new WellKnownObj(3723, "School bow");
     wellKnownObjs[WELL_KNOWN_OBJ_SCHOOL_LANCE]      = new WellKnownObj(3724, "School lance");
     wellKnownObjs[WELL_KNOWN_OBJ_SCHOOL_VEST]       = new WellKnownObj(3703, "School vest");
     wellKnownObjs[WELL_KNOWN_OBJ_SCHOOL_SHIELD]     = new WellKnownObj(3704, "School shield");
     wellKnownObjs[WELL_KNOWN_OBJ_SCHOOL_BANNER]     = new WellKnownObj(3716, "School banner");
     wellKnownObjs[WELL_KNOWN_OBJ_MAP]               = new WellKnownObj(3162, "Map");
     wellKnownObjs[WELL_KNOWN_OBJ_NMAP1]             = new WellKnownObj(3385, "Newbie map1");
     wellKnownObjs[WELL_KNOWN_OBJ_NMAP2]             = new WellKnownObj(3386, "Newbie map2");
     wellKnownObjs[WELL_KNOWN_OBJ_MAP_NT]            = new WellKnownObj(3167, "Map of New Thalos");
     wellKnownObjs[WELL_KNOWN_OBJ_MAP_OFCOL]         = new WellKnownObj(3162, "Map of Ofcol");
     wellKnownObjs[WELL_KNOWN_OBJ_MAP_SM]            = new WellKnownObj(3164, "Map of Southern Midgaard");
     wellKnownObjs[WELL_KNOWN_OBJ_MAP_TITAN]         = new WellKnownObj(3382, "Map of Titan's valley");
     wellKnownObjs[WELL_KNOWN_OBJ_MAP_OLD]           = new WellKnownObj(5333, "Map of Old midgaard");
     wellKnownObjs[WELL_KNOWN_OBJ_WHISTLE]           = new WellKnownObj(2116, "Map of Whistle");
     wellKnownObjs[WELL_KNOWN_OBJ_POTION_VIAL]       = new WellKnownObj(42, "Potion vial");
     wellKnownObjs[WELL_KNOWN_OBJ_STEAK]             = new WellKnownObj(27, "Steak");
     wellKnownObjs[WELL_KNOWN_OBJ_RANGER_STAFF]      = new WellKnownObj(28, "Ranger staff");
     wellKnownObjs[WELL_KNOWN_OBJ_RANGER_ARROW]      = new WellKnownObj(6, "Ranger arrow");
     wellKnownObjs[WELL_KNOWN_OBJ_RANGER_BOW]        = new WellKnownObj(7, "Ranger bow");
     wellKnownObjs[WELL_KNOWN_OBJ_RULER_BADGE]       = new WellKnownObj(70, "Ruler badge");
     wellKnownObjs[WELL_KNOWN_OBJ_RULER_SHIELD1]     = new WellKnownObj(71, "Ruler shield 1");
     wellKnownObjs[WELL_KNOWN_OBJ_RULER_SHIELD2]     = new WellKnownObj(72, "Ruler shield 2");
     wellKnownObjs[WELL_KNOWN_OBJ_RULER_SHIELD3]     = new WellKnownObj(73, "Ruler shield 3");
     wellKnownObjs[WELL_KNOWN_OBJ_RULER_SHIELD4]     = new WellKnownObj(74, "Ruler shield 4");
     wellKnownObjs[WELL_KNOWN_OBJ_CHAOS_BLADE]       = new WellKnownObj(87, "Chaos blade");
     wellKnownObjs[WELL_KNOWN_OBJ_DRAGONDAGGER]      = new WellKnownObj(80, "Dragon Dagger");
     wellKnownObjs[WELL_KNOWN_OBJ_DRAGONMACE]        = new WellKnownObj(81, "Dragon Mace");
     wellKnownObjs[WELL_KNOWN_OBJ_PLATE]             = new WellKnownObj(82, "Dragon Plate");
     wellKnownObjs[WELL_KNOWN_OBJ_DRAGONSWORD]       = new WellKnownObj(83, "Dragon Sword");
     wellKnownObjs[WELL_KNOWN_OBJ_DRAGONLANCE]       = new WellKnownObj(99, "Dragon Lance");
     wellKnownObjs[WELL_KNOWN_OBJ_PONCHO]            = new WellKnownObj(26, "Battlerager Poncho");
     wellKnownObjs[WELL_KNOWN_OBJ_BATTLE_THRONE]     = new WellKnownObj(542, "Battlerager throne");
     wellKnownObjs[WELL_KNOWN_OBJ_SHALAFI_ALTAR]     = new WellKnownObj(530, "Shalafi altar");
     wellKnownObjs[WELL_KNOWN_OBJ_CHAOS_ALTAR]       = new WellKnownObj(551, "Chaos altar");
     wellKnownObjs[WELL_KNOWN_OBJ_INVADER_SKULL]     = new WellKnownObj(560, "Invader skull");
     wellKnownObjs[WELL_KNOWN_OBJ_KNIGHT_ALTAR]      = new WellKnownObj(521, "Knight altar");
     wellKnownObjs[WELL_KNOWN_OBJ_RULER_STAND]       = new WellKnownObj(510, "Ruler stand");
     wellKnownObjs[WELL_KNOWN_OBJ_LIONS_ALTAR]       = new WellKnownObj(501, "Lions altar");
     wellKnownObjs[WELL_KNOWN_OBJ_HUNTER_ALTAR]      = new WellKnownObj(570, "Hunter altar");
     wellKnownObjs[WELL_KNOWN_OBJ_POTION_SILVER]     = new WellKnownObj(43, "Potion silver");
     wellKnownObjs[WELL_KNOWN_OBJ_POTION_GOLDEN]     = new WellKnownObj(44, "Potion golden");
     wellKnownObjs[WELL_KNOWN_OBJ_POTION_SWIRLING]   = new WellKnownObj(45, "Swirling potion");
     wellKnownObjs[WELL_KNOWN_OBJ_KATANA_SWORD]      = new WellKnownObj(98, "Katana sword");
     wellKnownObjs[WELL_KNOWN_OBJ_EYED_SWORD]        = new WellKnownObj(88, "Eyed sword");
     wellKnownObjs[WELL_KNOWN_OBJ_FIRE_SHIELD]       = new WellKnownObj(92, "Fire shield");
     wellKnownObjs[WELL_KNOWN_OBJ_MAGIC_JAR]         = new WellKnownObj(93, "Magic jar");
     wellKnownObjs[WELL_KNOWN_OBJ_HAMMER]            = new WellKnownObj(6522, "Hammer");
     wellKnownObjs[WELL_KNOWN_OBJ_FINE]              = new WellKnownObj(514, "Fine");
     wellKnownObjs[WELL_KNOWN_OBJ_CHUNK_IRON]        = new WellKnownObj(6521, "Chunk iron");
     wellKnownObjs[WELL_KNOWN_OBJ_TATTOO_APOLLON]    = new WellKnownObj(51, "Tattoo Apollon");
     wellKnownObjs[WELL_KNOWN_OBJ_TATTOO_ZEUS]       = new WellKnownObj(52, "Tattoo Zeus");
     wellKnownObjs[WELL_KNOWN_OBJ_TATTOO_SIEBELE]    = new WellKnownObj(53, "Tattoo Siebele");
     wellKnownObjs[WELL_KNOWN_OBJ_TATTOO_HEPHAESTUS] = new WellKnownObj(54, "Tattoo Hephaestus");
     wellKnownObjs[WELL_KNOWN_OBJ_TATTOO_EHRUMEN]    = new WellKnownObj(55, "Tattoo Ehrumen");
     wellKnownObjs[WELL_KNOWN_OBJ_TATTOO_AHRUMAZDA]  = new WellKnownObj(56, "Tattoo Ahrumazda");
     wellKnownObjs[WELL_KNOWN_OBJ_TATTOO_DEIMOS]     = new WellKnownObj(57, "Tattoo Deimos");
     wellKnownObjs[WELL_KNOWN_OBJ_TATTOO_PHOBOS]     = new WellKnownObj(58, "Tattoo Phobos");
     wellKnownObjs[WELL_KNOWN_OBJ_TATTOO_ODIN]       = new WellKnownObj(59, "Tattoo Odin");
     wellKnownObjs[WELL_KNOWN_OBJ_TATTOO_MARS]       = new WellKnownObj(60, "Tattoo Mars");
     wellKnownObjs[WELL_KNOWN_OBJ_TATTOO_ATHENA]     = new WellKnownObj(61, "Tattoo Athena");
     wellKnownObjs[WELL_KNOWN_OBJ_TATTOO_GOKTENGRI]  = new WellKnownObj(62, "Tattoo Goktengri");
     wellKnownObjs[WELL_KNOWN_OBJ_TATTOO_HERA]       = new WellKnownObj(63, "Tattoo Hera");
     wellKnownObjs[WELL_KNOWN_OBJ_TATTOO_VENUS]      = new WellKnownObj(64, "Tattoo Venus");
     wellKnownObjs[WELL_KNOWN_OBJ_TATTOO_ARES]       = new WellKnownObj(65, "Tattoo Ares");
     wellKnownObjs[WELL_KNOWN_OBJ_TATTOO_PROMETHEUS] = new WellKnownObj(66, "Tattoo Prometheus");
     wellKnownObjs[WELL_KNOWN_OBJ_TATTOO_EROS]       = new WellKnownObj(67, "Tattoo Eros");
  }

  public static int getDamageType(String damage)
  {
    int type = AnatoliaConsts.DAMAGE_TYPE_PIERCE;
    int found = -1;

    type = AnatoliaConsts.DAMAGE_TYPE_PIERCE;
    found = AreaBuilderConsts.findInTab(AnatoliaConsts.damageTexts[type], damage);
    if (found>=0) return type;

    type = AnatoliaConsts.DAMAGE_TYPE_BASH;
    found = AreaBuilderConsts.findInTab(AnatoliaConsts.damageTexts[type], damage);
    if (found>=0) return type;

    type = AnatoliaConsts.DAMAGE_TYPE_SLASH;
    found = AreaBuilderConsts.findInTab(AnatoliaConsts.damageTexts[type], damage);
    if (found>=0) return type;

    type = AnatoliaConsts.DAMAGE_TYPE_MAGIC;
    found = AreaBuilderConsts.findInTab(AnatoliaConsts.damageTexts[type], damage);
    if (found>=0) return type;

    return -1;
  }

  public static String[] getObjectConditionTexts()
  {
     String[] tab = new String[objectConditionTexts.length];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = objectConditionTexts[i].getName();
     return tab;
  }
  public static String[] getObjectConditionCodes()
  {
     String[] tab = new String[objectConditionTexts.length];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = objectConditionTexts[i].getCode();
     return tab;
  }

  public static WellKnownMob[] getWellKnownMobs()
  {
     return wellKnownMobs;
  }
  public static WellKnownObj[] getWellKnownObjs()
  {
     return wellKnownObjs;
  }

  public static String[] getDoorStateTexts()
  {
     String[] tab = new String[doorStateTexts.length];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = doorStateTexts[i].getName();
     return tab;
  }

  public static String[] getDoorSizeTexts()
  {
     String[] tab = new String[doorSizeTexts.length];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = doorSizeTexts[i].getName();
     return tab;
  }
  public static String[] getDoorSizeCodes()
  {
     String[] tab = new String[doorSizeTexts.length];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = doorSizeTexts[i].getCode();
     return tab;
  }
  public static String[] getMobileSizeTexts()
  {
     String[] tab = new String[mobileSizeTexts.length];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = mobileSizeTexts[i].getName();
     return tab;
  }
  public static String[] getMobileSizeCodes()
  {
     String[] tab = new String[mobileSizeTexts.length];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = mobileSizeTexts[i].getCode();
     return tab;
  }

  public static String[] getObjectTypeTexts()
  {
     String[] tab = new String[objectTypeTexts.length];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = objectTypeTexts[i].getName();
     return tab;
  }
  public static String[] getObjectTypeCodes()
  {
     String[] tab = new String[objectTypeTexts.length];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = objectTypeTexts[i].getCode();
     return tab;
  }

  public static String[] getLiquidTexts()
  {
     String[] tab = new String[liquids.length];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = liquids[i].getName();
     return tab;
  }

  public static String[] getRaceTexts()
  {
     String[] tab = new String[mobileRecommendedRaceFlagsValues.length];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = mobileRecommendedRaceFlagsValues[i].getName();
     return tab;
  }

  public static String[] getMobileLevelsTexts()
  {
     String[] tab = new String[LEVEL_MAX-LEVEL_MIN+1];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = Integer.toString(i+LEVEL_MIN);
     return tab;
  }

  public static String[] getObjectBulkTexts()
  {
     String[] tab = new String[objectBulkTexts.length];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = objectBulkTexts[i].getName();
     return tab;
  }

  public static String[] getWearTexts()
  {
     String[] tab = new String[wearTexts.length];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = wearTexts[i].getName();
     return tab;
  }
  public static String[] getWearCodes()
  {
     String[] tab = new String[wearTexts.length];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = wearTexts[i].getCode();
     return tab;
  }

  public static String[] getApplyCodes()
  {
     String[] tab = new String[applyTexts.length];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = applyTexts[i].getCode();
     return tab;
  }

  public byte oppositDirection(byte direction)
  {
    switch(direction)
    {
      case NORTH : return SOUTH;
      case EAST  : return WEST ;
      case SOUTH : return NORTH;
      case WEST  : return EAST ;
      case UP    : return DOWN ;
      case DOWN  : return UP   ;
      default    : return NORTH;
    }
  }

  static public String[] getSpellTexts()
  {
     String[] tab = new String[spellTexts.length];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = spellTexts[i].getName();
     return tab;
  }

  static public String[] getSpellCodes()
  {
     String[] tab = new String[spellTexts.length];
     for(int i=0 ; i<tab.length ; ++i)
        tab[i] = spellTexts[i].getCode();
     return tab;
  }
}